/****************************************************************************

    QLLTSimulation Class
        Copyright (C) 2015 David Marten david.marten@tu-berlin.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/

#ifndef QLLTSIMULATION_H
#define QLLTSIMULATION_H

#include <QObject>
#include "../Objects/CVector.h"
#include "VortexNode.h"
#include "../Graph/ShowAsGraphInterface.h"
#include "../StorableObject.h"
#include "DummyLine.h"

class WindField;
class VortexLine;
class CBlade;
class CPanel;
class CSurface;
class QLLTModule;


class QLLTSimulation : public StorableObject, public ShowAsGraphInterface
{
	Q_OBJECT
	
public:
    QLLTSimulation ();

    QLLTSimulation(QString name,
                   bool bisVawt,
                   WindField *usedWindField,
                   double timeStep,
                   double numTimeStep,
                   double aziDisc,
                   CBlade *usedRotor,
                   double rotorSpeed,
                   double rotorYaw,
                   double rotorTeeter,
                   double yroll,
                   double xroll,
                   double horinflowAngle,
                   double vertinflowSpeed,
                   double turbVisc,
                   double airDens,
                   double kinVisc,
                   double firstWakeRowLength,
                   double minGammaFrac,
                   double fullWakeRevs,
                   double fineWakeRevs,
                   double maxWakeAge,
                   double vortexTimeOffset,
                   double epsilon,
                   double relaxFactor,
                   double maxIter,
                   double thinFactor,
                   double z_leishman,
                   bool bIncludeTower,
                   double towerDrag,
                   double towerTopRadius,
                   double towerBottomRadius,
                   double towerDist,
                   bool bGround,
                   double hubHeight,
                   double towerHeight,
                   bool bWakeRollup,
                   bool bShed,
                   bool bTrail,
                   bool bPrescribed,
                   bool turbConvection,
                   bool storeWake,
                   double inflowSpeed,
                   QList<QList<double> > simParameters,
                   QList<QString> fileHeader,
                   QString fileName,
                   int windInputType,
                   double TStart,
                   int DecFactor,
                   bool isStandStill,
                   double shearCoeff,
                   bool vortexParticle,
                   bool predcorr,
                   int discType,
                   int numPanels,
                   double coneAngle,
                   bool upWind,
                   bool himmelskamp);

    ~QLLTSimulation();
	static QStringList prepareMissingObjectMessage();

    //virtual public functions
    void serialize ();
    void restorePointers ();
    static QLLTSimulation* newBySerialize ();
    NewCurve* newCurve (QString xAxis, QString yAxis, NewGraph::GraphType graphType);  // returns NULL if var not available
    QStringList getAvailableVariables (NewGraph::GraphType graphType);
    QString getObjectName () { return m_objectName; }

    //public functions
    WindField* getUsedWindField () { return m_WindField; }
    CBlade* getUsedBlade () { return m_Blade; }
    bool getIncludeTower () { return m_bIncludeTower; }
    bool getIncludeGround () { return m_bGround; }
    bool getIncludeHimmelskamp () { return m_bHimmelskamp; }
    bool getShed () { return m_bShed; }
    bool getTrail () { return m_bTrail; }
    bool getPrescribed () { return m_bPrescribed; }
    bool getTurbConvection () { return m_bTurbConvection; }
    bool getStoreWake () { return m_bStoreWake; }
    bool getWakeRollup () { return m_bWakeRollUp; }
    double getRunTime () { return dT*double(m_numTimesteps); }
    double getRotorSpeed () { return m_Omega * 60 / 2 / PI; }
    double getYawAngle () { if (m_windInputType != SIMFILE) return m_YawAngle; else return 0; }
    double getTeeterAngle () { return m_TeeterAngle; }
    double getHorInflowAngle () { if (m_windInputType == CONSTANT) return m_HorInflowAngle; else return 0;}
    double getvertinflowSpeed () { if (m_windInputType == CONSTANT) return m_VertInflowSpeed; else return 0;}
    double getTimeStep () { return dT; }
    double getTowerTopRadius () { return m_TowerTopRadius; }
    double getTowerBottomRadius () { return m_TowerBottomRadius; }
    double getTowerDistance () { return m_RotorOverhang; }
    double getTowerDrag () { return m_TowerDrag; }
    double getHubHeight () { return m_HubHeight; }
    double getTowerHeight () { return m_TowerHeight; }
    double getFirstWakeRowLength () { return m_firstWakeRowLength; }
    double getFullWakeRevolutions () { return m_FullWakeRevs; }
    double getFineWakeRevolutions () { return m_FineWakeRevs; }
    double getMaxWakeRevolutions () { return m_MaxWakeAge; }
    double getVortexTimeOffset () { return m_VortexTimeOffset; }
    double getEpsilon () { return m_Epsilon; }
    double getRelaxFactor () { return m_RelaxFactor; }
    double getMaxIterations () { return m_MaxIterations; }
    double getThinFactor () { return m_Thinfactor; }
    double get_z_Leishman () { return m_z_leishman; }
    double getKinVisc () { return m_KinViscosity; }
    double getTurbVisc () { return m_TurbulentViscosity; }
    double getDensity () { return m_Density; }
    double getNumberTimesteps () { return double(m_numTimesteps); }
    double getAziDisc () { return m_AngularDisc; }
    double getIsVawt () { return m_bisVawt; }
    double getYRoll () { return m_PlatformPitchAngleY; }
    double getXRoll () { return m_PlatformRollAngleX; }
    double getTStart () { return m_TStart; }
    double getDecFactor () { return m_DecFactor; }
    double isStandStill () { return m_bisStandStill; }
    void setShownBladeSection (double shownBladeSection) { m_shownBladeSection = shownBladeSection; }
    double getShownBladeSection () { return m_shownBladeSection; }
    void setShownTime(double shownTime);
    double getShownTime () { if(m_shownTimeIndex >= 0) return m_results.at(0)->at(m_shownTimeIndex); else return 0; }
    int getShownTimeIndex () { return m_shownTimeIndex; }
    void setShownTimeIndex (int shownTimeIndex) { m_shownTimeIndex = shownTimeIndex; }
    int getBladeNumber() {return m_numBlades; }
    int getNumTimestepsComputed () { return m_results.at(0)->size(); }
    int getWindInputType () { return m_windInputType; }
    double getInflowSpeed () { return m_InflowSpeed; }
    QList< QList<double> > getSimParameters () { return m_SimParameters; }
    QList< QString > getFileHeader () { return m_FileHeader; }
    QString getFileName () { return m_FileName; }
    bool isRunning() { return m_bisRunning ;}
    bool hasStoredData () { return !m_BladeOutputData.isEmpty(); }  // NM TODO this is wrong! Simulation is not certainly finished.
    QVector< QVector <double> *> getResultsArray () { return m_results; }
    void StopRequested(){ m_bStopRequested = true; }
    void drawGL();
    void callGLLists();
    void drawOverpaint(QPainter &painter);
    double getShearCoeff() { if (m_windInputType == CONSTANT) return m_ShearCoeff; else return 0; }
    void setCanContinue(bool canContinue) { m_bContinue = canContinue; }
    bool getCanContinue() { return m_bContinue; }
    int getCurrentTimeStep() { return m_currentTimeStep; }
    double getGammaFrac() { return m_MinGammaFrac; }
    bool getisVortexParticle() { return m_bVortexParticle; }
    bool getIntegrationScheme() { return m_bPredictorCorrector; }
    int getNumPanels() { return m_numPanels; }
    int getDiscType() { return m_discType; }
    double getConeAngle() { return m_ConeAngle; }
    bool getisUpWind() { return m_bUpWind; }

    void setForceReRender() {m_bforceReRender = true; }

    QList<QList <CVector> > m_savedCoordinateSystems;

private:

    //private variables
    bool m_bforceReRender;
    bool m_bAllConverged;
    bool m_bGlChanged;
    bool m_bGLPlotWakeNodes;
    bool m_bGLPlotWakeLinesShed;
    bool m_bGLPlotWakeLinesTrail;
    bool m_bGLPlotSingleBlade;
    bool m_bGLPlotRotor;
    bool m_bGLPlotTurbineModel;
    bool m_bGLPlotCoordinates;
    bool m_bGLColorNodes;
    bool m_bGLColorLines;
    bool m_bStopRequested;
    bool m_bisRunning;
    bool m_bContinue;
    bool m_bStopReplay;

    double m_PitchOld;
    double m_PlatformPitchAngleYOld;
    double m_PlatformRollAngleXOld;
    double m_PlatformYawAngleZOld;
    double m_TeeterOld;
    double m_YawOld;
    double m_HorShear, m_VertShear, m_LinVertShear, m_GustSpeed; //aerodyn wnd file params

    CVector V_in;

    CVector VFoot, VHub, VAxis, VTop;
    QList<CVector> VBOrigins, VBAxes;

    QList<CVector> m_BladeOrigins;
    QList<CVector> m_BladeAxes;

    CVector m_PlatformTranslationOld;
    QVector <CVector> m_storeWakeInfluence;
    QList <QList <VortexNode *> > TempConnectionNode;
    QList <CPanel *> m_BladePanel;
    QList <VortexLine *> m_BladeLine;
    QList <VortexLine *> m_WakeLine;
    QList <VortexNode *> m_BladeNode;
    QList <VortexNode *> m_WakeNode;
    QList <double > m_WakeParticles;

    QVector< QVector <double> *> m_results;
    QStringList m_availableVariables;
    QStringList m_availableBladeVariables;

    //private variables & arrays to serialize

    QLLTModule *m_module;
    WindField *m_WindField;
    CBlade *m_Blade;
	

    QList<QList <DummyLine> > m_savedWakeLines;
    QList<QList <DummyLine> > m_savedBladeVizLines;
    QList<QList <DummyLine> > m_savedBladeVortexLines;
    QList<QList <double> > m_savedWakeVortexParticles;

    QList<QString> m_FileHeader;

    QList< QList<double> > m_SimParameters;

    CVector m_RotorAxis, m_RotorHub, m_TowerFoot, m_TowerTop;

    int m_numBlades;
    int m_shownTimeIndex;
    int m_DecFactor;
    int m_numTimesteps;
    int iterations;
    int m_MaxIterations;
    int m_currentTimeStep;
    int m_InterpolationFactor;
    int m_Thinfactor;
    int m_windInputType;
    int m_nWakeColumn;
    int m_numPanels;
    int m_discType;

    int m_plotAtTime;

    double m_ConeAngle;
    double m_shownBladeSection;
    double m_TowerDrag, m_TowerTopRadius, m_TowerBottomRadius, m_RotorOverhang;
    double m_TStart;
    double m_currentTime;
    double m_z_leishman;
    double m_MinGammaFrac;
    double m_Omega;
    double m_Epsilon;
    double dT;
    CVector m_PlatformTranslation;
    double m_YawAngle;
    double m_TeeterAngle;
    double m_PlatformPitchAngleY;
    double m_PlatformRollAngleX;
    double m_PlatformYawAngleZ;
    double m_HorInflowAngle;
    double m_VertInflowSpeed;
    double m_PitchAngle;
    double m_InflowSpeed;
    double m_maxGamma;
    double m_MaxWakeAge;
    double m_AngularDisc;
    double m_FineWakeRevs;
    double m_FullWakeRevs;
    double TSR;
    double m_firstWakeRowLength;
    double m_RelaxFactor;
    double m_Density;
    double m_HubHeight;
    double m_TowerHeight;
    double m_KinViscosity;
    double m_TurbulentViscosity;
    double m_VortexTimeOffset;
    double m_ShearCoeff;

    bool m_bShed;
    bool m_bGround;
    bool m_bHimmelskamp;
    bool m_bTrail;
    bool m_bisStandStill;
    bool m_bIncludeTower;
    bool m_bPrescribed;
    bool m_bTurbConvection;
    bool m_bStoreWake;
    bool m_bWakeRollUp;
    bool m_bisVawt;
    bool m_bVortexParticle;
    bool m_bPredictorCorrector;
    bool m_bUpWind;

    QString m_FileName;

    //simulation results //

    QVector<QVector<QVector<double> > > m_BladeOutputData;

    //HAWT / VAWT DATA
    QVector <double> m_CP;
    QVector <double> m_CPAV;
    QVector <double> m_CT;
    QVector <double> m_CM;
    QVector <double> m_TSR;
    QVector <double> m_Power;
    QVector <double> m_Torque;
    QVector <double> m_Time;
    QVector <double> m_angPos;
    QVector <double> m_rotSpeed;
    QVector <double> m_ThrustX;
    QVector <double> m_ThrustY;
    QVector <double> m_ThrustZ;
    QVector <double> m_CurrentHHWindSpeed;
    QVector <double> m_CurrentPitchAngle;
    QVector <double> m_CurrentYawAngle;
    QVector <double> m_CurrentHorizontalInflowAngle;
    QVector <double> m_CurrentVerticalInflowSpeed;
    QVector <double> m_Iterations;
    QVector <double> m_CurrentPlatformYaw;
    QVector <double> m_CurrentPlatformRoll;
    QVector <double> m_CurrentPlatformPitch;
    QVector <double> m_CurrentPlatformTranslationX;
    QVector <double> m_CurrentPlatformTranslationY;
    QVector <double> m_CurrentPlatformTranslationZ;



    //HAWT ONLY
    QVector <double> m_RootBendingMomentOOP;
    QVector <double> m_RootBendingMomentIP;


    //PRIVATE FUNCTIONS

    double BladeOutputAtSection(QVector<double> output, double section);
    void ClearVortexArrays();
    void PrepareOutputVectors();
    VortexNode* IsNode(CVector &Pt);
    void createHAWTTurbineGeometry();
    void createVAWTTurbineGeometry();
    void CreateElements(CVector m_LA, CVector m_LB, CVector m_TA, CVector m_TB, int blade);
    void InitializePanels();
    void GuessInitialVortexDistribution();
    VortexNode *IsWakeNode(CVector &Pt);
    void GammaBoundFixedPointIteration();
    void CalcBladePanelVelocities();
    void KuttaCondition();
    void WakeConcentration();
    void WakeCutOff();
    void ConvectWakeNodes();
    void InterpolateWakeSpeeds(QList <VortexNode > tempNodes);
    void CalcAoA();
    void drawText(QPainter &painter);
    void drawLegend(QPainter &painter, bool redblue, int component);
    CVector getRelativeBladeVelocity(int ID);
    CVector CorrespondingAxisPoint( CVector Point, CVector Line1, CVector Line2);
    CVector BiotSavartInduction(CVector EvalPt, bool prescribed = false, bool indWing = false, int tempID = 0);
    CVector BiotSavartInductionFromSave(CVector EvalPt, int timeStep);
    CVector BiotSavartKernel(CVector R1, CVector R2, double Gamma, double coreSizeSquared);
    void ReadPanelLiftCoef();
    void CalcNewGamma();
    void PitchBlades();
    void UpdateBladePanels();
    void UpdateRotorGeometry();
    void InitializeBladePanelPositions();
    void RotateHAWTOriginalPos();
    void RotateVAWTOriginalPos();
    void UpdateHAWTCoordinates();
    void UpdateVAWTCoordinates();
    void RotateRotor();
    void InitRotorCoordinates();
    void ClearOutputArrays();
    void GammaConvergenceCheck();
    void AddFirstWakeLines();
    void AddNewWakeLines();
    void AssignGammaToWingLines();
    CVector getFreeStream(CVector EvalPt, double time = -1);
    CVector getMeanFreeStream(CVector EvalPt);
    void storeOutput();
    void storeGeometry();
    void GLDrawSimulationScene(bool replay = false);
    void SaveRotorCoords(bool replay);
    void DrawCoordinateSystems();
    void CalcVAWTResults();
    void CalcHAWTResults();
    CVector CalcTowerInfluence (CVector EvalPt, CVector V_ref, int timestep = -1);
    void ResetSimulation();
    void UpdateSimulationParameters(double time);

public slots:
    void onStartAnalysis(void);
    void onStartReplay();
    void onStopReplay() { m_bStopReplay = true; }

    void onComputeCutPlane(QLLTCutPlane *plane, int timestep);
    void lockStores();
    void unlockStores();

signals:
    void geomChanged();
    void updateProgress(int i);
    void updateGraphs();
};

#endif // QLLTSIMULATION_H
