/****************************************************************************

    QLLTSimulation Class
        Copyright (C) 2015 David Marten david.marten@tu-berlin.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/

#include "QLLTSimulation.h"
#include "../GLWidget.h"
#include <QDebug>
#include <QtOpenGL>
#include "../GlobalFunctions.h"
#include "../MainFrame.h"
#include "../Store.h"
#include "../Graph/NewCurve.h"
#include "../XBEM/Blade.h"
#include "../XUnsteadyBEM/WindField.h"
#include "QLLTModule.h"
#include "QLLTDock.h"
#include "VortexLine.h"
#include "../Serializer.h"
#include "QLLTToolBar.h"



QLLTSimulation::QLLTSimulation()
    : StorableObject ("< no name >"),
      ShowAsGraphInterface (false)
{
    m_module = g_QLLTModule;
    m_shownBladeSection = 1;
    m_shownTimeIndex = 0;
    m_bStopRequested = false;
    m_bGlChanged = true;
    m_bisRunning = false;    
    m_bContinue = false;
    V_in.Set(0,0,0);
    m_maxGamma = 0;
    m_bforceReRender = false;
    m_plotAtTime = -1;
    m_currentTimeStep= -1;
}

QLLTSimulation::QLLTSimulation(QString name,
                               bool bisVawt,
                               WindField *usedWindField,
                               double timeStep,
                               double numTimeStep,
                               double aziDisc,
                               CBlade *usedRotor,
                               double rotorSpeed,
                               double rotorYaw,
                               double rotorTeeter,
                               double yroll,
                               double xroll,
                               double horinflowAngle,
                               double vertinflowSpeed,
                               double turbVisc,
                               double airDens,
                               double kinVisc,
                               double firstWakeRowLength,
                               double minGammaFrac,
                               double fullWakeRevs,
                               double fineWakeRevs,
                               double maxWakeAge,
                               double vortexTimeOffset,
                               double epsilon,
                               double relaxFactor,
                               double maxIter,
                               double thinFactor,
                               double z_leishman,
                               bool bIncludeTower,
                               double towerDrag,
                               double towerTopRadius,
                               double towerBottomRadius,
                               double towerDist,
                               bool bGround,
                               double hubHeight,
                               double towerHeight,
                               bool bWakeRollup,
                               bool bShed,
                               bool bTrail,
                               bool bPrescribed,
                               bool turbConvection,
                               bool storeWake,
                               double inflowSpeed,
                               QList< QList<double> > simParameters,
                               QList<QString> fileHeader,
                               QString fileName,
                               int windInputType,
                               double TStart,
                               int DecFactor,
                               bool isStandStill,
                               double shearCoeff,
                               bool vortexParticle,
                               bool predcorr,
                               int discType,
                               int numPanels,
                               double coneAngle,
                               bool upWind,
                               bool himmelskamp)
    : StorableObject (name)
{
    if (usedWindField) addParent(usedWindField);
    addParent(usedRotor);

    m_maxGamma = 0;
    m_bisVawt = bisVawt;
    m_module = g_QLLTModule;
    m_bContinue = false;
    V_in.Set(0,0,0);
    m_discType = discType;
    m_numPanels = numPanels;
    m_ConeAngle = coneAngle;
    m_bUpWind = upWind;
    m_bHimmelskamp = himmelskamp;
    m_plotAtTime = -1;
    m_currentTimeStep = -1;

    m_FileHeader = fileHeader;

    if (m_bisVawt) m_pen.setColor(findColor (&g_QLLTVAWTSimulationStore));
    else m_pen.setColor(findColor (&g_QLLTHAWTSimulationStore));


    m_Blade = usedRotor;
    m_WindField = usedWindField;
    m_bVortexParticle = vortexParticle;
    m_bPredictorCorrector = predcorr;

    m_windInputType = windInputType;
    m_SimParameters = simParameters;
    m_InflowSpeed = inflowSpeed;
    m_bGlChanged = true;
    m_FileName = fileName;
    m_bisStandStill = isStandStill;
    m_bisRunning = false;
    m_pen.setWidth(1);
    m_pen.setStyle(Qt::SolidLine);
    m_shownBladeSection = 1;
    m_shownTimeIndex = 0;
    m_currentTimeStep = -1;
    m_numBlades = m_Blade->m_blades;
    m_ShearCoeff = shearCoeff;

    /// Main Settings for Op-Point

    m_YawAngle = rotorYaw;
    m_TeeterAngle = rotorTeeter;
    m_PlatformRollAngleX = xroll;
    m_PlatformPitchAngleY = yroll;
    m_PlatformYawAngleZ = 0;
    m_HorInflowAngle = horinflowAngle;
    m_VertInflowSpeed = vertinflowSpeed;
    m_PitchAngle = 0;
    m_PlatformTranslation.Set(0,0,0);
    dT = timeStep;

    m_numTimesteps = numTimeStep;
    m_currentTime = 0;
    m_AngularDisc = aziDisc;
    m_bTurbConvection = turbConvection;
    m_bStoreWake = storeWake;
    m_TStart = TStart;
    m_DecFactor = DecFactor;

    /// Additional Modells / Tower Shadow / Prescribed Wake / Goundeffects

    m_bIncludeTower = bIncludeTower;
    m_HubHeight = hubHeight;
    if (m_bisVawt) m_TowerHeight = towerHeight;
    if (!m_bisVawt) m_TowerHeight = hubHeight;


    m_bGround = bGround;
    m_bPrescribed = bPrescribed;

    /// Algorithm Parameters

    m_firstWakeRowLength = firstWakeRowLength;
    m_FullWakeRevs = fullWakeRevs;
    m_FineWakeRevs = fineWakeRevs;
    m_MaxWakeAge = maxWakeAge;
    m_VortexTimeOffset = vortexTimeOffset;
    m_Epsilon = epsilon;
    m_RelaxFactor = relaxFactor;
    m_MaxIterations = maxIter;
    m_InterpolationFactor = 1; // Wake Interpolation
    m_Thinfactor = thinFactor;
    m_MinGammaFrac = minGammaFrac;
    m_z_leishman = z_leishman;
    m_bShed  = bShed;
    m_bTrail = bTrail;
    m_bWakeRollUp = bWakeRollup;

    m_TowerDrag = towerDrag;
    m_TowerTopRadius = towerTopRadius;
    m_TowerBottomRadius = towerBottomRadius;
    m_RotorOverhang = towerDist;


    ///Environmental Variables
    m_KinViscosity = kinVisc;
    m_TurbulentViscosity = turbVisc;
    m_Density = airDens;

    m_Omega = rotorSpeed / 60.0 * 2 * PI;

    if (!m_bisVawt) createHAWTTurbineGeometry();
    else createVAWTTurbineGeometry();

    PrepareOutputVectors();
}

QLLTSimulation::~QLLTSimulation() {
        ClearVortexArrays();
		ClearOutputArrays();
}

QStringList QLLTSimulation::prepareMissingObjectMessage() {
	if (g_QLLTModule->m_bisVAWT && g_QLLTVAWTSimulationStore.isEmpty()) {
		QStringList message1 = WindField::prepareMissingObjectMessage();
		QStringList message2 = CBlade::prepareMissingObjectMessage(true);
		if (!message1.isEmpty() && !message2.isEmpty()) {
			message1.removeLast();  // remove the "create WindField" hint
		}
		QStringList message = message1 + message2;
		if (message.isEmpty()) {
			if (g_mainFrame->m_iApp == QLLTMODULE) {
				message = QStringList(">>> Click 'New' to create a new LLT VAWT Simulation");
			} else {
				message = QStringList(">>> unknown hint");
			}
		}
		message.prepend("- No LLT VAWT Simulation in Database");
		return message;
	} else if (!g_QLLTModule->m_bisVAWT && g_QLLTHAWTSimulationStore.isEmpty()) {
			QStringList message1 = WindField::prepareMissingObjectMessage();
			QStringList message2 = CBlade::prepareMissingObjectMessage(false);
			if (!message1.isEmpty() && !message2.isEmpty()) {
				message1.removeLast();  // remove the "create WindField" hint
			}
			QStringList message = message1 + message2;
			if (message.isEmpty()) {
				if (g_mainFrame->m_iApp == QLLTMODULE) {
					message = QStringList(">>> Click 'New' to create a new LLT HAWT Simulation");
				} else {
					message = QStringList(">>> unknown hint");
				}
			}
			message.prepend("- No LLT HAWT Simulation in Database");
			return message;
	} else {
		return QStringList();
	}
}

void QLLTSimulation::drawLegend(QPainter &painter, bool redblue, int component){
	const int barWidth = 25;
	const int barHeight = 100;
	const int marginLeft = 15;
	const int marginTop = 55;
	
	QLinearGradient gradient (QPointF(0, marginTop), QPointF(0, marginTop+barHeight));
	if (redblue) {
		for (int i = 0; i < 30; ++i) {
			double vel = 2-2.0/29*i;
			QColor color;
			if (vel > 1) color.setRgb(255,0,0,255*(vel-1));
			else color.setRgb(0,0,255,255*(1-vel));
			gradient.setColorAt(1.0/29*i, color);
		}
	} else {
		for (int i = 0; i < 30; ++i) {
			QColor color;
			color.setHsv(225.0/29*i, 255, 255);
			gradient.setColorAt(1.0/29*i, color);
		}
	}
	
	painter.setPen(QPen(QBrush("black"), 1));
	painter.setBrush(gradient);
	painter.drawRect(marginLeft, marginTop, barWidth, barHeight);
	
    CVector meanSpeed = getMeanFreeStream(CVector(-m_RotorOverhang,0,m_TowerHeight));
    double meanWindspeed = meanSpeed.VAbs();

    if (m_windInputType==SIMFILE || m_windInputType == AERODYN){
        meanWindspeed = 0;
        int speedpos;
        if (m_bisVawt) speedpos = 13;
        else speedpos = 12;
        for (int i = 0; i < m_results.at(speedpos)->size(); ++i) {
            meanWindspeed += m_results.at(speedpos)->at(i) / m_results.at(speedpos)->size();
		}
    }

    CVector curWindSpeed;
    double time;

    if (m_WindField){
        if (m_WindField->getIncludeShear()){
        curWindSpeed.Set(m_WindField->getMeanWindSpeedAtHub()*log(CVector(-m_RotorOverhang,0,m_HubHeight).z/m_WindField->getRoughnessLength())/log(m_WindField->getWindSpeedMeasurementHeight()/m_WindField->getRoughnessLength()),0,0);
        }
        else curWindSpeed.Set(m_WindField->getMeanWindSpeedAtHub(),0,0);
        curWindSpeed.RotZ(m_HorInflowAngle/180 * PI);
    }
    else{
        if (m_bStoreWake) time = m_results.at(0)->at(m_shownTimeIndex);
        else time = m_results.at(0)->at(m_results.at(0)->size()-1);
        curWindSpeed = getFreeStream(CVector(-m_RotorOverhang,0,m_HubHeight),time);
    }


    double max, min, mean;
    QString strong;

    double range = m_module->GetQLLTDock()->GetRange();
    const double fac =  1 + 10*(1-range);


    if (component == 0){
        max = curWindSpeed.VAbs()*(1/fac+1);
        mean = curWindSpeed.VAbs();
        min = (1-1/fac)*curWindSpeed.VAbs();
        strong = "magnitude: ";
    }
    else if (component == 1){
        max = curWindSpeed.x + meanWindspeed*range;
        mean = curWindSpeed.x;
        min = curWindSpeed.x - meanWindspeed*range;
        strong = "x-component: ";
    }
    else if (component == 2){
        max = curWindSpeed.y + meanWindspeed*range;
        mean = curWindSpeed.y;
        min = curWindSpeed.y - meanWindspeed*range;
        strong = "y-component: ";
    }
    else if (component == 3){
        max = curWindSpeed.z + meanWindspeed*range;
        mean = curWindSpeed.z;
        min = curWindSpeed.z - meanWindspeed*range;
        strong = "z-component: ";
    }
    else if (component == 4){
        max = curWindSpeed.x + meanWindspeed*range;
        mean = curWindSpeed.x;
        min = curWindSpeed.x - meanWindspeed*range;
        strong = "axial component: ";
    }
    else if (component == 5){
        max = curWindSpeed.y + meanWindspeed*range;
        mean = curWindSpeed.y;
        min = curWindSpeed.y - meanWindspeed*range;
        strong = "tang. component: ";
    }
    else if (component == 6){
        max = curWindSpeed.z + meanWindspeed*range;
        mean = curWindSpeed.z;
        min = curWindSpeed.z - meanWindspeed*range;
        strong = "radial component: ";
    }

	
	painter.setFont(QFont(g_mainFrame->m_TextFont.family(), 12));
	painter.drawText(marginLeft+barWidth+5, marginTop+6,
                     QString(strong+"%1 m/s").arg(max, 0, 'f', 2));
	painter.drawText(marginLeft+barWidth+5, marginTop+barHeight/2+6,
                     QString(strong+"%1 m/s").arg(mean, 0, 'f', 2));
	painter.drawText(marginLeft+barWidth+5, marginTop+barHeight+6,
                     QString(strong+"%1 m/s").arg(min, 0, 'f', 2));
}

void QLLTSimulation::drawText(QPainter &painter) {
    const double width = m_module->m_glWidget->width();
    const double height = m_module->m_glWidget->height();

	painter.setPen(g_mainFrame->m_TextColor);

    if (width > 300) {
		painter.setFont(QFont(g_mainFrame->m_TextFont.family(), 15));
		painter.drawText(10, 25, QString(getName() + " : " + m_Blade->getName()));
	
		painter.setFont(QFont(g_mainFrame->m_TextFont.family(), 10));
        if (hasStoredData()) {
            painter.drawText(width-220, 40, QString("Time: %1 s").arg(m_bStoreWake ?
																		  m_results.at(0)->at(m_shownTimeIndex) :
																		  m_results.at(0)->at(m_results.at(0)->size()-1)));
            painter.drawText(width-220, 60, QString("Power: %1 kW").arg(m_bStoreWake ?
																			m_results.at(7)->at(m_shownTimeIndex) :
																			m_results.at(7)->at(m_results.at(0)->size()-1)));
            if (m_bisVawt){
            painter.drawText(width-220, 80, QString("Averaged Cp: %1").arg(m_bStoreWake ?
                                                                      m_results.at(5)->at(m_shownTimeIndex) :
                                                                      m_results.at(5)->at(m_results.at(4)->size()-1)));
            }
            else{
            painter.drawText(width-220, 80, QString("Cp: %1").arg(m_bStoreWake ?
                                                                      m_results.at(4)->at(m_shownTimeIndex) :
                                                                      m_results.at(4)->at(m_results.at(4)->size()-1)));
            }
            if (m_bisVawt){
            painter.drawText(width-220, 100, QString("V_in @ hub: %1 m/s").arg(m_bStoreWake ?
                                                                            m_results.at(13)->at(m_shownTimeIndex) :
                                                                            m_results.at(13)->at(m_results.at(4)->size()-1)));
            }
            else{
            painter.drawText(width-220, 100, QString("V_in @ hub: %1 m/s").arg(m_bStoreWake ?
                                                                            m_results.at(12)->at(m_shownTimeIndex) :
                                                                            m_results.at(12)->at(m_results.at(4)->size()-1)));

            }
			if (!m_bStoreWake || m_savedWakeLines.size()) {
                painter.drawText(width-220, height-40, QString("Vortex Elements: %1").arg(m_bStoreWake ?
																				   m_savedWakeLines.at(m_shownTimeIndex).size() :
																				   m_WakeLine.size()));
			}
		}
    }
}

void QLLTSimulation::drawGL() {

    qDebug() << "render--------------"<< m_currentTimeStep;

    if (g_mainFrame->getCurrentModule() == m_module && m_module->GLView) {

        m_plotAtTime = -1;
        if (m_bStoreWake && hasStoredData()){
            if (m_shownTimeIndex != -1) m_plotAtTime = m_shownTimeIndex;
            else m_plotAtTime = m_savedBladeVizLines.size()-1;
        }

        if (m_bGlChanged || m_bforceReRender) {

            if (m_module->m_QLLTDock->m_perspectiveView->isChecked()) m_module->getGLWidget()->camera()->setType(qglviewer::Camera::PERSPECTIVE);
            else m_module->getGLWidget()->camera()->setType(qglviewer::Camera::ORTHOGRAPHIC);

            m_bGLPlotWakeLinesShed = m_module->m_QLLTDock->m_shed->isChecked();
            m_bGLPlotWakeLinesTrail = m_module->m_QLLTDock->m_trail->isChecked();
            m_bGLPlotWakeNodes = m_module->m_QLLTDock->m_nodes->isChecked();
            m_bGLPlotSingleBlade = m_module->m_QLLTDock->m_singleblade->isChecked();
            m_bGLPlotRotor = m_module->m_QLLTDock->m_showRotor->isChecked();
            m_bGLPlotTurbineModel = m_module->m_QLLTDock->m_showModel->isChecked();
            m_bGLPlotCoordinates = m_module->m_QLLTDock->m_showCoordinates->isChecked();
            m_bGLColorLines = m_module->m_QLLTDock->m_colorLines->isChecked();
            m_bGLColorNodes = m_module->m_QLLTDock->m_colorNodes->isChecked();


            if (glIsList(GLWINDFIELD)) glDeleteLists(GLWINDFIELD,1);
            if (glIsList(GLWINGWAKEPANELS)) glDeleteLists(GLWINGWAKEPANELS, 1);
            if (glIsList(GLBLADEOUTLINES)) glDeleteLists(GLBLADEOUTLINES, 1);
            if (glIsList(GLBODYMESHPANELS))  glDeleteLists(GLBODYMESHPANELS, 1);
            if (glIsList(GLTURBINECOORDINATES))  glDeleteLists(GLTURBINECOORDINATES, 1);

            m_bGlChanged = false;
            if (m_WindField && m_module->m_QLLTDock->m_showWindfield->isChecked()) {
                if (m_bStoreWake && hasStoredData()){
                    m_WindField->renderForQLLTSim(m_results.at(0)->at(m_plotAtTime) - dT/2, -m_WindField->getFieldDimensions(),getMeanFreeStream(CVector(-m_RotorOverhang, 0, m_HubHeight)).VAbs(),m_module->m_QLLTDock->m_colorMapButton->button(0)->isChecked());
                }
                else{
                    m_WindField->renderForQLLTSim(m_currentTime - dT/2, -m_WindField->getFieldDimensions(),getMeanFreeStream(CVector(-m_RotorOverhang, 0, m_HubHeight)).VAbs(),m_module->m_QLLTDock->m_colorMapButton->button(0)->isChecked());
                }
            }
            GLDrawSimulationScene(m_bStoreWake && hasStoredData() && !isRunning());
            SaveRotorCoords(m_bStoreWake && hasStoredData() && !isRunning());
		}
    }

    m_bforceReRender = false;
}

void QLLTSimulation::SaveRotorCoords(bool replay){
    int m;
    if (replay && m_shownTimeIndex != -1) m = m_shownTimeIndex;
    else m = m_savedBladeVizLines.size()-1;

    if (VBAxes.size()*2+5 < m_savedCoordinateSystems.at(m).size()){
        for (int i=5;i<m_savedCoordinateSystems.at(m).size();i++){
        VBAxes.append(CVector(0,0,0));
        i++;
        VBOrigins.append(CVector(0,0,0));
        }
    }

    if (m==-1) return;

    VAxis    = m_savedCoordinateSystems.at(m).at(0);
    VHub     = m_savedCoordinateSystems.at(m).at(1);
    VFoot    = m_savedCoordinateSystems.at(m).at(2);
    VTop     = m_savedCoordinateSystems.at(m).at(3);
    int k=0;
    for (int i=5;i<m_savedCoordinateSystems.at(m).size();i++){
        VBAxes[k]=(m_savedCoordinateSystems.at(m).at(i));
        i++;
        VBOrigins[k]=(m_savedCoordinateSystems.at(m).at(i));
        k++;
    }
}

void QLLTSimulation::callGLLists(){

    if (g_mainFrame->getCurrentModule() == m_module && m_module->GLView) {

        double HorInfAngle = m_HorInflowAngle;
        if (m_bStoreWake && hasStoredData() && m_plotAtTime != -1){
            HorInfAngle = m_CurrentHorizontalInflowAngle.at(m_plotAtTime);}

        glEnable(GL_DEPTH_TEST);
        glLineWidth(1.0);
        glCallList(GLTURBINECOORDINATES);
        glDisable(GL_DEPTH_TEST);
        glCallList(GLWINGWAKEPANELS);
        glEnable(GL_DEPTH_TEST);
        glCallList(GLBODYMESHPANELS);
        glDisable(GL_DEPTH_TEST);
        glCallList(GLBLADEOUTLINES);
        glEnable(GL_DEPTH_TEST);
        glRotated(HorInfAngle,0,0,1);
        glCallList(GLWINDFIELD);
        glRotated(-HorInfAngle,0,0,1);
        glPolygonOffset(1.0, 0);
        if (m_bGLPlotCoordinates) DrawCoordinateSystems();
    }
}

void QLLTSimulation::drawOverpaint(QPainter &painter) {
	if (g_mainFrame->getCurrentModule() == m_module && m_module->GLView) {
        if (m_module->GetQLLTDock()->m_showText->isChecked()){
            drawText(painter);
        };
		if (m_module->IsCutPlaneRendered() || (m_WindField && m_module->m_QLLTDock->m_showWindfield->isChecked())) {
            drawLegend(painter, m_module->m_QLLTDock->m_colorMapButton->button(0)->isChecked(), m_module->GetQLLTDock()->m_componentButton->checkedId());
        }
	}
}

void QLLTSimulation::GLDrawSimulationScene(bool replay){

    int m;

	if (replay && m_shownTimeIndex != -1) {
		m = m_shownTimeIndex;
    } else {
		m = m_savedBladeVizLines.size()-1;
	}

    int bladesize = 0;

    if (m != -1) bladesize = m_savedBladeVizLines.at(m).size()/m_numBlades;

    hsv hs;
    hs.s = 1;
    hs.v = 1;

    glDisable(GL_DEPTH_TEST);

    glNewList(GLWINGWAKEPANELS, GL_COMPILE);
    glEnable(GL_LINE_SMOOTH);
    glEnable(GL_POINT_SMOOTH);
    //Panele einfärben

    ///debug vectors
//    for (int i=0; i<m_BladePanel.size();i++){
//        glBegin(GL_LINES);
//        {
//            glColor4d(0,1,0,1);

//        glVertex3d(m_BladePanel[i]->CtrlPt.x, m_BladePanel[i]->CtrlPt.y, m_BladePanel[i]->CtrlPt.z);
//        glVertex3d(m_BladePanel[i]->CtrlPt.x+m_BladePanel[i]->LiftVector.x, m_BladePanel[i]->CtrlPt.y+m_BladePanel[i]->LiftVector.y, m_BladePanel[i]->CtrlPt.z+m_BladePanel[i]->LiftVector.z);
//        glColor4d(0,0,1,1);

//        glVertex3d(m_BladePanel[i]->CtrlPt.x, m_BladePanel[i]->CtrlPt.y, m_BladePanel[i]->CtrlPt.z);
//        glVertex3d(m_BladePanel[i]->CtrlPt.x+m_BladePanel[i]->DragVector.x*10, m_BladePanel[i]->CtrlPt.y+m_BladePanel[i]->DragVector.y*10, m_BladePanel[i]->CtrlPt.z+m_BladePanel[i]->DragVector.z*10);

////        glColor4d(1,0,0,1);

////        glVertex3d(m_BladePanel[i]->CtrlPt.x, m_BladePanel[i]->CtrlPt.y, m_BladePanel[i]->CtrlPt.z);
////        glVertex3d(m_BladePanel[i]->CtrlPt.x-m_BladePanel[i]->m_V_total.x, m_BladePanel[i]->CtrlPt.y-m_BladePanel[i]->m_V_total.y, m_BladePanel[i]->CtrlPt.z-m_BladePanel[i]->m_V_total.z);
////        glColor4d(0,0,0,1);
////        glVertex3d(m_BladePanel[i]->CtrlPt.x, m_BladePanel[i]->CtrlPt.y, m_BladePanel[i]->CtrlPt.z);
////        glVertex3d(m_BladePanel[i]->CtrlPt.x-m_BladePanel[i]->tangentialVector.x, m_BladePanel[i]->CtrlPt.y-m_BladePanel[i]->tangentialVector.y, m_BladePanel[i]->CtrlPt.z-m_BladePanel[i]->tangentialVector.z);
////        glEnd();

//        glColor4d(0,0,0,1);
//        glVertex3d(m_BladePanel[i]->CtrlPt.x, m_BladePanel[i]->CtrlPt.y, m_BladePanel[i]->CtrlPt.z);
//        glVertex3d(m_BladePanel[i]->CtrlPt.x-m_BladePanel[i]->a1.x*10, m_BladePanel[i]->CtrlPt.y-m_BladePanel[i]->a1.y*10, m_BladePanel[i]->CtrlPt.z-m_BladePanel[i]->a1.z*10);
//        glEnd();

//        glColor4d(1,0,0,1);
//        glVertex3d(m_BladePanel[i]->CtrlPt.x, m_BladePanel[i]->CtrlPt.y, m_BladePanel[i]->CtrlPt.z);
//        glVertex3d(m_BladePanel[i]->CtrlPt.x-m_BladePanel[i]->a2.x*10, m_BladePanel[i]->CtrlPt.y-m_BladePanel[i]->a2.y*10, m_BladePanel[i]->CtrlPt.z-m_BladePanel[i]->a2.z*10);
//        glEnd();
//        }

//    }


    glPolygonOffset(1.0, 1.0);
    glLineWidth(0.5);
        double normalized;
        if (m_savedWakeLines.size()){
            for (int p=0; p<m_savedWakeLines.at(m).size(); p++){

                    if (m_savedWakeLines.at(m).at(p).fromStation < m_savedBladeVizLines.at(m).size()/m_numBlades || !m_bGLPlotSingleBlade){
                        if ((m_savedWakeLines.at(m).at(p).Shed && m_bGLPlotWakeLinesShed) || (!m_savedWakeLines.at(m).at(p).Shed && m_bGLPlotWakeLinesTrail)){
                        glBegin(GL_LINES);
                        {
                            glColor4d(0,0,0,1);
                            if (m_bGLColorLines){
                            if (fabs(m_savedWakeLines.at(m).at(p).Gamma/m_maxGamma)>1) normalized = 1; else normalized = fabs(m_savedWakeLines.at(m).at(p).Gamma/m_maxGamma);
                            hs.h = (1-normalized)*225;
                            glColor4d(hsv2rgb(hs).r, hsv2rgb(hs).g, hsv2rgb(hs).b,1);
                            }
                            glVertex3d(m_savedWakeLines.at(m).at(p).Lx, m_savedWakeLines.at(m).at(p).Ly, m_savedWakeLines.at(m).at(p).Lz);
                            glVertex3d(m_savedWakeLines.at(m).at(p).Tx, m_savedWakeLines.at(m).at(p).Ty, m_savedWakeLines.at(m).at(p).Tz);
                        }
                        glEnd();
                        }
                    glPointSize(1.5);
                    if (m_bGLPlotWakeNodes){
                        glBegin(GL_POINTS);
                        {
                            glColor4d(0,0,0,1);
                            if (m_bGLColorNodes){
                            hs.h = m_savedWakeLines.at(m).at(p).fromTimeL%360;
                            glColor4d(hsv2rgb(hs).r, hsv2rgb(hs).g, hsv2rgb(hs).b,1);
                            }
                            glVertex3d(m_savedWakeLines.at(m).at(p).Lx, m_savedWakeLines.at(m).at(p).Ly, m_savedWakeLines.at(m).at(p).Lz);
                            if (m_bGLColorNodes){
                            hs.h = m_savedWakeLines.at(m).at(p).fromTimeT%360;
                            glColor4d(hsv2rgb(hs).r, hsv2rgb(hs).g, hsv2rgb(hs).b,1);
                            }
                            glVertex3d(m_savedWakeLines.at(m).at(p).Tx, m_savedWakeLines.at(m).at(p).Ty, m_savedWakeLines.at(m).at(p).Tz);

                        }
                        glEnd();
                    }
                }
            }
}
            if (m_savedBladeVortexLines.size()){
                for (int p=0; p<m_savedBladeVortexLines.at(m).size(); p++){
                            if ((m_savedBladeVortexLines.at(m).at(p).Shed &&  m_bGLPlotWakeLinesShed) || (!m_savedBladeVortexLines.at(m).at(p).Shed && m_bGLPlotWakeLinesTrail))
                            glBegin(GL_LINES);
                            {
                                if (fabs(m_savedBladeVortexLines.at(m).at(p).Gamma/m_maxGamma)>1) normalized = 1; else normalized = fabs(m_savedBladeVortexLines.at(m).at(p).Gamma/m_maxGamma);
                                hs.h = (1-normalized)*225;
                                glColor4d(hsv2rgb(hs).r, hsv2rgb(hs).g, hsv2rgb(hs).b,1);
                                glVertex3d(m_savedBladeVortexLines.at(m).at(p).Lx, m_savedBladeVortexLines.at(m).at(p).Ly, m_savedBladeVortexLines.at(m).at(p).Lz);
                                glVertex3d(m_savedBladeVortexLines.at(m).at(p).Tx, m_savedBladeVortexLines.at(m).at(p).Ty, m_savedBladeVortexLines.at(m).at(p).Tz);
                            }
                            glEnd();

                        glPointSize(0.5);
                        if (m_bGLPlotWakeNodes){
                            glBegin(GL_POINTS);
                            {
                                glColor4d(0,0,0,1);
                                glVertex3d(m_savedBladeVortexLines.at(m).at(p).Lx, m_savedBladeVortexLines.at(m).at(p).Ly, m_savedBladeVortexLines.at(m).at(p).Lz);
                                glVertex3d(m_savedBladeVortexLines.at(m).at(p).Tx, m_savedBladeVortexLines.at(m).at(p).Ty, m_savedBladeVortexLines.at(m).at(p).Tz);

                            }
                            glEnd();
                        }

                }
            }



    glEndList();






    glDisable(GL_DEPTH_TEST);

    glLineWidth(1.0);

    glNewList(GLBLADEOUTLINES, GL_COMPILE);
    {
        if(m_bGLPlotRotor){
            glColor4d(0,0,0,1);
            for (int p=0; p<m_savedBladeVizLines.at(m).size(); p++){

                glBegin(GL_LINES);
                {
                    if ((p % bladesize) == 0 || ((p+1) % bladesize) == 0){
                    glVertex3d(m_savedBladeVizLines.at(m).at(p).Lx, m_savedBladeVizLines.at(m).at(p).Ly, m_savedBladeVizLines.at(m).at(p).Lz);
                    glVertex3d(m_savedBladeVizLines.at(m).at(p).Tx, m_savedBladeVizLines.at(m).at(p).Ty, m_savedBladeVizLines.at(m).at(p).Tz);
                    }
                    if(!((p+1) % bladesize) == 0){
                    glVertex3d(m_savedBladeVizLines.at(m).at(p).Tx, m_savedBladeVizLines.at(m).at(p).Ty, m_savedBladeVizLines.at(m).at(p).Tz);
                    glVertex3d(m_savedBladeVizLines.at(m).at(p+1).Tx, m_savedBladeVizLines.at(m).at(p+1).Ty, m_savedBladeVizLines.at(m).at(p+1).Tz);

                    glVertex3d(m_savedBladeVizLines.at(m).at(p).Lx, m_savedBladeVizLines.at(m).at(p).Ly, m_savedBladeVizLines.at(m).at(p).Lz);
                    glVertex3d(m_savedBladeVizLines.at(m).at(p+1).Lx, m_savedBladeVizLines.at(m).at(p+1).Ly, m_savedBladeVizLines.at(m).at(p+1).Lz);
                    }
                }
                glEnd();

           }

        }
    }
    glEndList();

    glNewList(GLBODYMESHPANELS, GL_COMPILE);
    {
    if(m_bGLPlotRotor){
            double max = 0;
            for (int p=0; p<m_savedBladeVizLines.at(m).size(); p++){
                if(fabs(m_savedBladeVizLines.at(m).at(p).Gamma) > max) max = fabs(m_savedBladeVizLines.at(m).at(p).Gamma);
            }
            if (max != 0){
            for (int p=0; p<m_savedBladeVizLines.at(m).size(); p++){

                glBegin(GL_QUADS);
                {
                    if(!((p+1) % bladesize) == 0){

                        hs.h = (1-fabs(m_savedBladeVizLines.at(m).at(p).Gamma)/max)*225;
                        glColor4d(hsv2rgb(hs).r,hsv2rgb(hs).g,hsv2rgb(hs).b,1);

                    glVertex3d(m_savedBladeVizLines.at(m).at(p).Tx, m_savedBladeVizLines.at(m).at(p).Ty, m_savedBladeVizLines.at(m).at(p).Tz);
                    glVertex3d(m_savedBladeVizLines.at(m).at(p).Lx, m_savedBladeVizLines.at(m).at(p).Ly, m_savedBladeVizLines.at(m).at(p).Lz);

                    hs.h = (1-fabs(m_savedBladeVizLines.at(m).at(p+1).Gamma)/max)*225;
                    glColor4d(hsv2rgb(hs).r,hsv2rgb(hs).g,hsv2rgb(hs).b,1);

                    glVertex3d(m_savedBladeVizLines.at(m).at(p+1).Lx, m_savedBladeVizLines.at(m).at(p+1).Ly, m_savedBladeVizLines.at(m).at(p+1).Lz);
                    glVertex3d(m_savedBladeVizLines.at(m).at(p+1).Tx, m_savedBladeVizLines.at(m).at(p+1).Ty, m_savedBladeVizLines.at(m).at(p+1).Tz);
                    }
                }
                glEnd();
           }
        }
    }
        }
        glEndList();


    CVector X, Y, A, B, C, D, Foot, Hub, Top;
    double scale = -m_Blade->getRotorRadius()/3;

    Hub     = m_savedCoordinateSystems.at(m).at(1);
    Foot    = m_savedCoordinateSystems.at(m).at(2);
    Top     = m_savedCoordinateSystems.at(m).at(3);
    X       = m_savedCoordinateSystems.at(m).at(4);


    glNewList(GLTURBINECOORDINATES,GL_COMPILE);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_POLYGON_OFFSET_FILL);
    glPolygonOffset(1.0, 0);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);


    if(m_bGLPlotTurbineModel){
    glBegin(GL_LINES);
    glColor4d(1,0,0,1);
    glVertex3d(Foot.x, Foot.y, Foot.z);
    glVertex3d(Top.x, Top.y, Top.z);
    if (!m_bisVawt) glVertex3d(Top.x, Top.y, Top.z);
    if (!m_bisVawt) glVertex3d(Hub.x, Hub.y, Hub.z);
    glEnd();
    }

    if(m_bGLPlotTurbineModel){
    glBegin(GL_QUADS);
    glEnable(GL_POLYGON_OFFSET_FILL);
    glColor4d(0,0.8,0,0.8);

    A = Foot;
    B = Top;
    Y = X*(A-B);
    Y.Normalize();
    A = (Foot+X*scale+Y*scale);
    B = (Foot+X*scale-Y*scale);
    C = (Foot-X*scale-Y*scale);
    D = (Foot-X*scale+Y*scale);
    glVertex3d(A.x,A.y,A.z);
    glVertex3d(B.x,B.y,B.z);
    glVertex3d(C.x,C.y,C.z);
    glVertex3d(D.x,D.y,D.z);
    glEnd();
    glPointSize(5.0);
    glColor4d(0,0,1,1);

    glBegin(GL_POINTS);
    {
        glVertex3d(Top.x, Top.y, Top.z);
        glVertex3d(Foot.x, Foot.y, Foot.z);
        glVertex3d(Hub.x, Hub.y, Hub.z);
    }
    glEnd();
    glLineWidth(0.5);
    glColor4d(0,0,0,1);
    glBegin(GL_LINES);
    {
        glVertex3d(A.x,A.y,A.z);
        glVertex3d(B.x,B.y,B.z);

        glVertex3d(B.x,B.y,B.z);
        glVertex3d(C.x,C.y,C.z);

        glVertex3d(C.x,C.y,C.z);
        glVertex3d(D.x,D.y,D.z);

        glVertex3d(D.x,D.y,D.z);
        glVertex3d(A.x,A.y,A.z);
    }
    glEnd();
    }

    glEndList();
}

void QLLTSimulation::DrawCoordinateSystems(){

    double scale = -m_Blade->getRotorRadius()/3, rad = m_Blade->getRotorRadius()*1.2;

    glPushAttrib(GL_ALL_ATTRIB_BITS);

    glLineWidth(1.0);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_POLYGON_OFFSET_FILL);
    glPolygonOffset(1.0, 0);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    glBegin(GL_LINES);
    glColor4d(1,0,0,1);



    if(m_bGLPlotCoordinates){
    glColor4d(0,0,0,1);
    glVertex3d(VHub.x, VHub.y, VHub.z);
    if (!m_bisVawt){ glVertex3d(VHub.x+VAxis.x*scale, VHub.y+VAxis.y*scale, VHub.z+VAxis.z*scale);
        if (m_module->GetQLLTDock()->m_showText->isChecked()) g_mainFrame->getGlWidget()->overpaintText(VHub.x+VAxis.x*scale*1.8, VHub.y+VAxis.y*scale*1.8, VHub.z+VAxis.z*scale*1.8, "Rotor Axis");
    }
    if (m_bisVawt){  glVertex3d(VHub.x-VAxis.x*scale, VHub.y-VAxis.y*scale, VHub.z-VAxis.z*scale);
        if (m_module->GetQLLTDock()->m_showText->isChecked()) g_mainFrame->getGlWidget()->overpaintText(VHub.x-VAxis.x*scale*1.8, VHub.y-VAxis.y*scale*1.8, VHub.z-VAxis.z*scale*1.8, "Rotor Axis");
    }

    for (int i=0;i<VBAxes.size();i++){
        if (!m_bisVawt){
            glVertex3d(VBOrigins[i].x, VBOrigins[i].y, VBOrigins[i].z);
            glVertex3d(CVector(VBOrigins[i]+VBAxes[i]*rad).x, CVector(VBOrigins[i]+VBAxes[i]*rad).y, CVector(VBOrigins[i]+VBAxes[i]*rad).z);
            if (m_module->GetQLLTDock()->m_showText->isChecked()) g_mainFrame->getGlWidget()->overpaintText(CVector(VBOrigins[i]+VBAxes[i]*rad*1.3).x, CVector(VBOrigins[i]+VBAxes[i]*rad*1.3).y, CVector(VBOrigins[i]+VBAxes[i]*rad*1.3).z, QString("Blade "+QString().number(i+1,'f',0)));
        }
        if (m_bisVawt) {
            glVertex3d(CVector(VBOrigins[i]-VBAxes[i]*rad/2).x, CVector(VBOrigins[i]-VBAxes[i]*rad/2).y, CVector(VBOrigins[i]-VBAxes[i]*rad/2).z);
            glVertex3d(CVector(VBOrigins[i]+VBAxes[i]*rad/2).x, CVector(VBOrigins[i]+VBAxes[i]*rad/2).y, CVector(VBOrigins[i]+VBAxes[i]*rad/2).z);
            if (m_module->GetQLLTDock()->m_showText->isChecked()) g_mainFrame->getGlWidget()->overpaintText(CVector(VBOrigins[i]+VBAxes[i]*rad/2*1.6).x, CVector(VBOrigins[i]+VBAxes[i]*rad/2*1.6).y, CVector(VBOrigins[i]+VBAxes[i]*rad/2*1.6).z, QString("Blade "+QString().number(i+1,'f',0)));
            }
            }
    }
    glVertex3d(0,0,0);
    glVertex3d(rad,0,0);
    glVertex3d(0,0,0);
    glVertex3d(0,rad,0);
    glVertex3d(0,0,0);
    glVertex3d(0,0,rad);
    glEnd();

    if (m_module->GetQLLTDock()->m_showText->isChecked()) {
        if (!m_bisVawt) g_mainFrame->getGlWidget()->overpaintText(VHub.x+VAxis.x*scale*1.8, VHub.y+VAxis.y*scale*1.8, VHub.z+VAxis.z*scale*1.8, "Rotor Axis");
        if (m_bisVawt)  g_mainFrame->getGlWidget()->overpaintText(VHub.x-VAxis.x*scale*1.8, VHub.y-VAxis.y*scale*1.8, VHub.z-VAxis.z*scale*1.8, "Rotor Axis");
        for (int i=0;i<VBAxes.size();i++){
            if (!m_bisVawt) g_mainFrame->getGlWidget()->overpaintText(CVector(VBOrigins[i]+VBAxes[i]*rad*1.3).x, CVector(VBOrigins[i]+VBAxes[i]*rad*1.3).y, CVector(VBOrigins[i]+VBAxes[i]*rad*1.3).z, QString("Blade "+QString().number(i+1,'f',0)));
            if (m_bisVawt) g_mainFrame->getGlWidget()->overpaintText(CVector(VBOrigins[i]+VBAxes[i]*rad/2*1.6).x, CVector(VBOrigins[i]+VBAxes[i]*rad/2*1.6).y, CVector(VBOrigins[i]+VBAxes[i]*rad/2*1.6).z, QString("Blade "+QString().number(i+1,'f',0)));
        }
        g_mainFrame->getGlWidget()->overpaintText(rad*1.3,0,0,"X Global");
        g_mainFrame->getGlWidget()->overpaintText(0,rad*1.3,0,"Y Global");
        g_mainFrame->getGlWidget()->overpaintText(0,0,rad*1.3,"Z Global");
    }

    glPopAttrib();

}

void QLLTSimulation::InitRotorCoordinates(){

    if (!m_bisVawt){
        m_RotorAxis.Set(1,0,0);
        if (m_bUpWind) m_RotorHub.Set(-m_RotorOverhang,0,m_HubHeight);
        else m_RotorHub.Set(m_RotorOverhang,0,m_HubHeight);
        m_TowerTop.Set(0,0,m_HubHeight);
        m_TowerFoot.Set(0,0,0);
    }
    if (m_bisVawt){
        m_RotorAxis.Set(0,0,1);
        m_RotorHub.Set(0,0,m_HubHeight);
        m_TowerTop.Set(0,0,m_TowerHeight);
        m_TowerFoot.Set(0,0,0);
    }
    m_YawOld = 0;
    m_TeeterOld = 0;
    m_PlatformRollAngleXOld = 0;
    m_PlatformPitchAngleYOld = 0;
    m_PlatformYawAngleZOld = 0;
    m_PitchOld = 0;
    m_PitchAngle = 0;
    m_PlatformTranslationOld.Set(0,0,0);
    m_PlatformTranslation.Set(0,0,0);

    m_BladeAxes.clear();
    m_BladeOrigins.clear();
}

void QLLTSimulation::ClearOutputArrays(){

    m_BladeOutputData.clear();
    m_results.clear();
    m_availableVariables.clear();

    m_CP.clear();
    m_CT.clear();
    m_CM.clear();
    m_TSR.clear();
    m_Power.clear();
    m_Torque.clear();
    m_Torque.clear();
    m_Time.clear();
    m_angPos.clear();
    m_rotSpeed.clear();
    m_ThrustX.clear();
    m_ThrustY.clear();
    m_ThrustZ.clear();
    m_CurrentHHWindSpeed.clear();
    m_CurrentPitchAngle.clear();
    m_CurrentYawAngle.clear();
    m_CurrentHorizontalInflowAngle.clear();
    m_CurrentVerticalInflowSpeed.clear();
    m_Iterations.clear();
    m_RootBendingMomentIP.clear();
    m_RootBendingMomentOOP.clear();
    m_CurrentPlatformYaw.clear();
    m_CurrentPlatformRoll.clear();
    m_CurrentPlatformPitch.clear();
    m_CurrentPlatformTranslationX.clear();
    m_CurrentPlatformTranslationY.clear();
    m_CurrentPlatformTranslationZ.clear();
}

void QLLTSimulation::ClearVortexArrays(){

    for (int i=0; i<m_BladePanel.size();i++){
        delete m_BladePanel.at(i);
    }
    m_BladePanel.clear();

    for (int i=0; i<m_BladeLine.size();i++){
        delete m_BladeLine.at(i);
    }
    m_BladeLine.clear();

    for (int i=0; i<m_WakeLine.size();i++){
        delete m_WakeLine.at(i);
    }
    m_WakeLine.clear();

    for (int i=0; i<m_BladeNode.size();i++){
        delete m_BladeNode.at(i);
    }
    m_BladeNode.clear();

    for (int i=0; i<m_WakeNode.size();i++){
        delete m_WakeNode.at(i);
    }
    m_WakeNode.clear();

    m_savedWakeLines.clear();

    m_savedBladeVizLines.clear();

    m_savedBladeVortexLines.clear();

    m_savedCoordinateSystems.clear();
}

void QLLTSimulation::ResetSimulation(){

    ClearVortexArrays();
    ClearOutputArrays();
    PrepareOutputVectors();

    if (!m_bisVawt) createHAWTTurbineGeometry();
    else createVAWTTurbineGeometry();

    m_currentTimeStep = 0;
    m_maxGamma = 0;
    m_currentTime = 0;
}

void QLLTSimulation::UpdateSimulationParameters(double time){

    QList<double> curParams;

    if (!m_SimParameters.size()) return;

    for (int i=0; i<m_SimParameters.size()-1;i++){
        if (m_SimParameters.at(i).at(0)<= time && m_SimParameters.at(i+1).at(0) > time){
            for (int j=0;j<m_SimParameters.at(i).size();j++){
            curParams.append(m_SimParameters.at(i).at(j)+(time-m_SimParameters.at(i).at(0))/(m_SimParameters.at(i+1).at(0)-m_SimParameters.at(i).at(0))*(m_SimParameters.at(i+1).at(j)-m_SimParameters.at(i).at(j)));
            }
        }
    }
    if (m_SimParameters.at(m_SimParameters.size()-1).at(0) <= time ){
        for (int j=0;j<m_SimParameters.at(m_SimParameters.size()-1).size();j++){
        curParams.append(m_SimParameters.at(m_SimParameters.size()-1).at(j));
        }
    }
    if (m_SimParameters.at(0).at(0) > time ){
        for (int j=0;j<m_SimParameters.at(0).size();j++){
        curParams.append(m_SimParameters.at(0).at(j));
        }
    }

    if (m_windInputType == SIMFILE){
        m_Omega = curParams.at(1)/60.0*2.0*PI;
        V_in.x = curParams.at(2);
        if (curParams.size()>3) V_in.z = curParams.at(3);
        else V_in.z = 0;
        if (curParams.size()>4) m_HorInflowAngle = curParams.at(4);
        else m_HorInflowAngle = 0;
        if (curParams.size()>5) m_YawAngle = curParams.at(5);
        else m_YawAngle = 0;
        if (curParams.size()>6) m_PitchAngle = curParams.at(6);
        else m_PitchAngle = 0;
        if (curParams.size()>7) m_PlatformRollAngleX = curParams.at(7);
        else m_PlatformRollAngleX = 0;
        if (curParams.size()>8) m_PlatformPitchAngleY = curParams.at(8);
        else m_PlatformPitchAngleY = 0;
        if (curParams.size()>9) m_PlatformYawAngleZ = curParams.at(9);
        else m_PlatformYawAngleZ = 0;
        if (curParams.size()>10) m_PlatformTranslation.x = curParams.at(10);
        else m_PlatformTranslation.x = 0;
        if (curParams.size()>11) m_PlatformTranslation.y = curParams.at(11);
        else m_PlatformTranslation.y = 0;
        if (curParams.size()>12) m_PlatformTranslation.z = curParams.at(12);
        else m_PlatformTranslation.z = 0;
        m_VertShear = 0;
    }
    if (m_windInputType == AERODYN){
        V_in.x = curParams.at(1);
        if (curParams.size()>2) m_HorInflowAngle = curParams.at(2);
        else m_HorInflowAngle = 0;
        if (curParams.size()>3) V_in.z = curParams.at(3);
        else V_in.z = 0;
        if (curParams.size()>4) m_HorShear = curParams.at(4);
        else m_HorShear = 0;
        if (curParams.size()>5) m_VertShear = curParams.at(5);
        else m_VertShear = 0;
        if (curParams.size()>6) m_LinVertShear = curParams.at(6);
        else m_LinVertShear = 0;
        if (curParams.size()>7) m_GustSpeed = curParams.at(7);
        else m_GustSpeed = 0;
    }
}

void QLLTSimulation::onStartReplay(){

    lockStores();
    connect(this, SIGNAL(updateProgress(int)), m_module->GetQLLTToolBar(), SLOT(SetTimeStepSlider(int)), Qt::BlockingQueuedConnection);

    m_bStopReplay = false;
    int i;
    while (!m_bStopReplay){
        i = m_module->GetQLLTToolBar()->GetTimeStepSlider()->value();
        if (i<m_module->GetQLLTToolBar()->GetTimeStepSlider()->maximum()) i++;
        else i=0;
        emit updateProgress(i);
//        Sleep(m_module->GetQLLTToolBar()->GetDelay()*1000);
		QThread::msleep (m_module->GetQLLTToolBar()->GetDelay()*1000);
    }

    disconnect(this, SIGNAL(updateProgress(int)), m_module->GetQLLTToolBar(), SLOT(SetTimeStepSlider(int)));
    unlockStores();
}


void QLLTSimulation::onStartAnalysis(void) {
	qDebug() << "start analysis";
	
    connect(this, SIGNAL(geomChanged()),  m_module, SLOT(UpdateView()), Qt::BlockingQueuedConnection);
    connect(this, SIGNAL(updateProgress(int)), m_module->m_QLLTDock, SLOT(updateProgress(int)), Qt::BlockingQueuedConnection);
    connect(this, SIGNAL(updateGraphs()), m_module, SLOT(UpdateGraphs()), Qt::BlockingQueuedConnection);

    if (!m_bContinue) ResetSimulation();
    if (m_bContinue) m_currentTimeStep++;

    lockStores();

	m_bStopRequested = false;
	m_bisRunning = true;
	
    //start timer
	QElapsedTimer timer;
	qint64 nanoSec;
	qint64 sec;
	double min;
	timer.start();

		
    for (m_currentTimeStep; m_currentTimeStep < m_numTimesteps; ++m_currentTimeStep) {
				
        emit updateProgress(m_currentTimeStep);
		
        qDebug() << "start calculation for timestep:" << m_currentTimeStep << "Input Type"<<m_windInputType;
		
        if (m_windInputType == AERODYN ||  m_windInputType == SIMFILE) UpdateSimulationParameters(m_currentTime);

        if (m_currentTimeStep == 1){
            UpdateRotorGeometry();
            AddFirstWakeLines();
            KuttaCondition();
        }
        else if(m_currentTimeStep > 1){
            UpdateRotorGeometry();                                      //rotor rotates
            WakeCutOff();                                               //cuts off vortex lines that are too old
            if (!m_bisVawt) WakeConcentration();                        //lumps vortex lines together
            ConvectWakeNodes();                                         //euler integration of wakenodes
            AddNewWakeLines();
            KuttaCondition();
        }

		if (m_currentTimeStep == 0){
			InitializeBladePanelPositions();
			GuessInitialVortexDistribution();
			GammaBoundFixedPointIteration();
		}
		else{
			GammaBoundFixedPointIteration();
		}

        emit geomChanged();                                         //redraw gl
        storeOutput();

		if (!m_results.at(0)->isEmpty()) {
			m_shownTimeIndex = m_results.at(0)->size() - 1;
        } else {
			m_shownTimeIndex = -1;
		}
        m_module->setShownTimeForAllSimulations();

        emit updateGraphs();

        m_currentTime += dT;

		if (m_bStopRequested)
			break;
	}

    if(m_currentTimeStep == m_numTimesteps) {
        m_bContinue = false;
    }

    //timer
	nanoSec = timer.nsecsElapsed();
	sec = nanoSec / 1000000000;
	min = double(sec/60.0);
	qDebug() << "Process took: " << min << "minutes " << "[" << sec << " seconds" << "]" << endl;
	
	m_bisRunning = false;

    disconnect(this, SIGNAL(geomChanged()), 0, 0);
    disconnect(this, SIGNAL(updateGraphs()), 0, 0);
    disconnect(this, SIGNAL(updateProgress(int)), 0, 0);

    unlockStores();
}

void QLLTSimulation::onComputeCutPlane(QLLTCutPlane *plane, int timestep){

    if (!plane) return;

    #pragma omp parallel default (none) shared (plane, timestep)
    {
        #pragma omp for
        for (int i=0;i<plane->m_points.size();i++){
            for (int j=0;j<plane->m_points.at(i).size();j++){
                plane->m_velocities[i][j]=(CalcTowerInfluence(plane->m_points.at(i).at(j),BiotSavartInductionFromSave(plane->m_points.at(i).at(j),timestep)+getFreeStream(plane->m_points.at(i).at(j), plane->m_time),timestep));
            }
        }
    }

//    plane->m_curVelocity = getMeanFreeStream(CVector(-m_RotorOverhang, 0, m_HubHeight));
    plane->m_curVelocity = getFreeStream(CVector(-m_RotorOverhang, 0, m_HubHeight), plane->m_time);

    CVector meanSpeed = getMeanFreeStream(CVector(-m_RotorOverhang,0,m_TowerHeight));
    double meanWindspeed = meanSpeed.VAbs();
    if (m_windInputType==SIMFILE || m_windInputType == AERODYN){
        meanWindspeed = 0;
        int speedpos;
        if (m_bisVawt) speedpos = 13;
        else speedpos = 12;
        for (int i = 0; i < m_results.at(speedpos)->size(); ++i) {
            meanWindspeed += m_results.at(speedpos)->at(i) / m_results.at(speedpos)->size();
        }
    }
    plane->m_meanVelocity = meanWindspeed;

    qDebug() << plane->m_meanVelocity << "mean Plane";

    plane->is_computed = true;
}

void QLLTSimulation::storeOutput () {
    if (m_currentTime >= m_TStart){
        if (m_bisVawt) CalcVAWTResults();
        else if (!m_bisVawt) CalcHAWTResults();
    }
    storeGeometry();
}


void QLLTSimulation::CalcHAWTResults(){

    double Cp, Ct, Cn, Cm, power=0, thrust=0, bendingoop=0, bendingip=0, torque=0;
    CVector InflowNorm, thrustVec;
    thrustVec.Set(0,0,0);
    QVector<QVector<double> > BladeTimestepData;
    QVector<double> CL, CD, RE, VIND, VTOWER, DYNP, VABS, VSAMPLEIND, CN, CT, AXIND, TANIND, FORCEN, FORCET, AOA, RADIUS, CHORD, CLCD, TWIST, GAMMA, VSAMPLE, VROTATIONAL, DELTA_RE;

    TSR = m_Omega *m_Blade->getRotorRadius() / getFreeStream(m_RotorHub).VAbs();

    if (m_bisStandStill){
        TSR = 0;
        m_Omega = 0;
    }


    for (int k = 0; k < m_BladePanel.size(); ++k) {

//        phi = m_BladePanel[k]->m_AoA+m_BladePanel[k]->twistAngle;

        InflowNorm = m_BladePanel[k]->m_V_sampled;
        InflowNorm.Normalize();

        ///traditional way to compute Cn, Ct via angle phi -> developed for 2D BEM, should not be used with cone angles
//      Ct = m_BladePanel[k]->m_CL*sin(phi/180*PI)-m_BladePanel[k]->m_CD*cos(phi/180*PI);
//      Cn = m_BladePanel[k]->m_CL*cos(phi/180*PI)+m_BladePanel[k]->m_CD*sin(phi/180*PI);
        ///new implemented way to compute Cn, Ct through panel vectors -> more general, prefered version
        Ct = m_BladePanel[k]->LiftDragVector.dot(m_BladePanel[k]->tangentialVector);
        Cn =  m_BladePanel[k]->LiftDragVector.dot(m_RotorAxis);

        torque += Ct * pow(m_BladePanel[k]->m_V_inPlane.VAbs(), 2) * m_BladePanel[k]->Area * m_BladePanel[k]->radius *
                0.5*m_Density;
        power += Ct * pow(m_BladePanel[k]->m_V_inPlane.VAbs(), 2) * m_BladePanel[k]->Area * m_BladePanel[k]->radius *
                m_Omega * 0.5*m_Density;
        thrust += Cn * pow(m_BladePanel[k]->m_V_inPlane.VAbs(), 2) * m_BladePanel[k]->Area * 0.5*m_Density;
        bendingoop += Cn * pow(m_BladePanel[k]->m_V_inPlane.VAbs(), 2) * m_BladePanel[k]->Area * 0.5 * m_Density * m_BladePanel[k]->radius;
        bendingip +=  Ct * pow(m_BladePanel[k]->m_V_inPlane.VAbs(), 2) * m_BladePanel[k]->Area * 0.5 * m_Density * m_BladePanel[k]->radius;
        thrustVec += m_BladePanel[k]->LiftDragVector * pow(m_BladePanel[k]->m_V_inPlane.VAbs(), 2) * m_BladePanel[k]->Area * 0.5 * m_Density;

        double REY = m_BladePanel[k]->chord*m_BladePanel[k]->m_V_inPlane.VAbs()/m_KinViscosity;

        if (k<m_BladePanel.size()/m_numBlades){
        CL.append(m_BladePanel[k]->m_CL);
        CD.append(m_BladePanel[k]->m_CD);
        VABS.append(m_BladePanel[k]->m_V_total.VAbs());
        VSAMPLEIND.append(CVector(m_BladePanel[k]->m_V_induced+m_BladePanel[k]->m_V_sampled).VAbs());
        VSAMPLE.append(m_BladePanel[k]->m_V_sampled.VAbs());
        VROTATIONAL.append(m_BladePanel[k]->m_V_rotational.VAbs());
        VIND.append(m_BladePanel[k]->m_V_induced.VAbs());
        VTOWER.append(m_BladePanel[k]->m_V_tower.VAbs());
        CN.append(Cn);
        CT.append(Ct);
        FORCEN.append(Cn * pow(m_BladePanel[k]->m_V_inPlane.VAbs(), 2) * m_BladePanel[k]->chord * 0.5 * m_Density);
        FORCET.append(Ct * pow(m_BladePanel[k]->m_V_inPlane.VAbs(), 2) * m_BladePanel[k]->chord * 0.5 * m_Density);
        AOA.append(m_BladePanel[k]->m_AoA);
        RADIUS.append(m_BladePanel[k]->length);
        RE.append(REY);
        AXIND.append(1 - m_RotorAxis.dot(m_BladePanel[k]->m_V_total) / m_RotorAxis.dot(m_BladePanel[k]->m_V_sampled));
        TANIND.append(-m_BladePanel[k]->tangentialVector.dot(m_BladePanel[k]->m_V_total) / m_BladePanel[k]->radius / m_Omega - 1);
        CHORD.append(m_BladePanel[k]->chord);
        CLCD.append(m_BladePanel[k]->m_CL/m_BladePanel[k]->m_CD);
        TWIST.append(m_BladePanel[k]->twistAngle);
        GAMMA.append(m_BladePanel[k]->m_Gamma);
        DYNP.append(0.5*m_Density*pow(m_BladePanel[k]->m_V_total.VAbs(),2));
        DELTA_RE.append(REY-m_BladePanel[k]->m_RE);
        }
    }
    BladeTimestepData.append(CL);
    BladeTimestepData.append(CD);
    BladeTimestepData.append(CLCD);
    BladeTimestepData.append(GAMMA);
    BladeTimestepData.append(AOA);
    BladeTimestepData.append(RADIUS);
    BladeTimestepData.append(CHORD);
    BladeTimestepData.append(TWIST);
    BladeTimestepData.append(CN);
    BladeTimestepData.append(CT);
    BladeTimestepData.append(FORCEN);
    BladeTimestepData.append(FORCET);
    BladeTimestepData.append(DYNP);
    BladeTimestepData.append(VABS);
    BladeTimestepData.append(VSAMPLEIND);
    BladeTimestepData.append(VSAMPLE);
    BladeTimestepData.append(VROTATIONAL);
    BladeTimestepData.append(VIND);
    BladeTimestepData.append(VTOWER);
    BladeTimestepData.append(AXIND);
    BladeTimestepData.append(TANIND);
    BladeTimestepData.append(RE);
    BladeTimestepData.append(DELTA_RE);




    m_BladeOutputData.append(BladeTimestepData);

    Cp = power / (pow(m_Blade->getRotorRadius(),2) * PI * 0.5*m_Density * pow(getMeanFreeStream(m_RotorHub).VAbs(),3));
    Cm = Cp / TSR;
    Ct = thrust / (pow(m_Blade->getRotorRadius(),2) * PI * 0.5*m_Density * pow(getMeanFreeStream(m_RotorHub).VAbs(),2));

    m_CP.append(Cp);
    m_CT.append(Ct);
    m_CM.append(Cm);
    m_TSR.append(TSR);
    m_Power.append(power/1000);
    m_Torque.append(torque);
    m_Time.append(m_currentTime);
    m_angPos.append(m_BladePanel.at(0)->angularPos);
    m_rotSpeed.append(m_Omega/2/PI*60);
    m_ThrustX.append(thrustVec.x);
    m_ThrustY.append(thrustVec.y);
    m_ThrustZ.append(thrustVec.z);
    m_CurrentHHWindSpeed.append(getFreeStream(m_RotorHub).VAbs());
    m_CurrentPitchAngle.append(m_PitchAngle);
    m_CurrentYawAngle.append(m_YawAngle);
    m_CurrentHorizontalInflowAngle.append(m_HorInflowAngle);
    m_CurrentVerticalInflowSpeed.append(m_VertInflowSpeed);
    m_RootBendingMomentIP.append(bendingip);
    m_RootBendingMomentOOP.append(bendingoop);



}

void QLLTSimulation::CalcVAWTResults(){

    double Cp, Ct, Cn, Cm, power=0, area=0, PanelLength, torque=0;
    CVector InflowNorm, Thrust;
    Thrust.Set(0,0,0);

    TSR = m_Omega *m_Blade->m_MaxRadius / getFreeStream(m_RotorHub).VAbs();

    if (m_bisStandStill){
        TSR = 0;
        m_Omega = 0;
    }

    QVector<QVector<double> > BladeTimestepData;
    QVector<double> CL, CD, RE, VIND, VTOWER, DYNP, VABS, VSAMPLEIND, CN, CT, AXIND, INTER, FORCEN, FORCET, AOA, RADIUS, CHORD, CLCD, TWIST, GAMMA, VSAMPLE, VROTATIONAL, DELTA_RE;

            for (int ar=0; ar<m_Blade->TPos.size()-1; ar++){
                area += (m_Blade->TOffset[ar]+m_Blade->TOffset[ar+1])*(m_Blade->TPos[ar+1]-m_Blade->TPos[ar]);
            }
    for (int k = 0; k < m_BladePanel.size(); ++k) {

                PanelLength = fabs(m_RotorAxis.dot(CVector(m_BladePanel[k]->pLA->x, m_BladePanel[k]->pLA->y, m_BladePanel[k]->pLA->z)-CVector(m_BladePanel[k]->pLB->x, m_BladePanel[k]->pLB->y, m_BladePanel[k]->pLB->z)));

                InflowNorm = m_BladePanel[k]->m_V_sampled;
                InflowNorm.Normalize();

                Ct = m_BladePanel[k]->LiftDragVector.dot(m_BladePanel[k]->tangentialVector);
                Cn =  m_BladePanel[k]->LiftDragVector.dot(m_BladePanel[k]->radialVector);

                torque += Ct * pow(m_BladePanel[k]->m_V_inPlane.VAbs(),2) * m_BladePanel[k]->Area * m_BladePanel[k]->length * 0.5 * m_Density;
                power += Ct * pow(m_BladePanel[k]->m_V_inPlane.VAbs(),2) * m_BladePanel[k]->Area * m_Omega * m_BladePanel[k]->length * 0.5 * m_Density;
                Thrust += m_BladePanel[k]->LiftDragVector * pow(m_BladePanel[k]->m_V_inPlane.VAbs(),2) * m_BladePanel[k]->Area * 0.5 * m_Density;
                double REY = m_BladePanel[k]->chord*m_BladePanel[k]->m_V_inPlane.VAbs()/m_KinViscosity;

                if (k<m_BladePanel.size()/m_numBlades){
                CL.append(m_BladePanel[k]->m_CL);
                CD.append(m_BladePanel[k]->m_CD);
                VABS.append(m_BladePanel[k]->m_V_total.VAbs());
                VSAMPLEIND.append(CVector(m_BladePanel[k]->m_V_induced+m_BladePanel[k]->m_V_sampled).VAbs());
                VSAMPLE.append(m_BladePanel[k]->m_V_sampled.VAbs());
                VROTATIONAL.append(m_BladePanel[k]->m_V_rotational.VAbs());
                VIND.append(m_BladePanel[k]->m_V_induced.VAbs());
                VTOWER.append(m_BladePanel[k]->m_V_tower.VAbs());
                CN.append(Cn);
                CT.append(Ct);
                FORCEN.append(Cn * pow(m_BladePanel[k]->m_V_inPlane.VAbs(), 2) * m_BladePanel[k]->chord * 0.5 * m_Density);
                FORCET.append(Ct * pow(m_BladePanel[k]->m_V_inPlane.VAbs(), 2) * m_BladePanel[k]->chord * 0.5 * m_Density);
                AOA.append(m_BladePanel[k]->m_AoA);
                RADIUS.append(m_BladePanel[k]->heigth);
                RE.append(REY);
                double axind = 1 - InflowNorm.dot(m_BladePanel[k]->m_V_induced+m_BladePanel[k]->m_V_sampled) / InflowNorm.dot(m_BladePanel[k]->m_V_sampled);
                AXIND.append(axind);
                INTER.append(1-axind);
                CHORD.append(m_BladePanel[k]->chord);
                CLCD.append(m_BladePanel[k]->m_CL/m_BladePanel[k]->m_CD);
                TWIST.append(m_BladePanel[k]->twistAngle);
                GAMMA.append(m_BladePanel[k]->m_Gamma);
                DYNP.append(0.5*m_Density*pow(m_BladePanel[k]->m_V_total.VAbs(),2));
                DELTA_RE.append(REY-m_BladePanel[k]->m_RE);
                }
    }

    BladeTimestepData.append(CL);
    BladeTimestepData.append(CD);
    BladeTimestepData.append(CLCD);
    BladeTimestepData.append(GAMMA);
    BladeTimestepData.append(AOA);
    BladeTimestepData.append(RADIUS);
    BladeTimestepData.append(CHORD);
    BladeTimestepData.append(TWIST);
    BladeTimestepData.append(CN);
    BladeTimestepData.append(CT);
    BladeTimestepData.append(FORCEN);
    BladeTimestepData.append(FORCET);
    BladeTimestepData.append(DYNP);
    BladeTimestepData.append(VABS);
    BladeTimestepData.append(VSAMPLEIND);
    BladeTimestepData.append(VSAMPLE);
    BladeTimestepData.append(VROTATIONAL);
    BladeTimestepData.append(VIND);
    BladeTimestepData.append(VTOWER);
    BladeTimestepData.append(AXIND);
    BladeTimestepData.append(INTER);
    BladeTimestepData.append(RE);  
    BladeTimestepData.append(DELTA_RE);


    m_BladeOutputData.append(BladeTimestepData);

    Cp = power / ( area * 0.5*m_Density * pow(getMeanFreeStream(m_RotorHub).VAbs(),3));
    Cm = Cp / TSR;
    Ct = Thrust.VAbs() / (area * PI * 0.5*m_Density * pow(getMeanFreeStream(m_RotorHub).VAbs(),2));

    m_CP.append(Cp);
    int m = 360/m_AngularDisc;
    double cpAv = 0;
    if (m_CP.size() >= m){
    for (int k=m_CP.size()-m; k<m_CP.size();k++) cpAv+=m_CP.at(k)/m;
    }
    else cpAv = 0;

    m_CPAV.append(cpAv);
    m_CT.append(Ct);
    m_CM.append(Cm);
    m_TSR.append(TSR);
    m_Power.append(power/1000);
    m_Torque.append(torque);
    m_Time.append(m_currentTime);
    m_angPos.append(m_BladePanel.at(0)->angularPos);
    m_rotSpeed.append(m_Omega/2/PI*60);
    m_ThrustX.append(Thrust.x);
    m_ThrustY.append(Thrust.y);
    m_ThrustZ.append(Thrust.z);
    m_CurrentHHWindSpeed.append(getFreeStream(m_RotorHub).VAbs());
    m_CurrentPitchAngle.append(m_PitchAngle);
    m_CurrentYawAngle.append(m_YawAngle);
    m_CurrentHorizontalInflowAngle.append(m_HorInflowAngle);
    m_CurrentVerticalInflowSpeed.append(m_VertInflowSpeed);
}


void QLLTSimulation::storeGeometry(){

    if (!m_bStoreWake || m_currentTimeStep == 0 || (m_currentTime >= m_TStart && m_currentTime < m_TStart+dT) || m_currentTime < m_TStart){
        m_savedWakeLines.clear();
        m_savedBladeVizLines.clear();
        m_savedBladeVortexLines.clear();
        m_savedCoordinateSystems.clear();
    }

    QList<DummyLine> vizLines;
    for (int v=0; v<m_WakeLine.size();v++){
        DummyLine d(m_WakeLine.at(v)->pL->x,m_WakeLine.at(v)->pL->y,m_WakeLine.at(v)->pL->z,m_WakeLine.at(v)->pT->x,m_WakeLine.at(v)->pT->y,m_WakeLine.at(v)->pT->z,m_WakeLine.at(v)->Gamma,m_WakeLine.at(v)->fromStation,m_WakeLine.at(v)->isShed,m_WakeLine.at(v)->coreSizeSquared, m_WakeLine.at(v)->pL->fromTimestep, m_WakeLine.at(v)->pT->fromTimestep);
        vizLines.append(d);
    }
    m_savedWakeLines.append(vizLines);

    QList<DummyLine> vizBladeLines;
    for (int v=0; v<m_BladeLine.size();v++){
        DummyLine d(m_BladeLine.at(v)->pL->x,m_BladeLine.at(v)->pL->y,m_BladeLine.at(v)->pL->z,m_BladeLine.at(v)->pT->x,m_BladeLine.at(v)->pT->y,m_BladeLine.at(v)->pT->z,m_BladeLine.at(v)->VizGamma,m_BladeLine.at(v)->fromStation,m_BladeLine.at(v)->isShed,m_BladeLine.at(v)->coreSizeSquared);
        vizBladeLines.append(d);
    }
    m_savedBladeVizLines.append(vizBladeLines);

    double a = 1.25643;
    double coreSizeSquared = 4*a*m_TurbulentViscosity*m_KinViscosity*(m_VortexTimeOffset);

    QList<DummyLine> vortexBladeLines;
    for (int v=0; v<m_BladePanel.size();v++){

        DummyLine lalb(m_BladePanel.at(v)->VortPtA.x,m_BladePanel.at(v)->VortPtA.y,m_BladePanel.at(v)->VortPtA.z,m_BladePanel.at(v)->VortPtB.x,m_BladePanel.at(v)->VortPtB.y,m_BladePanel.at(v)->VortPtB.z,m_BladePanel.at(v)->m_Gamma,-1,true,coreSizeSquared);
        vortexBladeLines.append(lalb);

        if(m_currentTimeStep == 0){
        DummyLine tbta(m_BladePanel.at(v)->pTB->x,m_BladePanel.at(v)->pTB->y,m_BladePanel.at(v)->pTB->z,m_BladePanel.at(v)->pTA->x,m_BladePanel.at(v)->pTA->y,m_BladePanel.at(v)->pTA->z,m_BladePanel.at(v)->m_Gamma,-1,true,coreSizeSquared);
        vortexBladeLines.append(tbta);}

        if (m_BladePanel.at(v)->isHub){
            DummyLine tala(m_BladePanel.at(v)->pTA->x,m_BladePanel.at(v)->pTA->y,m_BladePanel.at(v)->pTA->z,m_BladePanel.at(v)->VortPtA.x,m_BladePanel.at(v)->VortPtA.y,m_BladePanel.at(v)->VortPtA.z,m_BladePanel.at(v)->m_Gamma,-1,false,coreSizeSquared);
            vortexBladeLines.append(tala);
        }

        else if (m_BladePanel.at(v)->isTip){
            DummyLine lbtb(m_BladePanel.at(v)->VortPtB.x,m_BladePanel.at(v)->VortPtB.y,m_BladePanel.at(v)->VortPtB.z,m_BladePanel.at(v)->pTB->x,m_BladePanel.at(v)->pTB->y,m_BladePanel.at(v)->pTB->z,m_BladePanel.at(v)->m_Gamma,-1,false,coreSizeSquared);
            vortexBladeLines.append(lbtb);

            DummyLine tala(m_BladePanel.at(v)->pTA->x,m_BladePanel.at(v)->pTA->y,m_BladePanel.at(v)->pTA->z,m_BladePanel.at(v)->VortPtA.x,m_BladePanel.at(v)->VortPtA.y,m_BladePanel.at(v)->VortPtA.z,m_BladePanel.at(v)->m_Gamma-m_BladePanel.at(v-1)->m_Gamma,-1,false,coreSizeSquared);
            vortexBladeLines.append(tala);
        }

        else{
            DummyLine tala(m_BladePanel.at(v)->pTA->x,m_BladePanel.at(v)->pTA->y,m_BladePanel.at(v)->pTA->z,m_BladePanel.at(v)->VortPtA.x,m_BladePanel.at(v)->VortPtA.y,m_BladePanel.at(v)->VortPtA.z,m_BladePanel.at(v)->m_Gamma-m_BladePanel.at(v-1)->m_Gamma,-1,false,coreSizeSquared);
            vortexBladeLines.append(tala);
        }


    }
    m_savedBladeVortexLines.append(vortexBladeLines);

    QList<CVector> vizCoordinateSystem;
    CVector axis, hub, foot, top, X;
    X.Set(1,0,0);
    axis = m_RotorAxis;
    hub = m_RotorHub;
    foot = m_TowerFoot;
    top = m_TowerTop;
    X.RotateY(CVector(0,0,0),m_PlatformPitchAngleY);
    X.RotateZ(CVector(0,0,0),m_PlatformYawAngleZ);
    vizCoordinateSystem.append(axis);
    vizCoordinateSystem.append(hub);
    vizCoordinateSystem.append(foot);
    vizCoordinateSystem.append(top);
    vizCoordinateSystem.append(X);
    for(int i=0; i<m_numBlades;i++){
       vizCoordinateSystem.append(m_BladeAxes.at(i));
       vizCoordinateSystem.append(m_BladeOrigins.at(i));
    }
    m_savedCoordinateSystems.append(vizCoordinateSystem);




}

CVector QLLTSimulation::getFreeStream (CVector EvalPt, double time) {
        if (time == -1) time = m_currentTime;
        else UpdateSimulationParameters(time); // reconstruction for cut-plane
    if (m_WindField) {
        // rotor coordinates need to be transformed!!!!!! (instead of transforming windfield for skewed inflow)
        EvalPt.RotZ(-m_HorInflowAngle/180*PI);
        CVector V_free;
        V_free.Set(m_WindField->getWindspeed (EvalPt, time - dT/2));
        V_free.RotZ(m_HorInflowAngle/180*PI);
		return V_free;
    } else if (m_windInputType == CONSTANT){
        CVector Vz;
        Vz.Set(m_InflowSpeed*pow(EvalPt.z/m_HubHeight,m_ShearCoeff),0,m_VertInflowSpeed);
        Vz.RotZ(m_HorInflowAngle/180*PI);
        return Vz;
	}
    else if (m_windInputType == SIMFILE) {
        CVector V_free(0,0,0);
        EvalPt.RotZ(-m_HorInflowAngle/180*PI);
        V_free.x = V_in.x * pow((EvalPt.z/m_RotorHub.z),m_VertShear);
        V_free.z = V_in.z;
        V_free.RotZ(m_HorInflowAngle/180*PI);
        return V_free;
    }
    else if (m_windInputType == AERODYN) {
        CVector V_free(0,0,0);
        double radius = m_Blade->getRotorRadius();
        EvalPt.RotZ(-m_HorInflowAngle/180*PI);
        V_free.x = EvalPt.y/radius*V_in.x*m_HorShear/2 + EvalPt.z/radius*V_in.x*m_LinVertShear/2 + V_in.x * pow((EvalPt.z/m_RotorHub.z),m_VertShear)+m_GustSpeed;
        V_free.z = V_in.z;
        V_free.RotZ(m_HorInflowAngle/180*PI);
        return V_free;
    }
}

CVector QLLTSimulation::getMeanFreeStream (CVector EvalPt) {
    if (m_WindField) {
        CVector V_free;
        if (m_WindField->getIncludeShear()){
        V_free.Set(m_WindField->getMeanWindSpeedAtHub()*log(EvalPt.z/m_WindField->getRoughnessLength())/log(m_WindField->getWindSpeedMeasurementHeight()/m_WindField->getRoughnessLength()),0,0);
        }
        else V_free.Set(m_WindField->getMeanWindSpeedAtHub(),0,0);
        V_free.RotZ(m_HorInflowAngle/180 * PI);
        return V_free;
    } else if (m_windInputType == CONSTANT){
        CVector Vz;
        Vz.Set(m_InflowSpeed*pow(EvalPt.z/m_HubHeight,m_ShearCoeff),0,m_VertInflowSpeed);
        Vz.RotZ(m_HorInflowAngle/180*PI);
        return Vz;
    }
    else if (m_windInputType == SIMFILE) {
        CVector V_free(0,0,0);
        EvalPt.RotZ(-m_HorInflowAngle/180*PI);
        V_free.x = V_in.x * pow((EvalPt.z/m_RotorHub.z),m_VertShear);
        V_free.z = V_in.z;
        V_free.RotZ(m_HorInflowAngle/180*PI);
        return V_free;
    }
    else if (m_windInputType == AERODYN) {
        CVector V_free(0,0,0);
        double radius = m_Blade->getRotorRadius();
        EvalPt.RotZ(-m_HorInflowAngle/180*PI);
        V_free.x = EvalPt.y/radius*V_in.x*m_HorShear/2 + EvalPt.z/radius*V_in.x*m_LinVertShear/2 + V_in.x * pow((EvalPt.z/m_RotorHub.z),m_VertShear)+m_GustSpeed;
        V_free.z = V_in.z;
        V_free.RotZ(m_HorInflowAngle/180*PI);
        return V_free;
    }
}

CVector QLLTSimulation::CalcTowerInfluence(CVector EvalPt, CVector V_ref, int timestep) {

    if (!m_bIncludeTower) return V_ref;

    CVector O(0,0,0);

    if (timestep < 0){
    // rotate evaluation point back to reference rotor position
    EvalPt.Translate(m_PlatformTranslationOld*-1);
    EvalPt.RotateZ(O,-m_PlatformYawAngleZOld);
    EvalPt.RotateY(O,-m_PlatformPitchAngleYOld);
    EvalPt.RotateX(O,-m_PlatformRollAngleXOld);

    // rotate reference velocity back to reference rotor position
    V_ref.RotateZ(O,-m_PlatformYawAngleZOld);
    V_ref.RotateY(O,-m_PlatformPitchAngleYOld);
    V_ref.RotateX(O,-m_PlatformRollAngleXOld);
    }
    else{
        EvalPt.Translate(CVector(m_CurrentPlatformTranslationX.at(timestep), m_CurrentPlatformTranslationY.at(timestep), m_CurrentPlatformTranslationZ.at(timestep))*-1);
        EvalPt.RotateZ(O,-m_CurrentPlatformYaw.at(timestep));
        EvalPt.RotateY(O,-m_CurrentPlatformPitch.at(timestep));
        EvalPt.RotateX(O,-m_CurrentPlatformRoll.at(timestep));

        // rotate reference velocity back to reference rotor position
        V_ref.RotateZ(O,-m_CurrentPlatformYaw.at(timestep));
        V_ref.RotateY(O,-m_CurrentPlatformPitch.at(timestep));
        V_ref.RotateX(O,-m_CurrentPlatformRoll.at(timestep));
    }

    if (EvalPt.z > m_TowerHeight) return V_ref;

    double radius = m_TowerBottomRadius + (m_TowerTopRadius-m_TowerBottomRadius) * EvalPt.z / m_TowerHeight;

    if (CVector(EvalPt.x,EvalPt.y,0).VAbs() <= radius) return V_ref;

    EvalPt.z = 0;
    CVector Vref2 = CVector(V_ref.x, V_ref.y, 0);
    double alpha = acos(CVector(1,0,0).dot(Vref2) / (Vref2).VAbs());
    if (Vref2.y < 0 && Vref2.x > 0)
		alpha = 2*PI-alpha;

    //now the coordinates need to be rotated
    const double sqrtOfD = sqrt(EvalPt.VAbs()/radius);

    CVector newCoord = EvalPt;
    newCoord.RotZ(-alpha);

    double x = newCoord.x/radius;
    double y = newCoord.y/radius;

    double utower;
	if (x < 0 || fabs(y) > sqrtOfD) {
		utower = 0;
	} else {
		utower = m_TowerDrag / sqrtOfD * pow(cos(PI/2 * y/sqrtOfD), 2);
	}

    const double u = 1-(pow((x+0.1),2)-pow(y,2))/pow((pow((x+0.1),2)+pow(y,2)),2)-utower;
    const double v = -2*((x+0.1)*y)/pow((pow((x+0.1),2)+pow(y,2)),2);

    CVector res(u*V_ref.VAbs(), v*V_ref.VAbs(), 0);
    res.RotZ(alpha);
    res.z = V_ref.z;

    //rotate resulting velocity vector back into current rotor position
    if (timestep < 0){
    res.RotateX(O,m_PlatformRollAngleXOld);
    res.RotateY(O,m_PlatformPitchAngleYOld);
    res.RotateZ(O,m_PlatformYawAngleZOld);
    }
    else{
    res.RotateZ(O,m_CurrentPlatformYaw.at(timestep));
    res.RotateY(O,m_CurrentPlatformPitch.at(timestep));
    res.RotateX(O,m_CurrentPlatformRoll.at(timestep));
    }

    return res;
}

void QLLTSimulation::UpdateVAWTCoordinates(){

    RotateVAWTOriginalPos();

    CVector O(0,0,0);

    for (int j=0;j<m_BladeNode.size();j++){
        m_BladeNode[j]->RotateX(CVector(0,0,0),m_PlatformRollAngleX);   //roll around x-axis
        m_BladeNode[j]->RotateY(CVector(0,0,0),m_PlatformPitchAngleY);  //roll around y-axis
        m_BladeNode[j]->RotateZ(CVector(0,0,0),m_PlatformYawAngleZ);    //roll around z-axis
        m_BladeNode[j]->Translate(m_PlatformTranslation);               //translation
    }

    for (int i=0;i<m_BladeAxes.size();i++){
        m_BladeAxes[i].RotateX(CVector(0,0,0),m_PlatformRollAngleX);
        m_BladeAxes[i].RotateY(CVector(0,0,0),m_PlatformPitchAngleY);
        m_BladeAxes[i].RotateZ(CVector(0,0,0),m_PlatformYawAngleZ);

        m_BladeOrigins[i].RotateX(CVector(0,0,0),m_PlatformRollAngleX);
        m_BladeOrigins[i].RotateY(CVector(0,0,0),m_PlatformPitchAngleY);
        m_BladeOrigins[i].RotateZ(CVector(0,0,0),m_PlatformYawAngleZ);
        m_BladeOrigins[i].Translate(m_PlatformTranslation);
    }

    m_RotorAxis.RotateX(O,m_PlatformRollAngleX);
    m_TowerTop.RotateX(O,m_PlatformRollAngleX);
    m_RotorHub.RotateX(O,m_PlatformRollAngleX);
    m_TowerFoot.RotateX(O,m_PlatformRollAngleX);

    m_RotorHub.RotateY(O,m_PlatformPitchAngleY);
    m_RotorAxis.RotateY(O,m_PlatformPitchAngleY);
    m_TowerTop.RotateY(O,m_PlatformPitchAngleY);
    m_TowerFoot.RotateY(O,m_PlatformPitchAngleY);

    m_TowerTop.RotateZ(O,m_PlatformYawAngleZ);
    m_RotorAxis.RotateZ(O,m_PlatformYawAngleZ);
    m_RotorHub.RotateZ(O,m_PlatformYawAngleZ);
    m_TowerFoot.RotateZ(O,m_PlatformYawAngleZ);

    m_RotorHub.Translate(m_PlatformTranslation);
    m_TowerTop.Translate(m_PlatformTranslation);
    m_TowerFoot.Translate(m_PlatformTranslation);

    m_PlatformRollAngleXOld = m_PlatformRollAngleX;
    m_PlatformPitchAngleYOld = m_PlatformPitchAngleY;
    m_PlatformYawAngleZOld = m_PlatformYawAngleZ;
    m_PlatformTranslationOld = m_PlatformTranslation;

    m_CurrentPlatformYaw.append(m_PlatformYawAngleZ);
    m_CurrentPlatformRoll.append(m_PlatformRollAngleX);
    m_CurrentPlatformPitch.append(m_PlatformPitchAngleY);
    m_CurrentPlatformTranslationX.append(m_PlatformTranslation.x);
    m_CurrentPlatformTranslationY.append(m_PlatformTranslation.y);
    m_CurrentPlatformTranslationZ.append(m_PlatformTranslation.z);

    PitchBlades();
}

void QLLTSimulation::RotateVAWTOriginalPos(){

    //rotations are now performed backwards to put the turbine back in its initial position
    CVector O(0,0,0);

    m_RotorHub.Translate(m_PlatformTranslationOld*-1);
    m_TowerTop.Translate(m_PlatformTranslationOld*-1);
    m_TowerFoot.Translate(m_PlatformTranslationOld*-1);

    m_TowerTop.RotateZ(O,-m_PlatformYawAngleZOld);
    m_RotorAxis.RotateZ(O,-m_PlatformYawAngleZOld);
    m_RotorHub.RotateZ(O,-m_PlatformYawAngleZOld);
    m_TowerFoot.RotateZ(O,-m_PlatformYawAngleZOld);

    m_RotorHub.RotateY(O,-m_PlatformPitchAngleYOld);
    m_RotorAxis.RotateY(O,-m_PlatformPitchAngleYOld);
    m_TowerTop.RotateY(O,-m_PlatformPitchAngleYOld);
    m_TowerFoot.RotateY(O,-m_PlatformPitchAngleYOld);

    m_RotorAxis.RotateX(O,-m_PlatformRollAngleXOld);
    m_TowerTop.RotateX(O,-m_PlatformRollAngleXOld);
    m_RotorHub.RotateX(O,-m_PlatformRollAngleXOld);
    m_TowerFoot.RotateX(O,-m_PlatformRollAngleXOld);


    for (int j=0;j<m_BladeNode.size();j++)
    {
        m_BladeNode[j]->Translate(m_PlatformTranslationOld*-1);              //translation
        m_BladeNode[j]->RotateZ(CVector(0,0,0),-m_PlatformYawAngleZOld);     //roll around z-axis
        m_BladeNode[j]->RotateY(CVector(0,0,0),-m_PlatformPitchAngleYOld);   //roll around y-axis
        m_BladeNode[j]->RotateX(CVector(0,0,0),-m_PlatformRollAngleXOld);    //roll around x-axis
    }
    for (int i=0;i<m_BladeAxes.size();i++){
        m_BladeAxes[i].RotateZ(CVector(0,0,0),-m_PlatformYawAngleZOld);
        m_BladeAxes[i].RotateY(CVector(0,0,0),-m_PlatformPitchAngleYOld);
        m_BladeAxes[i].RotateX(CVector(0,0,0),-m_PlatformRollAngleXOld);

        m_BladeOrigins[i].Translate(m_PlatformTranslationOld*-1);
        m_BladeOrigins[i].RotateZ(CVector(0,0,0),-m_PlatformYawAngleZOld);
        m_BladeOrigins[i].RotateY(CVector(0,0,0),-m_PlatformPitchAngleYOld);
        m_BladeOrigins[i].RotateX(CVector(0,0,0),-m_PlatformRollAngleXOld);
    }
}

void QLLTSimulation::RotateHAWTOriginalPos(){
    //rotations are now performed backwards to put the turbine back in its initial position
    CVector O(0,0,0);

    m_RotorHub.Translate(m_PlatformTranslationOld*-1);
    m_TowerTop.Translate(m_PlatformTranslationOld*-1);
    m_TowerFoot.Translate(m_PlatformTranslationOld*-1);

    m_RotorAxis.RotateZ(O,-m_PlatformYawAngleZOld);
    m_RotorHub.RotateZ(O,-m_PlatformYawAngleZOld);
    m_TowerTop.RotateZ(O,-m_PlatformYawAngleZOld);
    m_TowerFoot.RotateZ(O,-m_PlatformYawAngleZOld);

    m_RotorAxis.RotateY(O,-m_PlatformPitchAngleYOld);
    m_RotorHub.RotateY(O,-m_PlatformPitchAngleYOld);
    m_TowerFoot.RotateY(O,-m_PlatformPitchAngleYOld);
    m_TowerTop.RotateY(O,-m_PlatformPitchAngleYOld);

    m_RotorAxis.RotateX(O,-m_PlatformRollAngleXOld);
    m_RotorHub.RotateX(O,-m_PlatformRollAngleXOld);
    m_TowerFoot.RotateX(O,-m_PlatformRollAngleXOld);
    m_TowerTop.RotateX(O,-m_PlatformRollAngleXOld);

    m_RotorAxis.RotateZ(O,-m_YawOld);
    m_RotorHub.RotateZ(O,-m_YawOld);

    m_RotorAxis.RotateY(O,-m_TeeterOld);

    for (int j=0;j<m_BladeNode.size();j++)
    {
       m_BladeNode[j]->Translate(m_PlatformTranslationOld*-1);  //translation
       m_BladeNode[j]->RotateZ(O,-m_PlatformYawAngleZOld);      //roll around z-axis
       m_BladeNode[j]->RotateY(O,-m_PlatformPitchAngleYOld);    //roll around y-axis
       m_BladeNode[j]->RotateX(O,-m_PlatformRollAngleXOld);     //roll around x-axis
       m_BladeNode[j]->RotateZ(O,-m_YawOld);                    //yaw rotor
       m_BladeNode[j]->RotateY(m_RotorHub, -m_TeeterOld);      //add Teeter angle
    }

    for (int i=0;i<m_BladeAxes.size();i++){
        m_BladeAxes[i].RotateZ(O,-m_PlatformYawAngleZOld);
        m_BladeAxes[i].RotateY(O,-m_PlatformPitchAngleYOld);
        m_BladeAxes[i].RotateX(O,-m_PlatformRollAngleXOld);
        m_BladeAxes[i].RotateZ(O,-m_YawOld);
        m_BladeAxes[i].RotateY(O,-m_TeeterOld);

        m_BladeOrigins[i].Translate(m_PlatformTranslationOld*-1);
        m_BladeOrigins[i].RotateZ(O,-m_PlatformYawAngleZOld);
        m_BladeOrigins[i].RotateY(O,-m_PlatformPitchAngleYOld);
        m_BladeOrigins[i].RotateX(O,-m_PlatformRollAngleXOld);
        m_BladeOrigins[i].RotateZ(O,-m_YawOld);
        m_BladeOrigins[i].RotateY(m_RotorHub,-m_TeeterOld);
    }
}



void QLLTSimulation::UpdateHAWTCoordinates(){

    RotateHAWTOriginalPos();
    CVector O(0,0,0);

    for (int j=0;j<m_BladeNode.size();j++)
    {
       if (m_bUpWind) m_BladeNode[j]->RotateY(m_RotorHub, m_TeeterAngle); //add Teeter angle
       else m_BladeNode[j]->RotateY(m_RotorHub, -m_TeeterAngle); //add Teeter angle
       m_BladeNode[j]->RotateZ(O,-m_YawAngle);               //yaw rotor
       m_BladeNode[j]->RotateX(O,m_PlatformRollAngleX);     //roll around x-axis
       m_BladeNode[j]->RotateY(O,m_PlatformPitchAngleY);    //roll around y-axis
       m_BladeNode[j]->RotateZ(O,m_PlatformYawAngleZ);      //roll around z-axis
       m_BladeNode[j]->Translate(m_PlatformTranslation);    //translation
    }

    for (int i=0;i<m_BladeAxes.size();i++){
        if (m_bUpWind) m_BladeAxes[i].RotateY(O,m_TeeterAngle);
        else m_BladeAxes[i].RotateY(O,-m_TeeterAngle);
        m_BladeAxes[i].RotateZ(O,-m_YawAngle);
        m_BladeAxes[i].RotateX(O,m_PlatformRollAngleX);
        m_BladeAxes[i].RotateY(O,m_PlatformPitchAngleY);
        m_BladeAxes[i].RotateZ(O,m_PlatformYawAngleZ);

        if (m_bUpWind) m_BladeOrigins[i].RotateY(m_RotorHub,m_TeeterAngle);
        else m_BladeOrigins[i].RotateY(m_RotorHub,-m_TeeterAngle);
        m_BladeOrigins[i].RotateZ(O,-m_YawAngle);
        m_BladeOrigins[i].RotateX(O,m_PlatformRollAngleX);
        m_BladeOrigins[i].RotateY(O,m_PlatformPitchAngleY);
        m_BladeOrigins[i].RotateZ(O,m_PlatformYawAngleZ);
        m_BladeOrigins[i].Translate(m_PlatformTranslation);
    }

    if (m_bUpWind) m_RotorAxis.RotateY(O,m_TeeterAngle);
    else m_RotorAxis.RotateY(O,-m_TeeterAngle);

    m_RotorAxis.RotateZ(O,-m_YawAngle);
    m_RotorHub.RotateZ(O,-m_YawAngle);

    m_RotorAxis.RotateX(O,m_PlatformRollAngleX);
    m_RotorHub.RotateX(O,m_PlatformRollAngleX);
    m_TowerFoot.RotateX(O,m_PlatformRollAngleX);
    m_TowerTop.RotateX(O,m_PlatformRollAngleX);

    m_RotorAxis.RotateY(O,m_PlatformPitchAngleY);
    m_RotorHub.RotateY(O,m_PlatformPitchAngleY);
    m_TowerFoot.RotateY(O,m_PlatformPitchAngleY);
    m_TowerTop.RotateY(O,m_PlatformPitchAngleY);

    m_RotorAxis.RotateZ(O,m_PlatformYawAngleZ);
    m_RotorHub.RotateZ(O,m_PlatformYawAngleZ);
    m_TowerTop.RotateZ(O,m_PlatformYawAngleZ);
    m_TowerFoot.RotateZ(O,m_PlatformYawAngleZ);

    m_RotorHub.Translate(m_PlatformTranslation);
    m_TowerTop.Translate(m_PlatformTranslation);
    m_TowerFoot.Translate(m_PlatformTranslation);

    m_YawOld = -m_YawAngle;
    if (m_bUpWind) m_TeeterOld = m_TeeterAngle;
    else m_TeeterOld = -m_TeeterAngle;
    m_PlatformPitchAngleYOld = m_PlatformPitchAngleY;
    m_PlatformRollAngleXOld = m_PlatformRollAngleX;
    m_PlatformYawAngleZOld = m_PlatformYawAngleZ;
    m_PlatformTranslationOld = m_PlatformTranslation;

    m_CurrentPlatformYaw.append(m_PlatformYawAngleZ);
    m_CurrentPlatformRoll.append(m_PlatformRollAngleX);
    m_CurrentPlatformPitch.append(m_PlatformPitchAngleY);
    m_CurrentPlatformTranslationX.append(m_PlatformTranslation.x);
    m_CurrentPlatformTranslationY.append(m_PlatformTranslation.y);
    m_CurrentPlatformTranslationZ.append(m_PlatformTranslation.z);

    PitchBlades();
}

void QLLTSimulation::RotateRotor(){

    CVector O=CVector(0,0,0);

    for (int j=0;j<m_BladeNode.size();j++) m_BladeNode[j]->Rotate(m_RotorHub,m_RotorAxis,180/PI*m_Omega*dT);

    for (int i=0;i<m_BladeAxes.size();i++) {
        m_BladeAxes[i].Rotate(O,m_RotorAxis,180/PI*m_Omega*dT);
        m_BladeOrigins[i].Rotate(m_RotorHub,m_RotorAxis,180/PI*m_Omega*dT);
    }

    for (int i=0;i<m_BladePanel.size();i++)
    {
    m_BladePanel[i]->angularPos +=m_Omega*dT/PI*180;
    if (m_BladePanel[i]->angularPos > 360 || m_BladePanel[i]->angularPos == 360) m_BladePanel[i]->angularPos -= 360;
    }
}

void QLLTSimulation::UpdateRotorGeometry()
{
    // rotate the rotor geometry
    if(!m_bisVawt) UpdateHAWTCoordinates();
    if(m_bisVawt)  UpdateVAWTCoordinates();
    RotateRotor();
    UpdateBladePanels();
    m_bGlChanged = true;
}

void QLLTSimulation::InitializeBladePanelPositions(){
    // rotate the rotor geometry
    if(!m_bisVawt) UpdateHAWTCoordinates();
    if(m_bisVawt)  UpdateVAWTCoordinates();
    UpdateBladePanels();

    for (int i=0;i<m_BladePanel.size();i++){
        m_BladePanel.at(i)->Old_CtrlPt = m_BladePanel.at(i)->CtrlPt;
        m_BladePanel.at(i)->Old_CtrlPt.Rotate(m_RotorHub, m_RotorAxis,-180/PI*m_Omega*dT);
    }
    m_bGlChanged = true;
}

void QLLTSimulation::UpdateBladePanels(){

    CVector WLA,WLB,WTA,WTB,LATB,TALB, PointOnAxis, RadialVector, TangentVector;
    double Radius;

    for (int i=0;i<m_BladePanel.size();i++)
    {
        WLA.Copy(*m_BladePanel[i]->pLA);
        WLB.Copy(*m_BladePanel[i]->pLB);
        WTA.Copy(*m_BladePanel[i]->pTA);
        WTB.Copy(*m_BladePanel[i]->pTB);
        LATB.x = WTB.x - WLA.x;
        LATB.y = WTB.y - WLA.y;
        LATB.z = WTB.z - WLA.z;
        TALB.x = WLB.x - WTA.x;
        TALB.y = WLB.y - WTA.y;
        TALB.z = WLB.z - WTA.z;

        m_BladePanel[i]->Normal = LATB * TALB;
        m_BladePanel[i]->Area =  m_BladePanel[i]->Normal.VAbs()/2.0;
        m_BladePanel[i]->Normal.Normalize();

        m_BladePanel[i]->SetFrame(WLA, WLB, WTA, WTB);

        PointOnAxis = CorrespondingAxisPoint(m_BladePanel[i]->CtrlPt,m_RotorHub,m_RotorHub+m_RotorAxis);
        RadialVector = CVector(m_BladePanel[i]->CtrlPt-PointOnAxis);
        Radius = RadialVector.VAbs();
        TangentVector = m_RotorAxis * RadialVector;
        TangentVector.Normalize();
        RadialVector.Normalize();

        m_BladePanel[i]->tangentialVector = TangentVector;
        m_BladePanel[i]->radialVector = RadialVector*(-1.0);
        m_BladePanel[i]->radius = Radius;
    }

}

void QLLTSimulation::PitchBlades(){
    double DeltaPitch = m_PitchAngle - m_PitchOld;
    m_PitchOld = m_PitchAngle;
        for (int j=0;j<m_BladeNode.size();j++)
        {
            m_BladeNode[j]->Rotate(m_BladeOrigins[m_BladeNode[j]->fromBlade],m_BladeAxes[m_BladeNode[j]->fromBlade],DeltaPitch);
        }
}

void QLLTSimulation::AddNewWakeLines(){
//create the new shed wake line
    VortexNode *L = NULL, *T = NULL;
    VortexLine *line = NULL;

    if (m_bShed){
        int pos = 0;
        for (int i=m_WakeNode.size()-m_BladeLine.size();i<m_WakeNode.size();i++){

            if (((pos+1) % (m_BladeLine.size()/m_numBlades)) != 0){
                VortexLine *line = new VortexLine;
                line->fromStation = m_BladeLine[pos]->fromStation;
                line->fromTimestep = m_currentTimeStep;
                line->isShed = true;
                line->m_Lines = &m_WakeLine;
                line->m_Nodes = &m_WakeNode;
                line->m_KinViscosity = m_KinViscosity;
                line->m_TurbulentViscosity = m_TurbulentViscosity;
                line->m_VortexTimeOffset = m_VortexTimeOffset;
                L = m_WakeNode.at(i);
                T = m_WakeNode.at(i+1);
                line->pL = L;
                line->pT = T;
                L->attachLine(line);
                T->attachLine(line);
                line->Update(dT);
                m_WakeLine.push_back(line);
            }
            pos++;
        }
    }

    for (int i=0;i<m_BladeLine.size();i++)
    {
        //search for the corresponding downstream node
        for (int j=0;j<m_WakeLine.size();j++){
            if (m_bTrail && m_WakeLine.at(j)->isTrailing && (m_WakeLine.at(j)->fromTimestep == m_currentTimeStep-1) && (m_WakeLine.at(j)->fromStation == m_BladeLine[i]->fromStation)){
                 T = m_WakeLine.at(j)->pL;
            }
        }

        if (m_bTrail || m_bShed){
            //create new leading node
            L = new VortexNode;
            *L = *m_BladeLine[i]->pT;
            L->fromTimestep = m_currentTimeStep;
            L->fromStation  = m_BladeLine[i]->fromStation;
            m_WakeNode.push_back(L);
        }

        //add trailing vorticity
        if (m_bTrail){
            line = new VortexLine;
            line->pL = L;
            line->pT = T;
            line->fromStation = m_BladeLine[i]->fromStation;
            line->isHub = m_BladeLine[i]->isHub;
            line->isTip = m_BladeLine[i]->isTip;
            line->fromTimestep = m_currentTimeStep;
            line->m_KinViscosity = m_KinViscosity;
            line->m_TurbulentViscosity = m_TurbulentViscosity;
            line->m_VortexTimeOffset = m_VortexTimeOffset;
            line->isTrailing = true;
            line->m_Lines = &m_WakeLine;
            line->m_Nodes = &m_WakeNode;
            line->Update(dT);
            //connect node to line vortex
            T->attachLine(line);
            L->attachLine(line);
            m_WakeLine.push_back(line);
        }
    }
}

void QLLTSimulation::WakeCutOff(){
    int newlines = 0;
    if (m_bShed)  newlines+=m_BladeLine.size()-m_numBlades;
    if (m_bTrail) newlines+=m_BladeLine.size();
    if (m_windInputType != SIMFILE){
        for (int i=m_WakeLine.size()-newlines-1;i>=0;i--){
            if ((m_currentTimeStep - m_WakeLine.at(i)->fromTimestep) * m_AngularDisc / 360  > m_MaxWakeAge) m_WakeLine.at(i)->Delete();
        }
    }
    else{
        for (int i=m_WakeLine.size()-newlines-1;i>=0;i--){
            if ((m_currentTimeStep - m_WakeLine.at(i)->fromTimestep) > m_MaxWakeAge) m_WakeLine.at(i)->Delete();
        }
    }

    if (m_bisVawt && m_windInputType != SIMFILE){
        for (int i=m_WakeLine.size()-newlines-1;i>=0;i--){
            if ((fabs(m_WakeLine.at(i)->GetGamma() / m_maxGamma) < m_MinGammaFrac) && ((m_currentTimeStep - m_WakeLine.at(i)->fromTimestep) * m_AngularDisc / 360  > m_FullWakeRevs)) m_WakeLine.at(i)->Delete();
        }
    }

    else if (m_bisVawt){
        for (int i=m_WakeLine.size()-newlines-1;i>=0;i--){
            if ((fabs(m_WakeLine.at(i)->GetGamma() / m_maxGamma) < m_MinGammaFrac) && ((m_currentTimeStep - m_WakeLine.at(i)->fromTimestep) > m_FullWakeRevs)) m_WakeLine.at(i)->Delete();
        }
    }

}

void QLLTSimulation::WakeConcentration(){

    // here the wake is lumped into concentrated vortices after the specified number of "m_FullWakeRevs"
    int newlines = 0;
    if (m_bShed)  newlines+=m_BladeLine.size()-m_numBlades;
    if (m_bTrail) newlines+=m_BladeLine.size();
    QVector<VortexLine *> lines;
    QVector<VortexLine *> newLines;
    QList<QList <VortexNode *> > nodes;
    QList<VortexNode *> newVNodes;
    double Gamma_Max = 0;
    double age = 0;
    int fromTimeStep = 0, fromTimeStepL=0, fromTimeStepT=0;
    int num_bladelines = m_BladeLine.size() / m_numBlades;

    // here the vortexlines that have to be converted into lumped vortices are searched and appended into the lines array; the shed vorticity is removed

    for (int blade = 0;blade < m_numBlades; blade++){
        int linestart = blade * num_bladelines;
        int lineend = (blade+1) * num_bladelines;
            lines.clear();
            for (int i=m_WakeLine.size()-newlines-1;i>=0;i--){
                if (m_windInputType != SIMFILE){
                    if ((m_currentTimeStep - m_WakeLine.at(i)->fromTimestep) * m_AngularDisc / 360.0 >= m_FullWakeRevs){
                        if (m_WakeLine.at(i)->isShed) {
                            m_WakeLine.at(i)->Delete();
                        }
                        else if (!m_WakeLine.at(i)->isConcentrated && m_WakeLine.at(i)->fromStation >= linestart &&  m_WakeLine.at(i)->fromStation < lineend){
                            lines.append(m_WakeLine.at(i));
                        }
                    }
                }
                else{
                    if ((m_currentTimeStep - m_WakeLine.at(i)->fromTimestep) >= m_FullWakeRevs){
                        if (m_WakeLine.at(i)->isShed) {
                            m_WakeLine.at(i)->Delete();
                        }
                        else if (!m_WakeLine.at(i)->isConcentrated && m_WakeLine.at(i)->fromStation >= linestart &&  m_WakeLine.at(i)->fromStation < lineend){
                            lines.append(m_WakeLine.at(i));
                        }
                    }
                }
            }

            if (lines.size()){

                fromTimeStep = lines.at(0)->fromTimestep;
                fromTimeStepL = lines.at(0)->pL->fromTimestep;
                fromTimeStepT = lines.at(0)->pT->fromTimestep;
            }

            double Gamma = 0, Weight = 0;
            CVector Vec = CVector(0,0,0);
            CVector VecT = CVector(0,0,0);

            QVector<double>  VGamma, VWeight;
            QVector<CVector> VVec, VVecT;

            Gamma_Max = 0;
            for (int n=0;n< lines.size();n++){
                if (fabs(lines.at(n)->GetGamma()) > Gamma_Max) Gamma_Max = fabs(lines.at(n)->GetGamma());
            }


            // here the zero crossings of the gamma derivative are found, for every zero crossing a vortex is created later that rotates in the appropriate direction

            for (int l = 0; l<lines.size()-1;l++){
                    age = lines.at(0)->age;

            //gamma stores the total vortex strength of the vortex that will be created
            //vec and vect are used to compute a weighted position for the shedding of the vortex

                    Gamma += lines.at(l)->GetGamma();
                    Weight += fabs(lines.at(l)->GetGamma());
                    Vec += *lines.at(l)->pL*fabs(lines.at(l)->GetGamma());
                    VecT += *lines.at(l)->pT*fabs(lines.at(l)->GetGamma());

                    if (lines.at(l)->GetGamma() * lines.at(l+1)->GetGamma() < 0){

                        VGamma.append(Gamma);
                        VVec.append(Vec);
                        VVecT.append(VecT);
                        VWeight.append(Weight);

                        Gamma = 0;
                        Weight = 0;
                        Vec = CVector(0,0,0);
                        VecT = CVector(0,0,0);
                    }
                if (l==lines.size()-2){
                        Gamma += lines.at(l+1)->GetGamma();
                        Vec += *lines.at(l+1)->pL*fabs(lines.at(l+1)->GetGamma());
                        VecT += *lines.at(l+1)->pT*fabs(lines.at(l+1)->GetGamma());
                        Weight += fabs(lines.at(l+1)->GetGamma());

                        VGamma.append(Gamma);
                        VVec.append(Vec);
                        VVecT.append(VecT);
                        VWeight.append(Weight);
                }
            }

            //the array of lumped vortexes is searched for too small vortices which are then added to the nearest vortex of same orientation

            for (int m=0; m<VGamma.size();m++){
                if (VGamma.at(m)/Gamma_Max < m_MinGammaFrac){
                    int nearest = 10e10;
                    bool exists = false;
                    for (int l=0;l<VGamma.size();l++){
                        if (l!=m && fabs(VVec[l].VAbs()-VVec[m].VAbs()) < nearest && VGamma.at(m)*VGamma.at(l)>0){
                            nearest = l;
                            exists = true;
                        }
                    }
                    if (exists){
                        VGamma[nearest] += VGamma.at(m);
                        VVec[nearest] += VVec.at(m);
                        VVecT[nearest] += VVecT.at(m);
                        VWeight[nearest] += VWeight.at(m);
                        VGamma.removeAt(m);
                        VVec.removeAt(m);
                        VVecT.removeAt(m);
                        VWeight.removeAt(m);
                        m = 0;
                    }
                }
            }

            int fromStation = 0;
            if (lines.size()){
                fromStation = lines.at(0)->fromStation;
            }

            for (int m=lines.size()-1;m>=0;m--){
                lines.at(m)->Delete();
            }

            newVNodes.clear();
            for (int z = 0; z<VGamma.size();z++){

                        VortexLine *line = new VortexLine;
                        line->isConcentrated = true;
                        line->SetGamma(VGamma.at(z));
                        line->isTrailing = true;
                        line->m_Lines = &m_WakeLine;
                        line->m_Nodes = &m_WakeNode;
                        line->m_KinViscosity = m_KinViscosity;
                        line->m_TurbulentViscosity = m_TurbulentViscosity;
                        line->m_VortexTimeOffset = m_VortexTimeOffset;
                        line->age = age;
                        line->fromStation = fromStation;
                        line->fromTimestep = fromTimeStep;


                        bool found = false;
                        CVector Vec = VVec.at(z);
                        CVector VecT = VVecT.at(z);
                        Vec = Vec / fabs(VWeight.at(z));
                        VecT = VecT / fabs(VWeight.at(z));


                        if (TempConnectionNode.size()){
                                if (TempConnectionNode.at(blade).size() == VGamma.size()){
                                    if (CVector(*TempConnectionNode.at(blade).at(z)-VecT).VAbs() < m_Blade->getRotorRadius()/8){
                                        found = true;
                                        TempConnectionNode.at(blade).at(z)->attachLine(line);
                                        line->pT = TempConnectionNode.at(blade).at(z);
                                    }
                                }
                        }

                        if (!found){
                            VortexNode *node = new VortexNode;
                            *node = VecT;
                            node->fromTimestep = fromTimeStepT;
                            node->attachLine(line);
                            line->pT = node;
                            m_WakeNode.insert(m_WakeNode.begin(),node);
                        }


                        VortexNode *node = new VortexNode;
                        *node = Vec;
                        node->fromTimestep = fromTimeStepL;
                        node->attachLine(line);
                        line->pL = node;
                        m_WakeNode.insert(m_WakeNode.begin(),node);
                        newVNodes.push_back(node);

                        line->Update(dT);
                        newLines.append(line);
            }

            nodes.push_back(newVNodes);


    }

    TempConnectionNode.clear();
    for (int j=0;j<m_numBlades;j++)
        if (nodes.size()){
                TempConnectionNode.push_back(nodes.at(j));
        }

    for (int i=0; i< newLines.size();i++){
        m_WakeLine.insert(m_WakeLine.begin(),newLines.at(i));
    }

 if (newLines.size()){

//now thin out wake lines
    if (!(m_currentTimeStep % m_Thinfactor == 0)){
        VortexLine *vLine1;
        for (int i=m_WakeLine.size()-1; i>=0;i--){
            if (m_windInputType != SIMFILE){
                if (m_WakeLine.at(i)->fromTimestep == fromTimeStep-1-int(m_FineWakeRevs/m_AngularDisc*360.0)){

                  if (m_WakeLine.at(i)->pL->attachedLines.size()==2 && m_WakeLine.at(i)->pT->attachedLines.size()==2){
                      if (m_WakeLine.at(i)->pT->attachedLines[0] == m_WakeLine.at(i))  vLine1 = (VortexLine *) m_WakeLine.at(i)->pT->attachedLines[1];
                      else vLine1 = (VortexLine *) m_WakeLine.at(i)->pT->attachedLines[0];

                      m_WakeLine.at(i)->pT->detachLine(vLine1);
                      m_WakeLine.at(i)->pL->attachLine(vLine1);
                      vLine1->pL = m_WakeLine.at(i)->pL;
                      vLine1->oldLength = 0; // this prevents "artificial strain" from thinning out!
                      m_WakeLine.at(i)->Delete();
                  }
                }
            }
            else{
                if (m_WakeLine.at(i)->fromTimestep == fromTimeStep-1-int(m_FineWakeRevs)){

                  if (m_WakeLine.at(i)->pL->attachedLines.size()==2 && m_WakeLine.at(i)->pT->attachedLines.size()==2){
                      if (m_WakeLine.at(i)->pT->attachedLines[0] == m_WakeLine.at(i))  vLine1 = (VortexLine *) m_WakeLine.at(i)->pT->attachedLines[1];
                      else vLine1 = (VortexLine *) m_WakeLine.at(i)->pT->attachedLines[0];

                      m_WakeLine.at(i)->pT->detachLine(vLine1);
                      m_WakeLine.at(i)->pL->attachLine(vLine1);
                      vLine1->pL = m_WakeLine.at(i)->pL;
                      vLine1->oldLength = 0;  // this prevents "artificial strain" from thinning out!
                      m_WakeLine.at(i)->Delete();
                  }
                }
            }
        }
    }

}


}




void QLLTSimulation::ConvectWakeNodes(){


    if (!m_WakeNode.size()) return;
    //Update of all nodes in the wake due to new position calculated with m_V_induced and dT

    //temporary storage of wake nodes
    QList <VortexNode > TempWakeNode;
    QList <VortexNode > TempWakeNode2;

    CVector TempVelocities[m_WakeNode.size()-1];

    for (int i=0;i<m_WakeNode.size();i++){
        VortexNode C;
        C = *m_WakeNode.at(i);
        C.wasConvected = false;
        TempWakeNode.push_back(C);
    }

    if (m_bPredictorCorrector){

        for (int i=0;i<m_WakeNode.size();i++){
            VortexNode C;
            C = *m_WakeNode.at(i);
            C.wasConvected = false;
            TempWakeNode2.push_back(C);
        }

        // normal convection of wake panels except the newly created row
        #pragma omp parallel default (none) shared (TempWakeNode2, TempVelocities)
        {
            #pragma omp for
            for (int i=0;i<m_WakeNode.size()-m_BladeLine.size();i++){
                if(m_bWakeRollUp == true){
                    if (m_bTurbConvection){
                        TempVelocities[i] =  CalcTowerInfluence(*m_WakeNode[i],getFreeStream(*m_WakeNode[i])+BiotSavartInduction(*m_WakeNode[i], m_bPrescribed));
                        TempWakeNode2[i] +=  TempVelocities[i]*dT;
                    }
                    else {
                        TempVelocities[i] = getMeanFreeStream(*m_WakeNode[i])+BiotSavartInduction(*m_WakeNode[i], m_bPrescribed);
                        TempWakeNode2[i] += TempVelocities[i]*dT;
                    }
                }
                else{
                     if (m_bTurbConvection){
                         TempVelocities[i] = CalcTowerInfluence(*m_WakeNode[i],getFreeStream(*m_WakeNode[i]));
                         TempWakeNode2[i] += TempVelocities[i]*dT;
                     }
                     else{
                         TempVelocities[i] = getMeanFreeStream(*m_WakeNode[i]);
                         TempWakeNode2[i] += TempVelocities[i]*dT;
                     }
                }
            }
        }

        //convect new wake row at trailing edge -> the proximity to the trailing edge is a user variable: m_firstwakerowlength

        #pragma omp parallel default (none) shared (TempWakeNode2, TempVelocities)
        {
        #pragma omp for
                for (int l=m_WakeNode.size()-m_BladeLine.size();l<m_WakeNode.size();l++) // only the new row of nodes is treated here
                {
                        CVector TWN(TempWakeNode2[l].x,TempWakeNode2[l].y,TempWakeNode2[l].z);
                        TWN.Rotate(m_RotorHub, m_RotorAxis, m_Omega*dT*(1.0-m_firstWakeRowLength)*180/PI);
                        TempWakeNode2[l] = TWN;
                        if (m_bWakeRollUp){
                            TempVelocities[l] = (getMeanFreeStream(TempWakeNode2[l]) + BiotSavartInduction(TempWakeNode2[l], m_bPrescribed));
                            TempWakeNode2[l] += TempVelocities[l]*dT*m_firstWakeRowLength;
                        }
                        else{
                            TempVelocities[l] = getMeanFreeStream(TempWakeNode2[l]);
                            TempWakeNode2[l] += TempVelocities[l]*dT*m_firstWakeRowLength;
                        }
                }
        }

        #pragma omp parallel default (none) shared (TempWakeNode2)
        {
                for (int i=0;i<m_WakeNode.size();i++){
                    *m_WakeNode[i] = TempWakeNode2[i];
                }
                for (int i=0; i<m_WakeLine.size(); i++)
                {
                    m_WakeLine[i]->Update(dT);
                }
        }

        // normal convection of wake panels except the newly created row
        #pragma omp parallel default (none) shared (TempWakeNode, TempWakeNode2, TempVelocities)
        {
            #pragma omp for
            for (int i=0;i<m_WakeNode.size()-m_BladeLine.size();i++){
                if(m_bWakeRollUp == true){
                    if (m_bTurbConvection){
                        TempWakeNode[i] += CalcTowerInfluence(TempWakeNode2[i],(getFreeStream(TempWakeNode2[i])+BiotSavartInduction(TempWakeNode2[i], m_bPrescribed)))*dT*0.5;
                        TempWakeNode[i] += TempVelocities[i]*dT*0.5;
                    }
                    else {
                        TempWakeNode[i] += (getMeanFreeStream(TempWakeNode2[i])+BiotSavartInduction(TempWakeNode2[i], m_bPrescribed))*dT*0.5;
                        TempWakeNode[i] += TempVelocities[i]*dT*0.5;
                    }
                }
                else{
                     if (m_bTurbConvection){
                         TempWakeNode[i] += CalcTowerInfluence(TempWakeNode2[i],getFreeStream(TempWakeNode2[i]))*dT*0.5;
                         TempWakeNode[i] += TempVelocities[i]*dT*0.5;
                     }
                     else{
                         TempWakeNode[i] += getMeanFreeStream(TempWakeNode2[i])*dT*0.5;
                         TempWakeNode[i] += TempVelocities[i]*dT*0.5;
                     }
                }
            }
        }
        //convect new wake row at trailing edge -> the proximity to the trailing edge is a user variable: m_firstwakerowlength
        #pragma omp parallel default (none) shared (TempWakeNode, TempWakeNode2, TempVelocities)
        {
        #pragma omp for
                for (int l=m_WakeNode.size()-m_BladeLine.size();l<m_WakeNode.size();l++) // only the new row of nodes is treated here
                {
                        CVector TWN(TempWakeNode2[l].x,TempWakeNode2[l].y,TempWakeNode2[l].z);
                        TWN.Rotate(m_RotorHub, m_RotorAxis, m_Omega*dT*(1.0-m_firstWakeRowLength)*180/PI);
                        TempWakeNode2[l] = TWN;
                        if (m_bWakeRollUp){
                            TempWakeNode[l] += (getMeanFreeStream(TempWakeNode2[l]) + BiotSavartInduction(TempWakeNode2[l], m_bPrescribed))*dT*0.5;
                            TempWakeNode[l] += TempVelocities[l]*dT*m_firstWakeRowLength*0.5;
                        }
                        else{
                            TempWakeNode[l] += getMeanFreeStream(TempWakeNode2[l])*dT*0.5;
                            TempWakeNode[l] += TempVelocities[l]*dT*m_firstWakeRowLength*0.5;
                        }
                }
        }
    }

    else{
        // normal convection of wake panels except the newly created row
        #pragma omp parallel default (none) shared (TempWakeNode)
        {
            #pragma omp for
            for (int i=0;i<m_WakeNode.size()-m_BladeLine.size();i++){
                if(m_bWakeRollUp == true){
                    if (m_bTurbConvection) TempWakeNode[i] += CalcTowerInfluence(*m_WakeNode[i],(getFreeStream(*m_WakeNode[i])+BiotSavartInduction(*m_WakeNode[i], m_bPrescribed)))*dT;
                    else {
                        TempWakeNode[i] += (getMeanFreeStream(*m_WakeNode[i])+BiotSavartInduction(*m_WakeNode[i], m_bPrescribed))*dT;
                    }
                }
                else{
                     if (m_bTurbConvection) TempWakeNode[i] += CalcTowerInfluence(*m_WakeNode[i],getFreeStream(*m_WakeNode[i]))*dT;
                     else{
                         TempWakeNode[i] += getMeanFreeStream(*m_WakeNode[i])*dT;
                     }
                }
            }
        }

        //convect new wake row at trailing edge -> the proximity to the trailing edge is a user variable: m_firstwakerowlength

        #pragma omp parallel default (none) shared (TempWakeNode)
        {
        #pragma omp for
                for (int l=m_WakeNode.size()-m_BladeLine.size();l<m_WakeNode.size();l++) // only the new row of nodes is treated here
                {
                        CVector TWN(TempWakeNode[l].x,TempWakeNode[l].y,TempWakeNode[l].z);
                        TWN.Rotate(m_RotorHub, m_RotorAxis, m_Omega*dT*(1.0-m_firstWakeRowLength)*180/PI);
                        TempWakeNode[l] = TWN;
                        if (m_bWakeRollUp) TempWakeNode[l] += (getMeanFreeStream(TempWakeNode[l]) + BiotSavartInduction(TempWakeNode[l], m_bPrescribed))*dT*m_firstWakeRowLength;
                        else TempWakeNode[l] += getMeanFreeStream(TempWakeNode[l])*dT*m_firstWakeRowLength;
                }
        }
    }


   //copy Nodes back from temporary storage
   for (int i=0;i<m_WakeNode.size();i++){
       *m_WakeNode[i] = TempWakeNode.at(i);
   }

   // Update the wake lines
   for (int i=0; i<m_WakeLine.size(); i++)
   {
   m_WakeLine[i]->Update(dT);
   }


}

void QLLTSimulation::InterpolateWakeSpeeds (QList<VortexNode> tempNodes) {
    // linear velocity interpolation
    #pragma omp parallel default (none) shared (tempNodes)
    {
        #pragma omp for
        for (int i=0;i<m_WakeNode.size()-m_BladeLine.size();i++){
            if(m_bWakeRollUp == true && m_WakeNode[i]->fromTimestep % m_InterpolationFactor == 0){
                tempNodes[i].V_induced = (BiotSavartInduction(*m_WakeNode[i], m_bPrescribed));
                tempNodes[i] += (tempNodes[i].V_induced+getFreeStream(*m_WakeNode[i]))*dT;
                tempNodes[i].wasConvected = true;
            }
        }
    }

    #pragma omp parallel default (none) shared (tempNodes)
	{
	#pragma omp for
		for (int i=0;i<m_WakeNode.size()-m_BladeLine.size();i++){
			bool up = false;
			bool down = false;
            int jj = 0;
            int kk = 0;
            if (!tempNodes[i].wasConvected){
				for (int j=0;j<m_WakeNode.size()-m_BladeLine.size();j++){
                    if (m_WakeNode[j]->fromTimestep - m_WakeNode[i]->fromTimestep <= m_InterpolationFactor-1 && m_WakeNode[j]->fromTimestep - m_WakeNode[i]->fromTimestep >= 0 && tempNodes[j].wasConvected && m_WakeNode[j]->fromStation == m_WakeNode[i]->fromStation){
						up = true;
						jj = j;
						break;
					}
				}
				for (int k=0;k<m_WakeNode.size()-m_BladeLine.size();k++){
                    if (m_WakeNode[i]->fromTimestep - m_WakeNode[k]->fromTimestep <= m_InterpolationFactor-1 && m_WakeNode[i]->fromTimestep - m_WakeNode[k]->fromTimestep >= 0 && tempNodes[k].wasConvected && m_WakeNode[k]->fromStation == m_WakeNode[i]->fromStation){
						down = true;
						kk = k;
						break;
					}
				}
		
				
				if (up && down) {  // interpolation
                    tempNodes[i] += (tempNodes[jj].V_induced*(CVector(*m_WakeNode[jj]-*m_WakeNode[i]).VAbs())/(CVector(*m_WakeNode[jj]-*m_WakeNode[i]).VAbs()+CVector(*m_WakeNode[kk]-*m_WakeNode[i]).VAbs()) + tempNodes[kk].V_induced*(CVector(*m_WakeNode[kk]-*m_WakeNode[i]).VAbs())/(CVector(*m_WakeNode[jj]-*m_WakeNode[i]).VAbs()+CVector(*m_WakeNode[kk]-*m_WakeNode[i]).VAbs())+getMeanFreeStream(*m_WakeNode[i]))*dT;
				} else {  // no interpolation
                    tempNodes[i] += (getFreeStream(*m_WakeNode[i])+BiotSavartInduction(*m_WakeNode[i], m_bPrescribed))*dT;
				}
			}
		}
	}
}

void QLLTSimulation::KuttaCondition()
{
    //1)hier wird jeder neuen Wake-Reihe eine im loop neu berechnete Gamma-Verteilung vom Wing zugewiesen
    //2)die initiale Wirbellänge der vier Wirbelringelemente in jedem neuen Wake Panel berechnet
    //
    //Input: Wing: m_Gamma
    //Output: Wake: m_Gamma
        int newlines = 0;
        int IDP = 0;
        if (m_bTrail) {
            newlines += m_BladeLine.size();
            for(int ID = (m_WakeLine.size()-newlines);ID < m_WakeLine.size();ID++){
               m_WakeLine[ID]->Gamma = m_BladeLine[IDP]->Gamma;
               IDP++;
            }
        }
        if (m_bShed) {
            IDP = 0;
            newlines += m_BladeLine.size()-m_numBlades;
            for(int ID = (m_WakeLine.size()-newlines);ID < m_WakeLine.size()-(newlines-m_BladeLine.size()+m_numBlades);ID++){
               if (m_currentTimeStep==0) m_WakeLine[ID]->Gamma = -m_BladePanel[IDP]->m_Gamma;
               else m_WakeLine[ID]->Gamma = m_BladePanel[IDP]->m_Gamma_lastTimestep-m_BladePanel[IDP]->m_Gamma;
               IDP++;
            }
        }

}


VortexNode* QLLTSimulation::IsWakeNode(CVector &Pt)
{
        //
        // returns the index of a wake node if found, else returns NULL
        //


        int in;
        for (in=0; in<m_WakeNode.size(); in++)
        {
                if(Pt.IsSame(*m_WakeNode[in])) return m_WakeNode[in];
        }
        return NULL;
}

void QLLTSimulation::AddFirstWakeLines()
{

    VortexNode *L, *T;
    QVector <VortexNode*> leadingNodes, trailingNodes;

    if (!m_bShed && !m_bTrail) return;

    for (int i =0;i<m_BladeLine.size();i++){

    // create new nodes
    T = new VortexNode;
    L = new VortexNode;
    *T = *m_BladeLine[i]->pT;
    *L = *T;
    T->Rotate(m_RotorHub, m_RotorAxis,-m_Omega*dT*(m_firstWakeRowLength)*180/PI);
    *T += getMeanFreeStream(m_RotorHub)*dT*m_firstWakeRowLength;

    T->fromTimestep = -1;
    T->fromStation = m_BladeLine[i]->fromStation;
    trailingNodes.push_back(T);
    L->fromTimestep = 0;
    L->fromStation = m_BladeLine[i]->fromStation;
    leadingNodes.push_back(L);
    }

    if (m_bShed){
        for (int i=0;i<trailingNodes.size();i++){

            if (((i+1) % (m_BladeLine.size()/m_numBlades) != 0)){
                VortexLine *line = new VortexLine;
                line->fromStation = m_BladeLine[i]->fromStation;
                line->isHub = false;
                line->isTip = false;
                line->fromTimestep = m_currentTimeStep;
                line->isShed = true;
                line->m_KinViscosity = m_KinViscosity;
                line->m_TurbulentViscosity = m_TurbulentViscosity;
                line->m_VortexTimeOffset = m_VortexTimeOffset;
                line->m_Lines = &m_WakeLine;
                line->m_Nodes = &m_WakeNode;
                L = trailingNodes.at(i);
                T = trailingNodes.at(i+1);
                line->pL = L;
                line->pT = T;
                L->attachLine(line);
                T->attachLine(line);
                line->Update(dT);

                m_WakeLine.push_back(line);
            }
        }
    }

    if (m_bTrail){
        //create new trailing vortex line

        for (int i =0;i<m_BladeLine.size();i++){
            L = leadingNodes.at(i);
            T = trailingNodes.at(i);
            VortexLine *line = new VortexLine;
            line->fromStation = m_BladeLine[i]->fromStation;
            line->isHub = m_BladeLine[i]->isHub;
            line->isTip = m_BladeLine[i]->isTip;
            line->fromTimestep = m_currentTimeStep;
            line->m_KinViscosity = m_KinViscosity;
            line->m_TurbulentViscosity = m_TurbulentViscosity;
            line->m_VortexTimeOffset = m_VortexTimeOffset;
            line->isTrailing = true;
            line->m_Lines = &m_WakeLine;
            line->m_Nodes = &m_WakeNode;
            line->pL = L;
            line->pT = T;
            L->attachLine(line);
            T->attachLine(line);
            line->Update(dT);

            m_WakeLine.push_back(line);
        }
    }


    for (int i=0;i<trailingNodes.size();i++){
        m_WakeNode.push_back(trailingNodes.at(i));
    }

    for (int i=0;i<leadingNodes.size();i++){
        m_WakeNode.push_back(leadingNodes.at(i));
    }


}


void QLLTSimulation::GuessInitialVortexDistribution()
{
    //Anfangsverteilung von Gamma annehmen
    //Wing
    for (int i=0;i<m_BladePanel.size();i++)
    {
        m_BladePanel[i]->m_Gamma = 0;
        m_BladePanel[i]->m_Gamma_old = m_BladePanel[i]->m_Gamma;
    }
}

void QLLTSimulation::GammaBoundFixedPointIteration()
{
    //1)calculate panel velocities
    //2)calculate new AoA from velocities
    //3)read Cl and Cd from polar data
    //4)calculate new panel circulation
    //5)check for convergence

    iterations = 0;
    m_bAllConverged = false;

    for(int i = 0; i< m_BladePanel.size();i++){
        m_BladePanel[i]->m_Gamma_lastTimestep = m_BladePanel[i]->m_Gamma;
    }

    while ((m_bAllConverged == false) && (iterations < m_MaxIterations)){

        //1)
        CalcBladePanelVelocities();

        //2)
        CalcAoA();

        //3)
        ReadPanelLiftCoef();

        //4)
        CalcNewGamma();

        //5)
        GammaConvergenceCheck();

    iterations++;
    }

    m_Iterations.append(iterations);

        AssignGammaToWingLines();





////        qDebug() <<"PANEL:"<<i<< "radial position"<<m_Blade->getFASTRadiusAt(pos)<<"AoA"<<m_BladePanel[i]->m_AoA<<"m_BladePanel[i]->m_Gamma"<<m_BladePanel[i]->m_Gamma;
//}
//    qDebug()<<"--------------------------------------------------";

}

void QLLTSimulation::AssignGammaToWingLines(){

    int pos = 0;
    for(int i = 0; i < m_BladePanel.size();i++){
        if (m_BladePanel[i]->isHub && m_BladePanel[i]->isTip){
            m_BladeLine[pos]->Gamma = -m_BladePanel[i]->m_Gamma;
            m_BladeLine[pos]->VizGamma = -m_BladePanel[i]->m_Gamma;
            pos++;
            m_BladeLine[pos]->Gamma = m_BladePanel[i]->m_Gamma;
            m_BladeLine[pos]->VizGamma = m_BladePanel[i]->m_Gamma;
            pos++;
        }
        else if (m_BladePanel[i]->isHub){

            m_BladeLine[pos]->Gamma = -m_BladePanel[i]->m_Gamma;
            m_BladeLine[pos]->VizGamma = -m_BladePanel[i]->m_Gamma;

            pos++;

            m_BladeLine[pos]->Gamma = m_BladePanel[i]->m_Gamma - m_BladePanel[i+1]->m_Gamma;
            m_BladeLine[pos]->VizGamma = (m_BladePanel[i]->m_Gamma + m_BladePanel[i+1]->m_Gamma)/2;

            pos++;
        }
        else if (m_BladePanel[i]->isTip){
            m_BladeLine[pos]->Gamma = m_BladePanel[i]->m_Gamma;
            m_BladeLine[pos]->VizGamma = m_BladePanel[i]->m_Gamma;

            pos++;
        }
        else{
            m_BladeLine[pos]->Gamma = m_BladePanel[i]->m_Gamma - m_BladePanel[i+1]->m_Gamma;
            m_BladeLine[pos]->VizGamma = (m_BladePanel[i]->m_Gamma + m_BladePanel[i+1]->m_Gamma)/2;

            pos++;
        }
    }

    for (int i=0;i < m_BladeLine.size();i++){
        if (m_BladeLine[i]->Gamma > m_maxGamma) m_maxGamma = m_BladeLine[i]->Gamma;
    }


}


void QLLTSimulation::GammaConvergenceCheck(){
    //compares Gamma_old and Gamma at all stations
    //if somewhere the convergence condition is violated set m_bAllConverged = false

    m_bAllConverged = true;

    for(int i = 0; i< m_BladePanel.size();i++){
        m_BladePanel[i]->m_Gamma_new = m_BladePanel[i]->m_Gamma_old + m_RelaxFactor*(m_BladePanel[i]->m_Gamma - m_BladePanel[i]->m_Gamma_old);


        m_BladePanel[i]->m_DeltaGamma = (m_BladePanel[i]->m_Gamma - m_BladePanel[i]->m_Gamma_old);

        if(m_BladePanel[i]->m_DeltaGamma > m_Epsilon){
            m_bAllConverged = false;
        }
                m_BladePanel[i]->m_Gamma_old = m_BladePanel[i]->m_Gamma_new; // assign new gamma_old if not converged at panel
                m_BladePanel[i]->m_Gamma = m_BladePanel[i]->m_Gamma_new; //assign new gamma if not converged on panel //ist bei actualtimestep==0 && iterations==0  Gamma_old
    }
}



void QLLTSimulation::CalcNewGamma()
{
    // calculate with equation (van Garrel S. 11 (18)) Gamma _cal
    for (int i=0; i<m_BladePanel.size();i++){

        double Vcp_a1_skal = m_BladePanel[i]->m_V_total.dot(m_BladePanel[i]->a1);
        double Vcp_a3_skal = m_BladePanel[i]->m_V_total.dot(m_BladePanel[i]->a3);

        CVector LALB = m_BladePanel[i]->VortPtB - m_BladePanel[i]->VortPtA;

        double Vcp_dl_cross_a1_skal = CVector(m_BladePanel[i]->m_V_total*LALB).dot(m_BladePanel[i]->a1);
        double Vcp_dl_cross_a3_skal = CVector(m_BladePanel[i]->m_V_total*LALB).dot(m_BladePanel[i]->a3);

        //Calculate new gamma
        m_BladePanel[i]->m_Gamma = m_BladePanel[i]->m_CL*0.5*m_BladePanel[i]->GetArea()*(pow(Vcp_a1_skal,2)+pow(Vcp_a3_skal,2)) / sqrt(pow(Vcp_dl_cross_a1_skal,2)+pow(Vcp_dl_cross_a3_skal,2));
    }
}

void QLLTSimulation::ReadPanelLiftCoef()
{
    for(int i=0;i<m_BladePanel.size();i++){
        double angle = m_BladePanel[i]->m_AoA;
        if (m_Blade->m_bIsInverted) angle *= -1;
        double length;
        double tsr;

        if (!m_bisVawt){
            tsr = m_Omega *m_Blade->getRotorRadius() / getFreeStream(m_RotorHub).VAbs();
            length = m_BladePanel[i]->length;
        }
        else{
            tsr = m_Omega *m_Blade->m_MaxRadius / getFreeStream(m_RotorHub).VAbs();
            length = m_BladePanel[i]->heigth;
        }

        QList<double> ClCd = m_Blade->getClCdAt(length,angle,true,m_BladePanel[i]->chord*m_BladePanel[i]->m_V_inPlane.VAbs()/m_KinViscosity,m_bHimmelskamp,tsr,m_Blade->m_bisSinglePolar);

        m_BladePanel[i]->m_CL = ClCd.at(0);
        m_BladePanel[i]->m_CD = ClCd.at(1);
        m_BladePanel[i]->m_RE = ClCd.at(2);


        if (m_Blade->m_bIsInverted) m_BladePanel[i]->m_CL *= -1;

        m_BladePanel[i]->LiftVector = CVector(m_BladePanel[i]->a3*m_BladePanel[i]->m_CL);
        m_BladePanel[i]->LiftVector.RotateN(m_BladePanel[i]->a2,m_BladePanel[i]->m_AoA);
        m_BladePanel[i]->DragVector = CVector(m_BladePanel[i]->a3*m_BladePanel[i]->m_CD);
        m_BladePanel[i]->DragVector.RotateN(m_BladePanel[i]->a2,-90+m_BladePanel[i]->m_AoA);
        m_BladePanel[i]->LiftDragVector = m_BladePanel[i]->LiftVector + m_BladePanel[i]->DragVector;
    }
}

void QLLTSimulation::CalcAoA()
{
    for (int i=0; i<m_BladePanel.size();i++){

        m_BladePanel[i]->m_AoA_old = m_BladePanel[i]->m_AoA;
        m_BladePanel[i]->m_AoA = atan((m_BladePanel[i]->m_V_total.dot(m_BladePanel[i]->a3))/(m_BladePanel[i]->m_V_total.dot(m_BladePanel[i]->a1)))*180/PI;
        if (m_BladePanel[i]->m_V_total.dot(m_BladePanel[i]->a1) < 0 ) m_BladePanel[i]->m_AoA += 180;
        if (m_BladePanel[i]->m_AoA < -180.0) m_BladePanel[i]->m_AoA+=360.0;
        if (m_BladePanel[i]->m_AoA >  180.0) m_BladePanel[i]->m_AoA-=360.0;

    }
}

void QLLTSimulation::CalcBladePanelVelocities() {
	
    for(int ID = 0; ID < m_BladePanel.size(); ++ID) {
		m_BladePanel[ID]->m_V_induced.Set(0,0,0);
		m_BladePanel[ID]->m_V_total.Set(0,0,0);
        m_BladePanel[ID]->m_V_inPlane.Set(0,0,0);
		
		//calcs m_V_induced for every WingPanel in WingPanel Control Point
        m_BladePanel[ID]->m_V_induced = BiotSavartInduction(m_BladePanel[ID]->CtrlPt, false, true, ID);
		if (iterations > 0) m_BladePanel[ID]->m_V_induced += m_storeWakeInfluence[ID];
        m_BladePanel[ID]->m_V_sampled = getFreeStream(m_BladePanel[ID]->CtrlPt);
        m_BladePanel[ID]->m_V_rotational = getRelativeBladeVelocity(ID);
		
		//m_V_total for new Gamma calc
        m_BladePanel[ID]->m_V_total =  CalcTowerInfluence(m_BladePanel[ID]->CtrlPt, m_BladePanel[ID]->m_V_sampled +
                                                      m_BladePanel[ID]->m_V_induced) - m_BladePanel[ID]->m_V_rotational;

        m_BladePanel[ID]->m_V_inPlane = CVector(m_BladePanel[ID]->m_V_total.dot(m_BladePanel[ID]->a1), m_BladePanel[ID]->m_V_total.dot(m_BladePanel[ID]->a3), 0);


        m_BladePanel[ID]->m_V_tower = m_BladePanel[ID]->m_V_total - (m_BladePanel[ID]->m_V_sampled + m_BladePanel[ID]->m_V_induced - m_BladePanel[ID]->m_V_rotational);
	}
}

CVector QLLTSimulation::CorrespondingAxisPoint(CVector Point, CVector Line1, CVector Line2)
{
     CVector v = Line1 - Line2;
     CVector w = Point - Line1;

     double c1 = w.dot(v);
     double c2 = v.dot(v);
     double b = c1 / c2;

     CVector Pb = Line1 + v * b;
     return Pb;
}

CVector QLLTSimulation::getRelativeBladeVelocity(int ID){

    //calculates the rotational velocity based on then tangent of the current position so rotational velocity is insensitive against the azimuthal discretization
    //possible blade motions (pitching,yawing, deflection etc) are added via a first order backwards differential scheme
    //where the blade translation due to rotation is substracted because it was previously added via the tangent

    CVector  TangentVelocity, Motion, LastAzimuthPos;

    TangentVelocity = m_BladePanel[ID]->tangentialVector * m_Omega * m_BladePanel[ID]->radius;

    LastAzimuthPos = m_BladePanel[ID]->CtrlPt;
    LastAzimuthPos.Rotate(m_RotorHub, m_RotorAxis,-180/PI*m_Omega*dT);
    Motion = ((m_BladePanel[ID]->CtrlPt - m_BladePanel[ID]->Old_CtrlPt) - (m_BladePanel[ID]->CtrlPt - LastAzimuthPos)) /dT;

    return TangentVelocity + Motion;

}


CVector QLLTSimulation::BiotSavartInduction (CVector EvalPt, bool prescribed, bool indWing, int tempID) {

    CVector VGamma_total(0,0,0);
    VortexNode R1, R2, R3, R4;  // EvalPt-P1
	/////////////
	//Wing Panels
    /////////////
    /// \brief coreSizeSquared
    double a = 1.25643;
    double coreSizeSquared = 4*a*m_TurbulentViscosity*m_KinViscosity*(m_VortexTimeOffset); // core size for bound vortices

    for (int ID = 0; ID < m_BladePanel.size(); ++ID) {
            if (m_BladePanel[ID]->m_Gamma != 0){
            //calc vector P1234 -> EvalPt on WingPanels
            R1 = EvalPt - m_BladePanel[ID]->VortPtA;  // EvalPt-P1
            R2 = EvalPt - m_BladePanel[ID]->VortPtB;  // EvalPt-P2
            R3 = EvalPt - *m_BladePanel[ID]->pTB;  // EvalPt-P3
            R4 = EvalPt - *m_BladePanel[ID]->pTA;  // EvalPt-P4
    //        /////////////////////////////////////
    //        //calc induction of wing Panels
    //        /////////////////////////////////////
    //        //Induction Vortexelement 1-2 on WingPanels
            VGamma_total += BiotSavartKernel(R1,R2,m_BladePanel[ID]->m_Gamma,coreSizeSquared);
            //Induction Vortexelement 2-3 on WingPanels
            VGamma_total += BiotSavartKernel(R2,R3,m_BladePanel[ID]->m_Gamma,coreSizeSquared);
            if (m_bShed) {
                if (m_WakeLine.size() == 0) {
                    //Induction Vortexelement 3-4 on WingPanels
                    VGamma_total += BiotSavartKernel(R3,R4,m_BladePanel[ID]->m_Gamma,coreSizeSquared);}
            }
            //Induction Vortexelement 4-1 on WingPanels
            VGamma_total += BiotSavartKernel(R4,R1,m_BladePanel[ID]->m_Gamma,coreSizeSquared);
        }
    }

    if (m_bGround) {
		for (int ID = 0; ID < m_BladePanel.size(); ++ID) {
            if (m_BladePanel[ID]->m_Gamma != 0){
                //calc vector P1234 -> EvalPt on WingPanels
                R1 = EvalPt - CVector(m_BladePanel[ID]->VortPtA.x, m_BladePanel[ID]->VortPtA.y, -m_BladePanel[ID]->VortPtA.z);
                R2 = EvalPt - CVector(m_BladePanel[ID]->VortPtB.x, m_BladePanel[ID]->VortPtB.y, -m_BladePanel[ID]->VortPtB.z);
                R3 = EvalPt - CVector(m_BladePanel[ID]->pTB->x, m_BladePanel[ID]->pTB->y, -m_BladePanel[ID]->pTB->z);
                R4 = EvalPt - CVector(m_BladePanel[ID]->pTA->x, m_BladePanel[ID]->pTA->y, -m_BladePanel[ID]->pTA->z);
                /////////////////////////////////////
                //calc induction of wing Panels mirrored at the ground
                /////////////////////////////////////
                //Induction Vortexelement 1-2 on WingPanels
                VGamma_total += BiotSavartKernel(R1,R2,m_BladePanel[ID]->m_Gamma,coreSizeSquared);
                //Induction Vortexelement 2-3 on WingPanels
                VGamma_total += BiotSavartKernel(R2,R3,m_BladePanel[ID]->m_Gamma,coreSizeSquared);
                if (m_bShed){
                    if (m_WakeLine.size() == 0){
                        //Induction Vortexelement 3-4 on WingPanels
                        VGamma_total += BiotSavartKernel(R3,R4,m_BladePanel[ID]->m_Gamma,coreSizeSquared);}
                }
                //Induction Vortexelement 4-1 on WingPanels
                VGamma_total += BiotSavartKernel(R4,R1,m_BladePanel[ID]->m_Gamma,coreSizeSquared);
            }
        }
    }
/////////////
//Wake Panels
/////////////
    // if wake induction is computed for the wing panels this happens only during the first iteration, the induction is then stored in the m_storeWakeInfluence array for the timestep
    if (indWing && iterations > 0) return VGamma_total;

    if (indWing) {
        m_storeWakeInfluence[tempID].Set(0, 0, 0);
    }

		CVector gamma_cont;
        for(int ID=0;ID<m_WakeLine.size();ID++){
            if (m_WakeLine[ID]->Gamma != 0){
            //calc vector P1/P2 -> EvalPt on WakeLines
            R1 = EvalPt - *m_WakeLine[ID]->pL; //EvalPt-P1
            R2 = EvalPt - *m_WakeLine[ID]->pT; //EvalPt-P2
            /////////////////////////////////////
            //calc induction of wake Lines
            /////////////////////////////////////
            gamma_cont = BiotSavartKernel(R1,R2,m_WakeLine[ID]->Gamma,m_WakeLine[ID]->coreSizeSquared);
            VGamma_total += gamma_cont;
            if (indWing) m_storeWakeInfluence[tempID] += gamma_cont;
            }
        }
        if (m_bGround){
            for(int ID=0;ID<m_WakeLine.size();ID++){
                if (m_WakeLine[ID]->Gamma != 0){
                //calc vector P1/P2 -> EvalPt on WakeLines
                R1 = EvalPt - CVector(m_WakeLine[ID]->pL->x, m_WakeLine[ID]->pL->y, -m_WakeLine[ID]->pL->z);
                R2 = EvalPt - CVector(m_WakeLine[ID]->pT->x, m_WakeLine[ID]->pT->y, -m_WakeLine[ID]->pT->z);
                /////////////////////////////////////
                //calc induction of wake Lines
                /////////////////////////////////////
                gamma_cont =  BiotSavartKernel(R1,R2,-m_WakeLine[ID]->Gamma,m_WakeLine[ID]->coreSizeSquared);
                VGamma_total += gamma_cont;
                if (indWing) m_storeWakeInfluence[tempID] += gamma_cont ;
                }
            }
        }


	if (prescribed) {
		CVector WindDir;
        WindDir.Set(m_InflowSpeed,0,m_VertInflowSpeed);
        WindDir.Normalize();
        WindDir.RotZ(m_HorInflowAngle/180.0*PI);
		double mag = WindDir.dot(VGamma_total);
		VGamma_total = WindDir*mag;
	}
	
    return VGamma_total;
}

CVector QLLTSimulation::BiotSavartInductionFromSave (CVector EvalPt, int timeStep) {

    CVector VGamma_total(0,0,0);
    VortexNode R1, R2;  // EvalPt-P1
            /////////////
            //Blade Panels
            /////////////
            for(int ID=0;ID<m_savedBladeVortexLines.at(timeStep).size();ID++){
                if (m_savedBladeVortexLines[timeStep][ID].Gamma != 0){
                R1 = EvalPt - CVector(m_savedBladeVortexLines[timeStep][ID].Lx,m_savedBladeVortexLines[timeStep][ID].Ly,m_savedBladeVortexLines[timeStep][ID].Lz); //EvalPt-P1
                R2 = EvalPt - CVector(m_savedBladeVortexLines[timeStep][ID].Tx,m_savedBladeVortexLines[timeStep][ID].Ty,m_savedBladeVortexLines[timeStep][ID].Tz); //EvalPt-P2
                VGamma_total += BiotSavartKernel(R1,R2,m_savedBladeVortexLines[timeStep][ID].Gamma,m_savedBladeVortexLines[timeStep][ID].CoreSizeSquared);
                }
            }
            if (m_bGround){
            for(int ID=0;ID<m_savedBladeVortexLines.at(timeStep).size();ID++){
                if (m_savedBladeVortexLines[timeStep][ID].Gamma != 0){
                R1 = EvalPt - CVector(m_savedBladeVortexLines[timeStep][ID].Lx,m_savedBladeVortexLines[timeStep][ID].Ly,-m_savedBladeVortexLines[timeStep][ID].Lz); //EvalPt-P1
                R2 = EvalPt - CVector(m_savedBladeVortexLines[timeStep][ID].Tx,m_savedBladeVortexLines[timeStep][ID].Ty,-m_savedBladeVortexLines[timeStep][ID].Tz); //EvalPt-P2
                VGamma_total += BiotSavartKernel(R1,R2,-m_savedBladeVortexLines[timeStep][ID].Gamma,m_savedBladeVortexLines[timeStep][ID].CoreSizeSquared);
                    }
                }
            }
        /////////////
        //Wake Panels
        /////////////
        for(int ID=0;ID<m_savedWakeLines.at(timeStep).size();ID++){
            if (m_savedWakeLines[timeStep][ID].Gamma != 0){
            R1 = EvalPt - CVector(m_savedWakeLines[timeStep][ID].Lx,m_savedWakeLines[timeStep][ID].Ly,m_savedWakeLines[timeStep][ID].Lz); //EvalPt-P1
            R2 = EvalPt - CVector(m_savedWakeLines[timeStep][ID].Tx,m_savedWakeLines[timeStep][ID].Ty,m_savedWakeLines[timeStep][ID].Tz); //EvalPt-P2
            VGamma_total += BiotSavartKernel(R1,R2,m_savedWakeLines[timeStep][ID].Gamma,m_savedWakeLines[timeStep][ID].CoreSizeSquared);
            }
        }
        if (m_bGround){
        for(int ID=0;ID<m_savedWakeLines.at(timeStep).size();ID++){
            if (m_savedWakeLines[timeStep][ID].Gamma != 0){
            R1 = EvalPt - CVector(m_savedWakeLines[timeStep][ID].Lx,m_savedWakeLines[timeStep][ID].Ly,-m_savedWakeLines[timeStep][ID].Lz); //EvalPt-P1
            R2 = EvalPt - CVector(m_savedWakeLines[timeStep][ID].Tx,m_savedWakeLines[timeStep][ID].Ty,-m_savedWakeLines[timeStep][ID].Tz); //EvalPt-P2
            VGamma_total += BiotSavartKernel(R1,R2,-m_savedWakeLines[timeStep][ID].Gamma,m_savedWakeLines[timeStep][ID].CoreSizeSquared);
                }
            }
        }
    return VGamma_total;
}

CVector QLLTSimulation::BiotSavartKernel(CVector R1, CVector R2, double Gamma, double coreSizeSquared){
    double R1Vabs = R1.VAbs();
    double R2Vabs = R2.VAbs();
    double R1R2 = R1Vabs*R2Vabs;
    double R1R2_skal = R1.dot(R2);
    double R1_plus_R2 = R1.VAbs() + R2.VAbs();
    CVector R1R2_cross = R1*R2;
    if (m_z_leishman != -1){
    if (R1R2+R1R2_skal<=0.000000000001) return CVector(0,0,0);
    double h = R1R2_cross.VAbs()/fabs(R1Vabs-R2Vabs);
    double Kv = pow(h,2)/pow(pow(coreSizeSquared,2*m_z_leishman-2)+pow(h,2*m_z_leishman),1/m_z_leishman);
    return R1R2_cross*Gamma*R1_plus_R2/(4*PI*R1R2*(R1R2+R1R2_skal))*Kv;
    }
    else{
    return R1R2_cross*Gamma*R1_plus_R2/(4*PI*R1R2*(R1R2+R1R2_skal)+coreSizeSquared);
    }
}




void QLLTSimulation::createHAWTTurbineGeometry(){

        InitRotorCoordinates();

        if (!m_Blade) return;

        m_Blade->CreateLLTPanels(m_discType,m_numPanels);

        for (int i=0;i<m_numBlades;i++)
        {   //create the rotor geometry
            CBlade newBlade;
            newBlade.Duplicate(m_Blade);
            newBlade.CreateLLTPanels(m_discType,m_numPanels);
            for (int j = 0; j < newBlade.m_PanelPoints.size();j++){
                for (int k=0; k<newBlade.m_PanelPoints.at(j).size();k++){
                    if (m_bUpWind){

                        newBlade.m_PanelPoints[j][k].RotateX(CVector(0,newBlade.TPos[0],0), -m_ConeAngle);
                        newBlade.m_PanelPoints[j][k].Translate(CVector(-m_RotorOverhang,0,m_HubHeight));
                    }
                    else{
                        newBlade.m_PanelPoints[j][k].RotateX(CVector(0,newBlade.TPos[0],0), m_ConeAngle);
                        newBlade.m_PanelPoints[j][k].Translate(CVector(m_RotorOverhang,0,m_HubHeight));
                    }
                    newBlade.m_PanelPoints[j][k].RotateZ(m_RotorHub, 360/m_numBlades*i-90);
                    newBlade.m_PanelPoints[j][k].RotateY(m_RotorHub, 90);
                }
                CreateElements(newBlade.m_PanelPoints.at(j).at(0),newBlade.m_PanelPoints.at(j).at(1),newBlade.m_PanelPoints.at(j).at(2),newBlade.m_PanelPoints.at(j).at(3),i);
            }
            //create the blade coordinate system
            CVector BAxis, BOrigin;
            BAxis = CVector(0,1,0);
            BOrigin = CVector(0,newBlade.TPos[0],0);

            if (m_bUpWind){
                BAxis.RotateX(CVector(0,0,0), -m_ConeAngle);
                BOrigin.Translate(CVector(-m_RotorOverhang,0,m_HubHeight));
            }
            else{
                BAxis.RotateX(CVector(0,0,0), m_ConeAngle);
                BOrigin.Translate(CVector(m_RotorOverhang,0,m_HubHeight));
            }
            BAxis.RotateZ(CVector(0,0,0), 360/m_numBlades*i-90);
            BOrigin.RotateZ(m_RotorHub, 360/m_numBlades*i-90);
            BAxis.RotateY(CVector(0,0,0), 90);
            BOrigin.RotateY(m_RotorHub, 90);
            m_BladeAxes.append(BAxis);
            m_BladeOrigins.append(BOrigin);
        }

        InitializePanels();

        UpdateHAWTCoordinates();

        UpdateBladePanels();

        storeGeometry();

        for (int i=0;i<m_BladePanel.size();i++){
        m_BladePanel[i]->angularPos += 360 / m_numBlades * m_BladePanel[i]->bladeNumber;
        }

        m_bGlChanged = true;

        emit geomChanged();
}

void QLLTSimulation::createVAWTTurbineGeometry(){

        InitRotorCoordinates();

        if (!m_Blade) return;

        m_Blade->CreateLLTPanels(m_discType,m_numPanels);

        CVector translation(0,-(m_Blade->TPos[0]+(m_Blade->TPos[m_Blade->TPos.size()-1]-m_Blade->TPos[0])/2),0);


        for (int i=0;i<m_numBlades;i++)
        {   //create the rotor geometry
            CBlade newBlade;
            newBlade.Duplicate(m_Blade);
            newBlade.CreateLLTPanels(m_discType,m_numPanels);
            for (int j = 0; j < newBlade.m_PanelPoints.size();j++){
                for (int k=0; k<newBlade.m_PanelPoints.at(j).size();k++){
                    newBlade.m_PanelPoints[j][k].Translate(translation);
                    newBlade.m_PanelPoints[j][k].Translate(CVector(0,0,m_HubHeight));
                    newBlade.m_PanelPoints[j][k].RotateX(m_RotorHub, 90);
                    newBlade.m_PanelPoints[j][k].RotateZ(m_RotorHub, 360/m_numBlades*i);
                }
                CreateElements(newBlade.m_PanelPoints.at(j).at(0),newBlade.m_PanelPoints.at(j).at(1),newBlade.m_PanelPoints.at(j).at(2),newBlade.m_PanelPoints.at(j).at(3),i);
            }

            //create the blade coordinate system
            CVector BAxis, BOrigin;
            BAxis = CVector(0,1,0);
            BOrigin = CVector((newBlade.TOffset[0]+newBlade.TOffset[newBlade.TPos.size()-1])/2,0,0);
            BOrigin.Translate(CVector(0,0,m_HubHeight));
            BAxis.RotateX(CVector(0,0,0), 90);
            BOrigin.RotateX(m_RotorHub, 90);
            BAxis.RotateZ(CVector(0,0,0), 360/m_numBlades*i);
            BOrigin.RotateZ(m_RotorHub, 360/m_numBlades*i);
            m_BladeAxes.append(BAxis);
            m_BladeOrigins.append(BOrigin);
        }
        InitializePanels();

        UpdateVAWTCoordinates();

        UpdateBladePanels();

        storeGeometry();

        for (int i=0;i<m_BladePanel.size();i++){
        m_BladePanel[i]->angularPos += 360 / m_numBlades * m_BladePanel[i]->bladeNumber;
        m_BladePanel[i]->angularPos += 270;
        }

        m_bGlChanged = true;
        emit geomChanged();

}

void QLLTSimulation::InitializePanels()
{
        int blade = 0, pos=0;

        for (int i=0;i<m_BladePanel.size();i++){
            m_BladePanel[i]->bladeNumber = blade;
            if (i % (m_BladePanel.size()/m_numBlades) == 0){
                pos=0;
                m_BladePanel[i]->isHub = true;
            }
            if ((i+1) % (m_BladePanel.size()/m_numBlades) == 0){
                m_BladePanel[i]->isTip = true;
                blade += 1;
            }
            m_BladePanel[i]->chord = (m_Blade->TChord[pos]+m_Blade->TChord[pos+1])/2;
            if (!m_bisVawt){ m_BladePanel[i]->length = (m_Blade->TPos[pos]+m_Blade->TPos[pos+1])/2;
                m_BladePanel[i]->twistAngle = (m_Blade->TTwist[pos]+m_Blade->TTwist[pos+1])/2;
            }
            if (m_bisVawt)  {m_BladePanel[i]->length = (m_Blade->TOffset[pos]+m_Blade->TOffset[pos+1])/2;
                m_BladePanel[i]->heigth = (m_Blade->TPos[pos]+m_Blade->TPos[pos+1])/2;
                m_BladePanel[i]->twistAngle = 90 - (m_Blade->TTwist[pos]+m_Blade->TTwist[pos+1])/2;
            }
            pos++;
        }

        m_storeWakeInfluence.clear();
        m_storeWakeInfluence.resize(m_BladePanel.size());

        // initialize rotor lines for line vortices
        pos = 0;
        for (int i=0;i<m_BladePanel.size();i++){
            if (m_BladePanel[i]->isHub){
                VortexLine *line = new VortexLine;
                m_BladeLine.push_back(line);
                line->pL = m_BladePanel[i]->pLA;
                line->pT = m_BladePanel[i]->pTA;
                line->isHub = true;
                line->fromStation = pos;
                pos++;

                line = new VortexLine;
                m_BladeLine.push_back(line);
                line->pL = m_BladePanel[i]->pLB;
                line->pT = m_BladePanel[i]->pTB;
                line->fromStation = pos;
                pos++;
            }
            else{
                VortexLine *line = new VortexLine;
                m_BladeLine.push_back(line);
                line->pL = m_BladePanel[i]->pLB;
                line->pT = m_BladePanel[i]->pTB;
                if (m_BladePanel[i]->isTip) line->isTip = true;
                line->fromStation = pos;
                pos++;
            }
        }

        //copy vortex nodes

}

VortexNode *QLLTSimulation::IsNode(CVector &Pt)
{
        //
        // returns the index of a node if found, else returns NULL
        //
        int in;
        // explore in reverse order, since we have better chance of
        // finding the node close to the last point when creating elements
        //
        for (in=m_BladeNode.size()-1; in>=0; in--)
        {
                if(Pt.IsSame(*m_BladeNode[in])) return m_BladeNode[in];
        }
        return NULL;
}

void QLLTSimulation::CreateElements(CVector m_LA, CVector m_LB, CVector m_TA, CVector m_TB, int blade)
{
        // Creates the panel elements that will be used by the lifting line method
        // The panels are created from left to right on a surface
        VortexNode *n0, *n1, *n2, *n3;

        CPanel *panel;
        panel = new CPanel;

                        n0 = IsNode(m_LA);
                        n1 = IsNode(m_TA);
                        n2 = IsNode(m_LB);
                        n3 = IsNode(m_TB);

                        if(n0)
                        {
                                panel->pLA = n0;
                        }
                        else {
                                VortexNode *vec = new VortexNode;
                                *vec = m_LA;
                                panel->pLA = vec;
                                vec->fromBlade = blade;
                                m_BladeNode.push_back(vec);
                        }

                        if(n1)
                        {
                                panel->pTA = n1;
                        }
                        else {
                                VortexNode *vec = new VortexNode;
                                *vec = m_TA;
                                panel->pTA = vec;
                                vec->fromBlade = blade;
                                m_BladeNode.push_back(vec);
                        }

                        if(n2)
                        {
                                panel->pLB = n2;
                        }
                        else {
                                VortexNode *vec = new VortexNode;
                                *vec = m_LB;
                                panel->pLB = vec;
                                vec->fromBlade = blade;
                                m_BladeNode.push_back(vec);
                        }

                        if(n3)
                        {
                                panel->pTB = n3;
                        }
                        else {
                                VortexNode *vec = new VortexNode;
                                *vec = m_TB;
                                panel->pTB = vec;
                                vec->fromBlade = blade;
                                m_BladeNode.push_back(vec);
                        }


                        panel->SetFrame(m_LA, m_LB, m_TA, m_TB);

                        m_BladePanel.push_back(panel);

}



void QLLTSimulation::serialize() {
    StorableObject::serialize();
    ShowAsGraphInterface::serialize();

    if (g_serializer.m_archiveFormat >= 100027) g_serializer.readOrWriteBool(&m_bHimmelskamp);

    g_serializer.readOrWriteInt(&m_windInputType);

    bool isWind;
    if (g_serializer.isReadMode()) { g_serializer.readOrWriteBool(&isWind);}
    else { isWind = m_WindField; g_serializer.readOrWriteBool(&isWind); }
    if (isWind) g_serializer.readOrWriteStorableObject(&m_WindField);
    else m_WindField = NULL;

    g_serializer.readOrWriteStorableObject(&m_Blade);

    g_serializer.readOrWriteDoubleVector3D(&m_BladeOutputData);

    g_serializer.readOrWriteDoubleList2D(&m_SimParameters);
    g_serializer.readOrWriteStringList1D(&m_FileHeader);
    g_serializer.readOrWriteString(&m_FileName);

    g_serializer.readOrWriteDouble(&m_VortexTimeOffset);
    g_serializer.readOrWriteDouble(&m_TurbulentViscosity);
    g_serializer.readOrWriteDouble(&m_KinViscosity);
    g_serializer.readOrWriteDouble(&m_TowerHeight);
    g_serializer.readOrWriteDouble(&m_HubHeight);
    g_serializer.readOrWriteDouble(&m_Density);
    g_serializer.readOrWriteDouble(&m_RelaxFactor);
    g_serializer.readOrWriteDouble(&m_firstWakeRowLength);
    g_serializer.readOrWriteDouble(&TSR);
    g_serializer.readOrWriteDouble(&m_FullWakeRevs);
    g_serializer.readOrWriteDouble(&m_FineWakeRevs);
    g_serializer.readOrWriteDouble(&m_AngularDisc);
    g_serializer.readOrWriteDouble(&m_MaxWakeAge);
    g_serializer.readOrWriteDouble(&m_maxGamma);
    g_serializer.readOrWriteDouble(&m_InflowSpeed);
    g_serializer.readOrWriteDouble(&m_PitchAngle);
    g_serializer.readOrWriteDouble(&m_VertInflowSpeed);
    g_serializer.readOrWriteDouble(&m_HorInflowAngle);
    g_serializer.readOrWriteDouble(&m_PlatformYawAngleZ);
    g_serializer.readOrWriteDouble(&m_PlatformRollAngleX);
    g_serializer.readOrWriteDouble(&m_PlatformPitchAngleY);
    g_serializer.readOrWriteDouble(&m_TeeterAngle);
    g_serializer.readOrWriteDouble(&m_YawAngle);
    g_serializer.readOrWriteDouble(&dT);
    g_serializer.readOrWriteDouble(&m_Epsilon);
    g_serializer.readOrWriteDouble(&m_Omega);
    g_serializer.readOrWriteDouble(&m_MinGammaFrac);
    g_serializer.readOrWriteDouble(&m_z_leishman);
    g_serializer.readOrWriteDouble(&m_currentTime);
    g_serializer.readOrWriteDouble(&m_TStart);
    g_serializer.readOrWriteDouble(&m_RotorOverhang);
    g_serializer.readOrWriteDouble(&m_TowerTopRadius);
    g_serializer.readOrWriteDouble(&m_TowerBottomRadius);
    g_serializer.readOrWriteDouble(&m_TowerDrag);
    g_serializer.readOrWriteDouble(&m_ShearCoeff);
    g_serializer.readOrWriteDouble(&m_shownBladeSection);
    g_serializer.readOrWriteDouble(&m_ConeAngle);

    g_serializer.readOrWriteInt(&m_numBlades);
    g_serializer.readOrWriteInt(&m_shownTimeIndex);
    g_serializer.readOrWriteInt(&m_DecFactor);
    g_serializer.readOrWriteInt(&m_numTimesteps);
    g_serializer.readOrWriteInt(&iterations);
    g_serializer.readOrWriteInt(&m_MaxIterations);
    g_serializer.readOrWriteInt(&m_currentTimeStep);
    g_serializer.readOrWriteInt(&m_InterpolationFactor);
    g_serializer.readOrWriteInt(&m_Thinfactor);
    g_serializer.readOrWriteInt(&m_nWakeColumn);
    g_serializer.readOrWriteInt(&m_numPanels);
    g_serializer.readOrWriteInt(&m_discType);

    g_serializer.readOrWriteBool(&m_bShed);
    g_serializer.readOrWriteBool(&m_bGround);
    g_serializer.readOrWriteBool(&m_bTrail);
    g_serializer.readOrWriteBool(&m_bisStandStill);
    g_serializer.readOrWriteBool(&m_bIncludeTower);
    g_serializer.readOrWriteBool(&m_bPrescribed);
    g_serializer.readOrWriteBool(&m_bTurbConvection);
    g_serializer.readOrWriteBool(&m_bStoreWake);
    g_serializer.readOrWriteBool(&m_bWakeRollUp);
    g_serializer.readOrWriteBool(&m_bisVawt);
    g_serializer.readOrWriteBool(&m_bVortexParticle);
    g_serializer.readOrWriteBool(&m_bPredictorCorrector);
    g_serializer.readOrWriteBool(&m_bUpWind);

    g_serializer.readOrWriteDoubleVector1D(&m_CurrentPlatformYaw);
    g_serializer.readOrWriteDoubleVector1D(&m_CurrentPlatformRoll);
    g_serializer.readOrWriteDoubleVector1D(&m_CurrentPlatformPitch);
    g_serializer.readOrWriteDoubleVector1D(&m_CurrentPlatformTranslationX);
    g_serializer.readOrWriteDoubleVector1D(&m_CurrentPlatformTranslationY);
    g_serializer.readOrWriteDoubleVector1D(&m_CurrentPlatformTranslationZ);
    g_serializer.readOrWriteDoubleVector1D(&m_Iterations);
    g_serializer.readOrWriteDoubleVector1D(&m_RootBendingMomentIP);
    g_serializer.readOrWriteDoubleVector1D(&m_RootBendingMomentOOP);
    g_serializer.readOrWriteDoubleVector1D(&m_CurrentVerticalInflowSpeed);
    g_serializer.readOrWriteDoubleVector1D(&m_CurrentHorizontalInflowAngle);
    g_serializer.readOrWriteDoubleVector1D(&m_CurrentYawAngle);
    g_serializer.readOrWriteDoubleVector1D(&m_CurrentPitchAngle);
    g_serializer.readOrWriteDoubleVector1D(&m_CurrentHHWindSpeed);
    g_serializer.readOrWriteDoubleVector1D(&m_ThrustZ);
    g_serializer.readOrWriteDoubleVector1D(&m_ThrustY);
    g_serializer.readOrWriteDoubleVector1D(&m_ThrustX);
    g_serializer.readOrWriteDoubleVector1D(&m_rotSpeed);
    g_serializer.readOrWriteDoubleVector1D(&m_angPos);
    g_serializer.readOrWriteDoubleVector1D(&m_Time);
    g_serializer.readOrWriteDoubleVector1D(&m_Torque);
    g_serializer.readOrWriteDoubleVector1D(&m_Power);
    g_serializer.readOrWriteDoubleVector1D(&m_TSR);
    g_serializer.readOrWriteDoubleVector1D(&m_CM);
    g_serializer.readOrWriteDoubleVector1D(&m_CT);
    g_serializer.readOrWriteDoubleVector1D(&m_CP);
    g_serializer.readOrWriteDoubleVector1D(&m_CPAV);

    m_PlatformTranslation.serialize();

    // serialize the visualization and post processing arrays
    if (g_serializer.isReadMode()) {
        int n = g_serializer.readInt();
        for (int i = 0; i < n; ++i) {
            QList<DummyLine> list;
            int m = g_serializer.readInt();
            for (int j = 0; j < m; ++j) {
                DummyLine line;
                line.serialize();
                list.append(line);
            }
            m_savedWakeLines.append(list);
        }
    } else {
        g_serializer.writeInt(m_savedWakeLines.size());
        for (int i = 0; i < m_savedWakeLines.size(); ++i) {
            g_serializer.writeInt(m_savedWakeLines.at(i).size());
            for (int j = 0; j < m_savedWakeLines.at(i).size(); ++j) {
                m_savedWakeLines[i][j].serialize();
            }
        }
    }

    if (g_serializer.isReadMode()) {
        int n = g_serializer.readInt();
        for (int i = 0; i < n; ++i) {
            QList<double> list;
            int m = g_serializer.readInt();
            for (int j = 0; j < m; ++j) {
                double line;
                list.append(line);
            }
            m_savedWakeVortexParticles.append(list);
        }
    } else {
        g_serializer.writeInt(m_savedWakeVortexParticles.size());
        for (int i = 0; i < m_savedWakeVortexParticles.size(); ++i) {
            g_serializer.writeInt(m_savedWakeVortexParticles.at(i).size());
            for (int j = 0; j < m_savedWakeVortexParticles.at(i).size(); ++j) {
                g_serializer.readOrWriteDouble(&m_savedWakeVortexParticles[i][j]);
            }
        }
    }

    if (g_serializer.isReadMode()) {
        int n = g_serializer.readInt();
        for (int i = 0; i < n; ++i) {
            QList<DummyLine> list;
            int m = g_serializer.readInt();
            for (int j = 0; j < m; ++j) {
                DummyLine line;
                line.serialize();
                list.append(line);
            }
            m_savedBladeVizLines.append(list);
        }
    } else {
        g_serializer.writeInt(m_savedBladeVizLines.size());
        for (int i = 0; i < m_savedBladeVizLines.size(); ++i) {
            g_serializer.writeInt(m_savedBladeVizLines.at(i).size());
            for (int j = 0; j < m_savedBladeVizLines.at(i).size(); ++j) {
                m_savedBladeVizLines[i][j].serialize();
            }
        }
    }

    if (g_serializer.isReadMode()) {
        int n = g_serializer.readInt();
        for (int i = 0; i < n; ++i) {
            QList<DummyLine> list;
            int m = g_serializer.readInt();
            for (int j = 0; j < m; ++j) {
                DummyLine line;
                line.serialize();
                list.append(line);
            }
            m_savedBladeVortexLines.append(list);
        }
    } else {
        g_serializer.writeInt(m_savedBladeVortexLines.size());
        for (int i = 0; i < m_savedBladeVortexLines.size(); ++i) {
            g_serializer.writeInt(m_savedBladeVortexLines.at(i).size());
            for (int j = 0; j < m_savedBladeVortexLines.at(i).size(); ++j) {
                m_savedBladeVortexLines[i][j].serialize();
            }
        }
    }

    if (g_serializer.isReadMode()) {
        int n = g_serializer.readInt();
        for (int i = 0; i < n; ++i) {
            QList<CVector> list;
            int m = g_serializer.readInt();
            for (int j = 0; j < m; ++j) {
                CVector vec;
                vec.serialize();
                list.append(vec);
            }
            m_savedCoordinateSystems.append(list);
        }
    } else {
        g_serializer.writeInt(m_savedCoordinateSystems.size());
        for (int i = 0; i < m_savedCoordinateSystems.size(); ++i) {
            g_serializer.writeInt(m_savedCoordinateSystems.at(i).size());
            for (int j = 0; j < m_savedCoordinateSystems.at(i).size(); ++j) {
                m_savedCoordinateSystems[i][j].serialize();
            }
        }
    }

    if (g_serializer.isReadMode()){
        if (!m_bisVawt) m_pen.setColor(findColor (&g_QLLTHAWTSimulationStore));
        else m_pen.setColor(findColor (&g_QLLTVAWTSimulationStore));
        PrepareOutputVectors();
    }
}

void QLLTSimulation::restorePointers() {
    StorableObject::restorePointers();
    if (m_WindField) g_serializer.restorePointer(reinterpret_cast<StorableObject**>(&m_WindField));
    g_serializer.restorePointer(reinterpret_cast<StorableObject**>(&m_Blade));
}


void QLLTSimulation::PrepareOutputVectors() {

    if (!m_bisVawt){
        m_results.append(&m_Time);
        m_availableVariables.append("Time [s]");
        m_results.append(&m_angPos);
        m_availableVariables.append("Azimuthal Position of Blade 1 [deg]");
        m_results.append(&m_rotSpeed);
        m_availableVariables.append("Rotational Speed [rpm]");
        m_results.append(&m_TSR);
        m_availableVariables.append("Tip Speed Ratio [-]");
        m_results.append(&m_CP);
        m_availableVariables.append("Power Coefficient [-]");
        m_results.append(&m_CT);
        m_availableVariables.append("Thrust Coefficient [-]");
        m_results.append(&m_CM);
        m_availableVariables.append("Torque Coefficient [-]");
        m_results.append(&m_Power);
        m_availableVariables.append("Power [kW]");
        m_results.append(&m_Torque);
        m_availableVariables.append("Torque [N/m]");
        m_results.append(&m_ThrustX);
        m_availableVariables.append("Thrust in X Direction [N]");
        m_results.append(&m_ThrustY);
        m_availableVariables.append("Thrust in Y Direction [N]");
        m_results.append(&m_ThrustZ);
        m_availableVariables.append("Thrust in Z Direction [N]");
        m_results.append(&m_CurrentHHWindSpeed);
        m_availableVariables.append("Wind Speed at Hub Height [m/s]");
        m_results.append(&m_CurrentPitchAngle);
        m_availableVariables.append("Pitch Angle [deg]");
        m_results.append(&m_CurrentYawAngle);
        m_availableVariables.append("Yaw Angle [deg]");
        m_results.append(&m_CurrentHorizontalInflowAngle);
        m_availableVariables.append("Horizontal Inflow Angle [deg]");
        m_results.append(&m_CurrentVerticalInflowSpeed);
        m_availableVariables.append("Vertical Inflow Angle [deg]");
        m_results.append(&m_RootBendingMomentOOP);
        m_availableVariables.append("Out Of Plane Root Bending Moment [N/m]");
        m_results.append(&m_RootBendingMomentIP);
        m_availableVariables.append("In Plane Root Bending Moment [N/m]");
        m_results.append(&m_Iterations);
        m_availableVariables.append("Iterations [-]");

        m_availableVariables.append("Lift Coefficient (at section) [-]");
        m_availableVariables.append("Drag Coefficient (at section) [-]");
        m_availableVariables.append("Lift to Drag Ratio (at section) [-]");
        m_availableVariables.append("Circulation (at section) [m^2/s]");
        m_availableVariables.append("Angle of Attack (at section) [deg]");
        m_availableVariables.append("Length (at section) [m]");
        m_availableVariables.append("Chord (at section) [m]");
        m_availableVariables.append("Blade Twist Angle (at section) [deg]");
        m_availableVariables.append("Normal Force Coefficient (at section) [-]");
        m_availableVariables.append("Tangential Force Coefficient (at section) [-]");
        m_availableVariables.append("Normal Force (at section) [N/m]");
        m_availableVariables.append("Tangential Force (at section) [N/m]");
        m_availableVariables.append("Dynamic Pressure (at section) [Pa]");
        m_availableVariables.append("Total Velocity (at section) [m/s]");
        m_availableVariables.append("Inflow Velocity (incl. Induction) (at section) [m/s]");
        m_availableVariables.append("Inflow Velocity (w/o induction) (at section) [m/s]");
        m_availableVariables.append("Relative Velocity from Blade Movement (at section) [m/s]");
        m_availableVariables.append("Velocity induced from Wake (at section) [m/s]");
        m_availableVariables.append("Velocity induced from Tower (at section) [m/s]");
        m_availableVariables.append("Axial Induction (at section) [-]");
        m_availableVariables.append("Tangential Induction (at section) [-]");
        m_availableVariables.append("Reynolds Number (at section) [-]");
        m_availableVariables.append("Delta Re Blade - Re Polar (at section) [-]");



        m_availableBladeVariables.clear();
        m_availableBladeVariables.append("Lift Coefficient [-]");
        m_availableBladeVariables.append("Drag Coefficient [-]");
        m_availableBladeVariables.append("Lift to Drag Ratio [-]");
        m_availableBladeVariables.append("Circulation [m^2/s]");
        m_availableBladeVariables.append("Angle of Attack [deg]");
        m_availableBladeVariables.append("Length [m]");
        m_availableBladeVariables.append("Chord [m]");
        m_availableBladeVariables.append("Blade Twist Angle [deg]");
        m_availableBladeVariables.append("Normal Force Coefficient [-]");
        m_availableBladeVariables.append("Tangential Force Coefficient [-]");
        m_availableBladeVariables.append("Normal Force [N/m]");
        m_availableBladeVariables.append("Tangential Force [N/m]");
        m_availableBladeVariables.append("Dynamic Pressure [Pa]");
        m_availableBladeVariables.append("Total Velocity [m/s]");
        m_availableBladeVariables.append("Inflow Velocity (incl. Induction) [m/s]");
        m_availableBladeVariables.append("Inflow Velocity (w/o induction) [m/s]");
        m_availableBladeVariables.append("Relative Velocity from Blade Movement [m/s]");
        m_availableBladeVariables.append("Velocity induced from Wake [m/s]");
        m_availableBladeVariables.append("Velocity induced from Tower [m/s]");
        m_availableBladeVariables.append("Axial Induction [-]");
        m_availableBladeVariables.append("Tangential Induction [-]");
        m_availableBladeVariables.append("Reynolds Number [-]");
        m_availableBladeVariables.append("Delta Re Blade - Re Polar [-]");


	}
    else if (m_bisVawt){
        m_results.append(&m_Time);
        m_availableVariables.append("Time [s]");
        m_results.append(&m_angPos);
        m_availableVariables.append("Azimuthal Position of Blade 1 [deg]");
        m_results.append(&m_rotSpeed);
        m_availableVariables.append("Rotational Speed [rpm]");
        m_results.append(&m_TSR);
        m_availableVariables.append("Tip Speed Ratio [-]");
        m_results.append(&m_CP);
        m_availableVariables.append("Momentary Power Coefficient [-]");
        m_results.append(&m_CPAV);
        m_availableVariables.append("Averaged Power Coefficient [-]");
        m_results.append(&m_CT);
        m_availableVariables.append("Thrust Coefficient [-]");
        m_results.append(&m_CM);
        m_availableVariables.append("Torque Coefficient [-]");
        m_results.append(&m_Power);
        m_availableVariables.append("Power [kW]");
        m_results.append(&m_Torque);
        m_availableVariables.append("Torque [N/m]");
        m_results.append(&m_ThrustX);
        m_availableVariables.append("Thrust in X Direction [N]");
        m_results.append(&m_ThrustY);
        m_availableVariables.append("Thrust in Y Direction [N]");
        m_results.append(&m_ThrustZ);
        m_availableVariables.append("Thrust in Z Direction [N]");
        m_results.append(&m_CurrentHHWindSpeed);
        m_availableVariables.append("Wind Speed at Hub Height [m/s]");
        m_results.append(&m_CurrentPitchAngle);
        m_availableVariables.append("Pitch Angle [deg]");
        m_results.append(&m_CurrentHorizontalInflowAngle);
        m_availableVariables.append("Horizontal Inflow Angle [deg]");
        m_results.append(&m_CurrentVerticalInflowSpeed);
        m_availableVariables.append("Vertical Inflow Angle [deg]");
        m_results.append(&m_Iterations);
        m_availableVariables.append("Iterations [-]");

        m_availableVariables.append("Lift Coefficient (at section) [-]");
        m_availableVariables.append("Drag Coefficient (at section) [-]");
        m_availableVariables.append("Lift to Drag Ratio (at section) [-]");
        m_availableVariables.append("Circulation [m^2/s]");
        m_availableVariables.append("Angle of Attack (at section) [deg]");
        m_availableVariables.append("Height (at section) [m]");
        m_availableVariables.append("Chord (at section) [m]");
        m_availableVariables.append("Blade Twist Angle (at section) [deg]");
        m_availableVariables.append("Normal Force Coefficient (at section) [-]");
        m_availableVariables.append("Tangential Force Coefficient (at section) [-]");
        m_availableVariables.append("Normal Force (at section) [N/m]");
        m_availableVariables.append("Tangential Force (at section) [N/m]");
        m_availableVariables.append("Dynamic Pressure (at section) [Pa]");
        m_availableVariables.append("Total Velocity (at section) [m/s]");
        m_availableVariables.append("Inflow Velocity (incl. Induction) (at section) [m/s]");
        m_availableVariables.append("Inflow Velocity (w/o induction) (at section) [m/s]");
        m_availableVariables.append("Relative Velocity from Blade Movement (at section) [m/s]");
        m_availableVariables.append("Velocity induced from Wake (at section) [m/s]");
        m_availableVariables.append("Velocity induced from Tower (at section) [m/s]");
        m_availableVariables.append("Axial Induction a (at section) [-]");
        m_availableVariables.append("Interference Factor (1-a) (at section) [-]");
        m_availableVariables.append("Reynolds Number (at section) [-]");
        m_availableVariables.append("Delta Re Blade - Re Polar (at section) [-]");


        m_availableBladeVariables.clear();
        m_availableBladeVariables.append("Lift Coefficient [-]");
        m_availableBladeVariables.append("Drag Coefficient [-]");
        m_availableBladeVariables.append("Lift to Drag Ratio [-]");
        m_availableBladeVariables.append("Circulation [m^2/s]");
        m_availableBladeVariables.append("Angle of Attack [deg]");
        m_availableBladeVariables.append("Height [m]");
        m_availableBladeVariables.append("Chord [m]");
        m_availableBladeVariables.append("Blade Twist Angle [deg]");
        m_availableBladeVariables.append("Normal Force Coefficient [-]");
        m_availableBladeVariables.append("Tangential Force Coefficient [-]");
        m_availableBladeVariables.append("Normal Force [N/m]");
        m_availableBladeVariables.append("Tangential Force [N/m]");
        m_availableBladeVariables.append("Dynamic Pressure [Pa]");
        m_availableBladeVariables.append("Total Velocity [m/s]");
        m_availableBladeVariables.append("Inflow Velocity (incl. Induction) [m/s]");
        m_availableBladeVariables.append("Inflow Velocity (w/o induction) [m/s]");
        m_availableBladeVariables.append("Relative Velocity from Blade Movement [m/s]");
        m_availableBladeVariables.append("Velocity induced from Wake [m/s]");
        m_availableBladeVariables.append("Velocity induced from Tower [m/s]");
        m_availableBladeVariables.append("Axial Induction a [-]");
        m_availableBladeVariables.append("Interference Factor (1-a) [-]");
        m_availableBladeVariables.append("Reynolds Number [-]");
        m_availableBladeVariables.append("Delta Re Blade - Re Polar (at section) [-]");
    }    
}

double QLLTSimulation::BladeOutputAtSection(QVector<double> output, double section){
    //radius at (6)
    if (!m_BladeOutputData.size()) return -1;
    QVector<double> positions = m_BladeOutputData.at(0).at(5);
    double radius = m_Blade->getRotorRadius()*section;
    if (radius <= positions.at(0)) return output.at(0);
    else if (radius >= positions.at(positions.size()-1)) return output.at(positions.size()-1);
    else{
        for (int i=0;i<positions.size()-1;i++){            
            if (positions.at(i) < radius && radius < positions.at(i+1)){
                return output.at(i) + (output.at(i+1)-output.at(i)) * (radius-positions.at(i))/(positions.at(i+1)-positions.at(i));
            }
        }
    }
    return -1;
}


NewCurve* QLLTSimulation::newCurve(QString xAxis, QString yAxis, NewGraph::GraphType graphType) {
	
    if (!m_BladeOutputData.size()) return NULL;
    if (!m_results.size()) return NULL;

    switch (graphType) {
        case NewGraph::QLLTTimeGraph:
        {
        const int xAxisIndex = m_availableVariables.indexOf(xAxis);
        const int yAxisIndex = m_availableVariables.indexOf(yAxis);
        const bool xAxisIsBladeData = xAxisIndex >= m_results.size();
        const bool yAxisIsBladeData = yAxisIndex >= m_results.size();
            if (xAxisIndex == -1 || yAxisIndex == -1) {
                return NULL;
            }
            else if (!xAxisIsBladeData && !yAxisIsBladeData) {
                NewCurve *curve = new NewCurve (this);
                curve->setAllPoints(m_results[xAxisIndex]->data(),
                                    m_results[yAxisIndex]->data(),
                                    m_results[0]->size());  // numberOfRows is the same for all results
                return curve;
            }
            else if (xAxisIsBladeData && yAxisIsBladeData) {
                NewCurve *curve = new NewCurve (this);
                QVector<double> xOut, yOut;
                for (int m=0;m<m_BladeOutputData.size();m++){
                    if (m_BladeOutputData.at(m).size() > xAxisIndex-m_results.size()) xOut.append(BladeOutputAtSection(m_BladeOutputData.at(m).at(xAxisIndex-m_results.size()), m_shownBladeSection));
                    if (m_BladeOutputData.at(m).size() > yAxisIndex-m_results.size()) yOut.append(BladeOutputAtSection(m_BladeOutputData.at(m).at(yAxisIndex-m_results.size()), m_shownBladeSection));
                }
                curve->setAllPoints(xOut.data(), yOut.data(), m_BladeOutputData.size());  // numberOfRows is the same for all results
                return curve;
            }
            else if (xAxisIsBladeData && !yAxisIsBladeData) {
                NewCurve *curve = new NewCurve (this);
                QVector<double> xOut;
                for (int m=0;m<m_BladeOutputData.size();m++){
                    if (m_BladeOutputData.at(m).size() > xAxisIndex-m_results.size()) xOut.append(BladeOutputAtSection(m_BladeOutputData.at(m).at(xAxisIndex-m_results.size()), m_shownBladeSection));
                }
                curve->setAllPoints(xOut.data(), m_results[yAxisIndex]->data(), m_BladeOutputData.size());  // numberOfRows is the same for all results
                return curve;
            }
            else if (!xAxisIsBladeData && yAxisIsBladeData) {
                NewCurve *curve = new NewCurve (this);
                QVector<double> yOut;
                for (int m=0;m<m_BladeOutputData.size();m++){
                    if (m_BladeOutputData.at(m).size() > yAxisIndex-m_results.size()) yOut.append(BladeOutputAtSection(m_BladeOutputData.at(m).at(yAxisIndex-m_results.size()), m_shownBladeSection));
                }
                curve->setAllPoints(m_results[xAxisIndex]->data(), yOut.data(), m_BladeOutputData.size());  // numberOfRows is the same for all results
                return curve;
            }
            else return NULL;
        break;
        }
        case NewGraph::QLLTBladeGraph:
        {
        if (m_shownTimeIndex == -1) return NULL;
        const int xAxisIndex = m_availableBladeVariables.indexOf(xAxis);
        const int yAxisIndex = m_availableBladeVariables.indexOf(yAxis);
            if (xAxisIndex == -1 || yAxisIndex == -1) {
                return NULL;
            }
            else{
                NewCurve *curve = new NewCurve (this);
                curve->setAllPoints(m_BladeOutputData[m_shownTimeIndex][xAxisIndex].data(),
                                    m_BladeOutputData[m_shownTimeIndex][yAxisIndex].data(),
                                    m_BladeOutputData[0][0].size());  // numberOfRows is the same for all results
                return curve;
            }
        return NULL;
        break;
        }
    }
}

QStringList QLLTSimulation::getAvailableVariables(NewGraph::GraphType graphType) {
    switch (graphType) {
    case NewGraph::QLLTTimeGraph:
        return m_availableVariables;
    case NewGraph::QLLTBladeGraph:
        return m_availableBladeVariables;
    default:
        return QStringList();
    }
}

QLLTSimulation* QLLTSimulation::newBySerialize() {
    QLLTSimulation* fastSimulation = new QLLTSimulation ();
    fastSimulation->serialize();
    return fastSimulation;
}

void QLLTSimulation::lockStores(){
    QString message = "Objects cannot be saved or removed because a LLT simulation or animation is running\n- Stop the simulation or animation first";

    g_windFieldStore.lockStore();
    g_windFieldStore.setLockMessage(message);
    g_rotorStore.lockStore();
    g_rotorStore.setLockMessage(message);
    g_360PolarStore.lockStore();
    g_360PolarStore.setLockMessage(message);
    g_verticalRotorStore.lockStore();
    g_verticalRotorStore.setLockMessage(message);
    g_foilStore.lockStore();
    g_foilStore.setLockMessage(message);
    g_polarStore.lockStore();
    g_polarStore.setLockMessage(message);
    g_QLLTVAWTSimulationStore.lockStore();
    g_QLLTVAWTSimulationStore.setLockMessage(message);
    g_QLLTHAWTSimulationStore.lockStore();
    g_QLLTHAWTSimulationStore.setLockMessage(message);
}

void QLLTSimulation::unlockStores(){
    g_windFieldStore.unlockStore();
    g_rotorStore.unlockStore();
    g_360PolarStore.unlockStore();
    g_verticalRotorStore.unlockStore();
    g_foilStore.unlockStore();
    g_polarStore.unlockStore();
    g_QLLTVAWTSimulationStore.unlockStore();
    g_QLLTHAWTSimulationStore.unlockStore();
}

void QLLTSimulation::setShownTime(double shownTime) {
    if (hasStoredData()) {

        int timeIndex = (shownTime - m_TStart) / dT;

        if (timeIndex < 0) timeIndex = 0;

        if (timeIndex > m_results.at(0)->size()-1) timeIndex = m_results.at(0)->size()-1;

        if (fabs(m_results.at(0)->at(timeIndex) - shownTime) < 10e-5) {
            m_shownTimeIndex = timeIndex;
            return;
        }

        while (m_results.at(0)->at(timeIndex) < shownTime && timeIndex < m_results[0]->size()-1) {
            timeIndex++;
            if (fabs(m_results.at(0)->at(timeIndex) - shownTime) <10e-5) {
                m_shownTimeIndex = timeIndex;
                return;
            } else if (m_results.at(0)->at(timeIndex) > shownTime) {
                m_shownTimeIndex = -1;
                return;  // searched from lower to higher value but didn't find a matching one
            }
        }

        while (m_results.at(0)->at(timeIndex) > shownTime && timeIndex > 0) {
            timeIndex--;
            if (fabs(m_results.at(0)->at(timeIndex) - shownTime) <10e-5) {
                m_shownTimeIndex = timeIndex;
                return;
            } else if (m_results.at(0)->at(timeIndex) < shownTime) {
                m_shownTimeIndex = -1;
                return;  // searched from higher to lower value but didn't find a matching one
            }
        }

        if (shownTime > m_results.at(0)->at(m_results.at(0)->size()-1)){ m_shownTimeIndex = -1; return;}
        if (shownTime < m_results.at(0)->at(0) ) { m_shownTimeIndex = -1; return;}
    }
}

